
! ================================================================================================================================
!
! MODULE       : stomate_lpj
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Main entry point for daily processes in STOMATE and LPJ (phenology, 
!! allocation, kill, turn, light, establish, crown, cover, lcchange)
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_lpj

  ! modules used:

  USE ioipsl_para
  USE xios_orchidee
  USE grid
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE structures
  USE lpj_constraints
  USE lpj_pftinout
  USE lpj_kill
  USE lpj_crown
  USE lpj_fire
  USE lpj_gap
  USE lpj_light
  USE lpj_establish
  USE lpj_cover
  USE sapiens_agriculture
  USE sapiens_lcchange
  USE sapiens_kill,     ONLY : anthropogenic_mortality
  USE sapiens_forestry
  USE sapiens_product_use
  USE stomate_prescribe
  USE stomate_phenology
  USE stomate_mark_kill
  USE stomate_kill
  USE stomate_growth_fun_all
  USE stomate_stand_structure
  USE stomate_turnover
  USE stomate_litter
  USE stomate_laieff,   ONLY : find_lai_per_level, calculate_z_level_photo,&
                               combine_lai_levels, fitting_laieff
  USE stomate_som_dynamics
  USE stomate_vmax
  USE stomate_windthrow, ONLY : wind_damage  
 
  USE function_library, ONLY: wood_to_qmdia, cc_to_lai,&
                              wood_to_qmheight, check_biomass_sync, &
                              check_surface_area, check_mass_balance, &
                              wood_to_volume

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC stomate_lpj_vegetation,stomate_lpj_clear

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : stomate_lpj_clear
!!
!>\BRIEF        Re-initialisation of variable
!!
!! DESCRIPTION  : This subroutine reinitializes variables. To be used if we want to relaunch 
!! ORCHIDEE but the routine is not used in current version.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE stomate_lpj_clear

    CALL prescribe_clear
    CALL phenology_clear
    CALL turnover_clear
    CALL som_dynamics_clear
    CALL constraints_clear
    CALL establish_clear
    CALL fire_clear
    CALL gap_clear
    CALL light_clear
    CALL pftinout_clear

  END SUBROUTINE stomate_lpj_clear


!! ================================================================================================================================
!! SUBROUTINE   : stomate_lpj
!!
!>\BRIEF        Main entry point for daily processes in STOMATE and LPJ, structures the call sequence 
!!              to the different processes such as dispersion, establishment, competition and mortality of PFT's.
!! 
!! DESCRIPTION  : This routine is the main entry point to all processes calculated on a 
!! daily time step. Is mainly devoted to call the different STOMATE and LPJ routines 
!! depending of the ok_dgvm (is dynamic veg used) and lpj_constant_mortality (is background mortality used).
!! It also prepares the cumulative 
!! fluxes or pools (e.g TOTAL_M TOTAL_BM_LITTER etc...)
!!
!! This routine makes frequent use of "weekly", "monthly" and "long term" variables. Quotion is used because
!! by default "weekly" denotes 7 days, by default "monthly" denotes 20 days and by default "Long term" denotes
!! 3 years. dtslow refers to 24 hours (1 day).
!!
!!
!! RECENT CHANGE(S) : None
!! 
!! MAIN OUTPUT VARIABLE(S): All variables related to stomate and required for LPJ dynamic vegetation mode.
!!
!! REFERENCE(S) : 
!! - Krinner, G., N. Viovy, N. de Noblet-Ducoudré, J. Ogeé, J. Polcher, P. Friedlingstein, P. Ciais, S. Sitch, 
!! and I. C. Prentice. 2005. A dynamic global vegetation model for studies of the coupled atmosphere-biosphere 
!! system. Global Biogeochemical Cycles 19:GB1015, doi:1010.1029/2003GB002199.
!! - Sitch, S., B. Smith, I. C. Prentice, A. Arneth, A. Bondeau, W. Cramer, J. O. Kaplan, S. Levis, W. Lucht, 
!! M. T. Sykes, K. Thonicke, and S. Venevsky. 2003. Evaluation of ecosystem dynamics, plant geography and 
!! terrestrial carbon cycling in the LPJ dynamic global vegetation model. Global Change Biology 9:161-185.
!!
!! FLOWCHART    : Update with existing flowchart from N Viovy (Jan 19, 2012)
!! \n
!_ ================================================================================================================================
 
  SUBROUTINE stomate_lpj_vegetation (npts, dt_days, EndOfYear, &
       neighbours, resolution, &
       herbivores, &
       tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
       litterhum_daily, soilhum_daily, &
       maxmoiavail_lastyear, minmoiavail_lastyear, &
       gdd0_lastyear, precip_lastyear, &
       moiavail_month, moiavail_week, &
       t2m_longterm, t2m_month, t2m_week, &
       tsoil_month, soilhum_month, &
       gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       turnover_longterm, gpp_daily, &
       time_hum_min, hum_min_dormance, maxfpc_lastyear, resp_maint_part, &
       PFTpresent, age, fireindex, firelitter, &
       leaf_age, leaf_frac, adapted, regenerate, &
       plant_status, when_growthinit, litter, &
       dead_leaves, som, lignin_struc, lignin_wood, &
       veget_cov_max, veget_cov, veget_cov_max_new, npp_longterm, lm_lastyearmax, &
       veget_lastlight, everywhere, need_adjacent, RIP_time, &
       rprof, npp_daily, turnover_daily, turnover_time,&
       control_moist, control_temp, som_input, &
       atm_to_bm, co2_fire, resp_hetero, resp_maint, resp_growth, &
       height, deadleaf_cover, vcmax, &
       nue,bm_to_litter, &
       prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
       flux_prod_s, flux_prod_m, flux_prod_l, carb_mass_total, &
       fpc_max, MatrixA, MatrixV, VectorB, VectorU, &
       Tseason, Tmin_spring_time, onset_date, KF, k_latosa_adapt, &
       cn_leaf_min_season, nstress_season, moiavail_growingseason, soil_n_min, &
       rue_longterm, plant_n_uptake_daily, &
       circ_class_n, circ_class_biomass, forest_managed, forest_managed_lastyear, &
       tau_eff_leaf, tau_eff_sap, tau_eff_root, &
       species_change_map, fm_change_map, lpft_replant, &
       age_stand, rotation_n, last_cut, mai, pai, previous_wood_volume, &
       mai_count, coppice_dens, &
       lab_fac, circ_class_dist, store_sum_delta_ba, &
       harvest_pool_bound, harvest_pool, harvest_type, harvest_cut, harvest_area, &
       lai_per_level, laieff_fit, h_array_out, z_array_out, max_height_store, &
       wstress_season, wstress_month, st_dist, litter_demand, &
       mass_balance_closure, light_tran_to_level_season, p_O2, bact, &
       CN_som_litter_longterm,nbp_pool_start, &
       wind_speed_daily, soil_temp_daily, harvest_5y_area)
    
  !! 0. Variable and parameter declaration

    !! 0.1 input

    INTEGER(i_std), INTENT(in)                                 :: npts                 !! Domain size (unitless)
    REAL(r_std), INTENT(in)                                    :: dt_days              !! Time step of stomate (days)
    LOGICAL, INTENT(in)                                        :: EndOfYear            !! Flag set on the last day of the year used 
                                                                                       !! to update "yearly variables". This 
                                                                                       !! variable must be .TRUE. once a year
    INTEGER(i_std), DIMENSION(npts,8), INTENT(in)              :: neighbours           !! Indices of the 8 neighbours of each grid 
                                                                                       !! point [1=N, 2=NE, 3=E, 4=SE,
                                                                                       !!  5=S, 6=SW, 7=W, 8=NW] 
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                 :: resolution           !! Resolution at each grid point (m)  
                                                                                       !! [1=E-W, 2=N-S] 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: herbivores           !! Time constant of probability of a leaf to 
                                                                                       !! be eaten by a herbivore (days) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: tsurf_daily          !! Daily surface temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: tsoil_daily          !! Daily soil temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_daily            !! Daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_min_daily        !! Daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: litterhum_daily      !! Daily litter humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: soilhum_daily        !! Daily soil humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: maxmoiavail_lastyear !! Last year's maximum moisture availability 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: minmoiavail_lastyear !! Last year's minimum moisture availability 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: gdd0_lastyear        !! Last year's GDD0 (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: precip_lastyear      !! Lastyear's precipitation 
                                                                                       !! @tex $(mm year^{-1})$ @endtex
                                                                                       !! to determine if establishment possible
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: moiavail_month       !! "Monthly" moisture availability (0 to 1, 
                                                                                       !! unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: moiavail_week        !! "Weekly" moisture availability 
                                                                                       !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_longterm         !! "Long term" 2 meter reference 
                                                                                       !! temperatures (K) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_month            !! "Monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_week             !! "Weekly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: Tseason              !! "seasonal" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: Tmin_spring_time
    REAL(r_std), DIMENSION(npts,nvm,2), INTENT(in)             :: onset_date
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: tsoil_month          !! "Monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: soilhum_month        !! "Monthly" soil humidity
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gdd_m5_dormance      !! Growing degree days (K), threshold -5 deg 
                                                                                       !! C (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gdd_from_growthinit  !! growing degree days, since growthinit for crops
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gdd_midwinter        !! Growing degree days (K), since midwinter 
                                                                                       !! (for phenology) - this is written to the history files 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: ncd_dormance         !! Number of chilling days (days), since 
                                                                                       !! leaves were lost (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)        :: turnover_longterm    !! "Long term" turnover rate
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: wind_speed_daily     !! Actual wind speed calculated from the actual half hourly
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: soil_temp_daily      !! Soil temperature at 80 cm below ground

                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gpp_daily            !! Daily gross primary productivity  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: maxfpc_lastyear      !! Last year's maximum foliage projected
                                                                                       !! coverage for each natural PFT,
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)        :: resp_maint_part      !! Maintenance respiration of different 
                                                                                       !! plant parts  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: fpc_max              !! "Maximal" coverage fraction of a PFT (LAI 
                                                                                       !! -> infinity) on ground  
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
!!$    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                :: veget_cov_max_new        !! Old timestep "maximal" coverage fraction of a PFT 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: cn_leaf_min_season   !! Seasonal min CN ratio of leaves 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: nstress_season       !! N-related seasonal stress (used for allocation)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: moiavail_growingseason !! mean growing season moisture availability 
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: tau_eff_root         !! Effective root turnover time that accounts
                                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: tau_eff_sap          !! Effective sapwood turnover time that accounts
                                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: tau_eff_leaf         !! Effective leaf turnover time that accounts
                                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(0:), INTENT(in)                     :: harvest_pool_bound   !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)                      :: st_dist              !! The probability distribution of trees 
                                                                                       !! removed from a circ class in case of  
                                                                                       !! self thinning (unitless).
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                 :: species_change_map   !! A map which gives the PFT number that each
                                                                                       !! PFT will be replanted as in case of a clearcut.
                                                                                       !! (1-nvm,unitless)
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                 :: fm_change_map        !! A map which gives the desired FM strategy when
                                                                                       !! the PFT will be replanted after a clearcut.
                                                                                       !! (1-nvm,unitless)
    REAL(r_std), DIMENSION(:,:,:),INTENT(in)                   :: light_tran_to_level_season !! Mean seasonal fraction of light transmitted
                                                                                       !! to canopy levels  
    
  !! 0.2 Output variables    

    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: npp_daily            !! Net primary productivity 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out) :: turnover_daily   !! Turnover rates            
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: co2_fire             !! Carbon emitted into the atmosphere by 
                                                                                       !! fire (living and dead biomass)  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: resp_hetero          !! Heterotrophic respiration
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: resp_maint           !! Maintenance respiration  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: resp_growth          !! Growth respiration  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: deadleaf_cover       !! Fraction of soil covered by dead leaves 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: vcmax                !! Maximum rate of carboxylation 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out):: bm_to_litter      !! Conversion of biomass to litter 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm), INTENT(out)               :: nue                  !! Nitrogen use Efficiency with impact of leaf 
                                                                                       !! age (umol CO2 (gN)-1 s-1)

    REAL(r_std), DIMENSION(:,:), INTENT(out)                   :: lab_fac              !! Activity of labile pool factor (??units??)
    REAL(r_std), DIMENSION(:,:,:), INTENT(out)                 :: lai_per_level        !! This is the LAI per vertical level
                                                                                       !! @tex $(m^{2} m^{-2})$
    REAL(r_std),DIMENSION(npts,nvm,ncirc,nlevels_tot), INTENT(out) &               
                                                               :: h_array_out          !! An output of h_array, to use in sechiba    
    REAL(r_std),DIMENSION(npts,nvm,ncirc,nlevels_tot), INTENT(out) &
                                                               :: z_array_out          !! Height above soil of the Pgap points.
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(npts, nvm), INTENT(out)             :: max_height_store     !! ???
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(out)            :: laieff_fit           !! Fitted parameters for the effective LAI
    
    
    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: time_hum_min         !! Time elapsed since strongest moisture 
                                                                                       !! availability (days) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: hum_min_dormance     !! minimum moisture during dormance 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: ngd_minus5           !! Number of growing days (days), threshold 
                                                                                       !! -5 deg C (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: height               !! Height of vegetation (m) 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)          :: control_moist        !! Moisture control of heterotrophic 
                                                                                       !! respiration (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)          :: control_temp         !! Temperature control of heterotrophic 
                                                                                       !! respiration, above and below 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(inout) :: som_input       !! Quantity of carbon going into carbon 
                                                                                       !! pools from litter decomposition  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: rprof                !! Prescribed root depth (m) 
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                :: PFTpresent           !! Tab indicating which PFTs are present in 
                                                                                       !! each pixel 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: age                  !! Age (years)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: fireindex            !! Probability of fire (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: firelitter           !! Longer term litter above the ground that 
                                                                                       !! can be burned, @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_age             !! Leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_frac            !! Fraction of leaves in leaf age class, 
                                                                                       !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: adapted              !! Adaptation of PFT (killed if too cold) 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: regenerate           !! "Fitness": Winter sufficiently cold for 
                                                                                       !! PFT regeneration ? (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: plant_status         !! Growth and phenological status of the plant
                                                                                       !! istatus = Phases defined in constantes
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: when_growthinit      !! How many days ago was the beginning of 
                                                                                       !! the growing season (days) 
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout):: litter     !! Metabolic and structural litter, above 
                                                                                       !! and below ground 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)      :: dead_leaves          !! Dead leaves on ground, per PFT, metabolic 
                                                                                       !! and structural,  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(inout)      :: som        !! Carbon pool: active, slow, or passive, 
                                                                                       !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)      :: lignin_struc         !! Ratio of Lignin/Carbon in structural 
                                                                                       !! litter, above and below ground,  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)      :: lignin_wood          !! Ratio of Lignin/Carbon in woody
                                                                                       !! litter, above and below ground, 	
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: veget_cov_max            !! "Maximal" coverage fraction of a PFT (LAI 
                                                                                       !! -> infinity) on ground
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)                :: veget_cov            !! Fractional coverage: actually share of the pixel 
                                                                                       !! covered by a PFT (fraction of ground area), 
                                                                                       !! taking into account LAI ??(= grid scale fpc)?? 
    
    
 !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: npp_longterm         !! "Long term" mean yearly primary 
                                                                                       !! productivity 
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: lm_lastyearmax       !! Last year's maximum leaf mass, for each 
                                                                                       !! PFT @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: veget_lastlight      !! Vegetation fractions (on ground) after 
                                                                                       !! last light competition  
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: everywhere           !! Is the PFT everywhere in the grid box or 
                                                                                       !! very localized (after its introduction) 
                                                                                       !! (unitless) 
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                :: need_adjacent        !! In order for this PFT to be introduced, 
                                                                                       !! does it have to be present in an 
                                                                                       !! adjacent grid box? 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: RIP_time             !! How much time ago was the PFT eliminated 
                                                                                       !! for the last time (y) 
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)     :: turnover_time        !! Turnover_time of leaves for grasses 
                                                                                       !! (days)
    REAL(r_std), DIMENSION(:,:),INTENT(inout)               :: veget_cov_max_new           !! New "maximal" coverage fraction of a PFT 
                                                                                !! (LAI -> infinity)  Includes
                                                                                !! the nobio fraction, so may sum to
                                                                                !! less than unity. (unitless)    
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: flux_prod_s           !! C-released during first years (short term) 
                                                                                !! following land cover change 
                                                                                !! @tex ($gC year^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: flux_prod_m           !! Total annual release from decomposition of 
                                                                                !! the medium-lived product pool 
                                                                                !! @tex ($gC year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: flux_prod_l           !! Total annual release from decomposition of 
                                                                                !! the long-lived product pool 
                                                                                !! @tex ($gC year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)      :: prod_s                !! Short-lived product pool after the annual 
                                                                                !! release of each compartment (short + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex    
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)      :: prod_m                !! Medium-lived product pool after the annual
                                                                                !! release of each compartment (medium + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex 
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)      :: prod_l                !! long-lived product pool after the annual 
                                                                                !! release of each compartment (long + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: flux_s                !! Annual release from the short-lived product 
                                                                                !! pool @tex ($gC) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: flux_m                !! Annual release from the medium-lived product  
                                                                                !! pool compartments @tex ($gC) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: flux_l                !! Annual release from the long-lived product
                                                                                !! pool @tex ($gC) @endtex


    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: carb_mass_total      !! Carbon Mass total (soil, litter, veg) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: KF                            !! Scaling factor to convert sapwood mass
                                                                                       !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: k_latosa_adapt                !! Leaf to sapwood area adapted for long 
                                                                                       !! term water stress (m)
    REAL(r_std), DIMENSION(npts,nvm,nionspec), INTENT(inout)      :: plant_n_uptake_daily    !! Uptake of soil N by plants  
       !! (gN/m**2/day)  
    REAL(r_std), DIMENSION(npts,nvm,nnspec), INTENT(inout)       :: soil_n_min         !! mineral nitrogen in the soil (gN/m**2)  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rue_longterm            !! Longterm radiation use efficiency
                                                                                !! (??units??) 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass      !! Biomass of the componets of the model  
                                                                                !! tree within a circumference
                                                                                !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n            !! Number of individuals in each circ class
                                                                                !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)    :: store_sum_delta_ba      !! Sum of increase of ba (m^2) which doesn't pass 
                                                                                !! through the gap clean
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: MatrixA                 !! Matrix containing the fluxes  
                                                                                !! between the carbon pools
                                                                                !! per sechiba time step 
                                                                                !! @tex $(gC.m^2.day^{-1})$ @endtex

    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                       INTENT(inout) :: VectorB                 !! Vector containing the litter increase per
                                                                                !! sechiba time step
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), &
                                       INTENT(inout) :: MatrixV                 !! Matrix containing the accumulated values of matrixA
    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                       INTENT(inout) :: VectorU                 !! Matrix containing the accumulated values of VectorB

    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)   :: forest_managed          !! forest management flag (is the forest 
                                                                                !! being managed?)
    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)   :: forest_managed_lastyear !! forest management flag for the 
                                                                                !! previous year
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: rotation_n              !! Rotation number (number of rotation 
                                                                                !! since pft is managed)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: last_cut                !! Years since last thinning (years)

    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: lpft_replant            !! Set to true if a PFT has been clearcut
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: atm_to_bm               !! C and N taken from atmosphere when 
                                                                                !! introducing a new PFT (introduced for 
                                                                                !! carbon balance closure) 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: circ_class_dist         !! The probability distribution of trees 
                                                                                !! in a circ class in case of a 
                                                                                !! redistribution (unitless).
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: harvest_pool            !! The wood and biomass that have been
                                                                                !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_type            !! Type of management that resulted
                                                                                !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_cut             !! Type of cutting that was used for the harvest
                                                                                !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_area            !! Harvested area (m^{2})
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: harvest_5y_area         !! Harvested area accumulated in previous five year (m^{2})  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: mai                     !! The mean annual increment
                                                                                !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: pai                     !! The period annual increment
                                                                                !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                !! in a stand for the previous year.
                                                                                !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(:,:),INTENT(inout)     :: mai_count               !! The number of times we've
                                                                                !! calculated the volume increment
                                                                                !! for a stand
    REAL(r_std), DIMENSION(:,:),INTENT(inout)        :: coppice_dens            !! The density of a coppice at the first
                                                                                !! cutting @tex $( 1 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_month           !! Water stress factor, based on hum_rel_daily
                                                                                !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season          !! Water stress factor, based on hum_rel_daily
                                                                                !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:),INTENT(inout)        :: mass_balance_closure    !! Missing mass in the carbon balance
                                                                                !! @tex $(gC(N) pixel^{-1} dt^{-1})$@endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: p_O2                    !! partial pressure of oxigen in the soil (hPa)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: bact                    !! denitrifier biomass (gC/m**2)
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)      :: CN_som_litter_longterm  !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), DIMENSION(:,:),INTENT(inout)        :: nbp_pool_start          !! Variable used to calculate the pool based NBP 
                                                                                !! @tex $(gC day^{-1} m^{-2})$@endtex
  
    !! 0.4 Local variables
    REAL(r_std), DIMENSION(npts,nvm)                 :: var_real                !! temporary variable used to convert an integer
                                                                                !! when writing to a history file
    REAL(r_std), DIMENSION(npts,nelements)           :: prod_s_total            !! Total products remaining in the short-lived  
                                                                                !! pool after the annual decomposition 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts,nelements)           :: prod_m_total            !! Total products remaining in the medium-lived
                                                                                !! pool after the annual decomposition 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts,nelements)           :: prod_l_total            !! Total products remaining in the long-lived
                                                                                !! pool after the annual release 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts,nelements)           :: flux_prod_total         !! Total flux from decomposition of the short,
                                                                                !! medium and long lived product pools 
                                                                                !! @tex $(gC year^{-1})$ @endtex



    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_bm_to_litter        !! Total conversion of biomass to litter 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_live_biomass        !! Total living biomass  
                                                                                !! @tex $(gC m{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: bm_alloc                !! Biomass increase, i.e. NPP per plant part 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_turnover            !! Total turnover rate  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_litter_soil_carb    !! Total soil and litter carbon  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_litter_carb         !! Total litter carbon 
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_soil_carb           !! Total soil carbon  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts)                     :: carb_mass_variation     !! Carbon Mass variation  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: cn_ind                  !! Crown area of individuals 
                                                                                !! @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: lai                     !! Leaf area index OF AN INDIVIDUAL PLANT,
                                                                                !! where a PFT contains n indentical plants
                                                                                !! i.e., using the mean individual approach 
                                                                                !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times) &
                                                     :: circ_class_kill         !! Number of trees within a circ class that needs
                                                                                !! to be killed @tex $(ind m^{-2})$ @endtex
                                                                                !! IMPORTANT: See the note in constantes.f90
                                                                                !! regarding the indicies for this variable.
    REAL(r_std), DIMENSION(npts,nvm)                 :: woodmass_ind            !! Woodmass of individuals (gC) 
    REAL(r_std), DIMENSION(npts,nvm,nparts)          :: f_alloc                 !! Fraction that goes into plant part 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                     :: avail_tree              !! Space availability for trees 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                                :: avail_grass         !! Space availability for grasses 
                                                                                       !! (0 to 1, unitless) 
    INTEGER                                                     :: j,k,l,ipts,ivm,icut,ifm,ispec 
    INTEGER                                                     :: ilev, ilit,icarb,iele, ipar, imbc, icir, iyear
!!$    REAL(r_std),DIMENSION(npts,nvm)                             :: veget_cov_max_tmp       !! "Maximal" coverage fraction of a PFT  
!!$                                                                                       !! (LAI-> infinity) on ground (unitless) 
    REAL(r_std), DIMENSION(npts,nvm)                            :: mortality           !! Fraction of individual dying this time 
                                                                                       !! step (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                                :: vartmp              !! Temporary variable used to add history
    REAL(r_std), DIMENSION(npts,nvm)                            :: histvar             !! History variables
    REAL(r_std), DIMENSION(npts,nvm)                            :: use_reserve         !! Mass taken from carbohydrate reserve 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    CHARACTER(LEN=2), DIMENSION(nelements)                      :: element_str         !! string suffix indicating element 
    REAL(r_std), DIMENSION(npts,nvm)                            :: vcmax_new   
    REAL(r_std), DIMENSION(npts,nvm)                            :: gammas              !! Slope for individual tree growth (m)
    REAL(r_std), DIMENSION(npts,nvm)                            :: sigma               !! Threshold for indivudal tree growth (m)
    REAL(r_std), DIMENSION(npts,nvm)                            :: lrake_frac          !! Relative amount of litter that raked (-)
    REAL(r_std), DIMENSION(npts,nvm)                            :: qm_height           !! Quadratic mean height of vegetation (m)
    REAL(r_std), DIMENSION(npts,nvm)                            :: qm_dia              !! Quadratic mean diameter of vegetation (m)
    REAL(r_std), DIMENSION(npts,nvm)                            :: rdi                 !! relative density index (ntrees/Nmax)         
    REAL(r_std), DIMENSION(npts,nvm)                            :: rdi_target_upper    !! relative density index (ntrees/Nmax)
    REAL(r_std), DIMENSION(npts,nvm)                            :: rdi_target_lower    !! relative density index (ntrees/Nmax)
    REAL(r_std), DIMENSION(npts,nvm,nlevels)                    :: lai_per_level_temp  !! This is the LAI per vertical level
                                                                                       !! @tex $(m^{2} m^{-2})$
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot)                :: z_level_photo       !! The height of the levels that we will
                                                                                       !! use to calculate the effective LAI for
                                                                                       !! the albedo routines and photosynthesis.
                                                                                       !! @tex $(m)$ @endtex 
    REAL(r_std), DIMENSION(npts)                                :: litter_demand       !! The amount of litter which will
                                                                                       !! be moved from forest to crop pools
                                                                                       !! at the end of the year.
                                                                                       !! @tex $( gC year^{-1} )$ @endtex
    CHARACTER(30)                                               :: var_name            !! Temporary variable name for histwrite
    REAL(r_std), DIMENSION(npts)                                :: pixel_area          !! surface area of the pixel @tex ($m^{2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                            :: N_support           !! Nitrogen which is added to the ecosystem to 
                                                                                       !! support vegetation growth (only used when 
                                                                                       !! IMPOSE_CN .EQ. true) (gC/m2/day)
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)          :: check_intern        !! Contains the components of the internal
                                                                                       !! mass balance chech for this routine
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: closure_intern      !! Check closure of internal mass balance
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: pool_start          !! Start pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: pool_end            !! End pool of this routine 
    REAL(r_std), DIMENSION(npts,nvm)                            :: temp                
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot)                :: dummy               !! dummy variable for prescribe

    ! Debug
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)           :: temp_bio            !! Biomass @tex $(gC m^{-2})$ @endtex
    CHARACTER(30)                                               :: losses              !! Loss distribution across age classes
    REAL(r_std), DIMENSION(npts,nvm,nupdates,nelements)         :: atm_to_bm_hist      !! History of atm_to_bm for C and N for the 
                                                                                       !! different subroutines where it is used recruitment
    REAL(r_std), DIMENSION(npts,nvm,nupdates)                   :: veget_cov_max_hist      !! History of veget_cov_max for C and N for the 
    integer, dimension(3)  :: pft_t


   ! Windthrow
    REAL(r_std), DIMENSION(npts,nvm,wind_years)                 :: tmp_5y_area         !! Temporary array for harvested area in previous 5years (m^{2})
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements,nfm_types,ncut_times) &
                                                                :: biomass_cut         !! Biomass change due to  ncut
    REAL(r_std), DIMENSION(npts,ncut_times)                     :: wood_volume_pix_cut !! Wood volume change due to ncut

!_ ================================================================================================================================

    IF (printlev>=2) WRITE(numout,*) 'Entering stomate_lpj'

!!$  !! 0. Calculate the LAI from the biomass. 
!!$  !! This is done to prevent passing an incorrect
!!$  !! LAI around, since LAI is diagnostic.
!!$  !! We can't do this if we use an LAI map! So this breaks DO_STOMATE=N
!!$     lai(:,ibare_sechiba) = zero
!!$     DO ipts = 1, npts
!!$        DO ivm = 1, nvm
!!$           lai(ipts,ivm) = cc_to_lai(circ_class_biomass(ipts,ivm,:,ileaf,icarbon),&
!!$                circ_class_n(ipts,ivm,:),ivm)
!!$        ENDDO
!!$     ENDDO
 
  !! 1. Initializations
    
    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('stomate_lpj')

    !! 1.2 Initialize variables to zero
    co2_fire(:,:) = zero
    npp_daily(:,:) = zero
    resp_maint(:,:) = zero
    resp_growth(:,:) = zero
    bm_to_litter(:,:,:,:) = zero
    cn_ind(:,:) = zero
    woodmass_ind(:,:) = zero
    turnover_daily(:,:,:,:) = zero
    circ_class_kill(:,:,:,:,:) = zero
    bm_alloc(:,:,:,:) = zero
    rdi(:,:) = zero
    rdi_target_upper(:,:) = zero
    rdi_target_lower(:,:) = zero
    lrake_frac(:,:) = zero
    use_reserve(:,:) = zero
    atm_to_bm_hist(:,:,:,:) = zero
    veget_cov_max_hist(:,:,:) = zero
    harvest_pool(:,:,:,:) = zero
    biomass_cut(:,:,:,:,:,:) = zero

    !! 1.3 Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)   

    !! 1.4 Initialize check for mass balance closure
    !  This test is always performed. If err_act.EQ.1 then 
    !  the value of the mass balance error -if any- is 
    !  written to the history file.

    ! Based on the dimensions of the variables it looks like 
    ! we can distinguish different PFTs but that is not true.
    ! We need to account for the wood products and those are 
    ! only defined at the pixel level. The highest resolution
    ! we can thus check for mass balance closure is the pixel.
    check_intern(:,:,:,:) = zero
    pool_start(:,:,:) = zero
    DO iele = 1,nelements

       ! atm_to_bm has as intent inout, the variable 
       ! accumulates carbon over the course of a day. 
       ! Use the difference between start and the end of 
       ! this routine
       DO ipar = 1,nparts
          DO icir=1, ncirc
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  (circ_class_biomass(:,:,icir,ipar,iele) * &
                  circ_class_n(:,:,icir) * veget_cov_max(:,:))
          ENDDO
       ENDDO

       ! The biomass harvest pool shouldn't be multiplied by veget_cov_max
       ! its units are already in gC or gN pixel-1. The total amount 
       ! for the pixel was stored in harvest_pool. Account for C and N 
       ! stored in the wood product pools. There are no longer PFTs 
       pool_start(:,1,iele) = pool_start(:,1,iele) + &
            (SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
            SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
            SUM(prod_s(:,:,iele),2) ) / pixel_area(:)

    ENDDO

    ! Account for the N-uptake calculated in stomate.f90
    pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
         (plant_n_uptake_daily(:,:,iammonium) + &
         plant_n_uptake_daily(:,:,initrate)) * veget_cov_max(:,:)

       !! DEBUG !!
       ! Debug...only use if you know what you are doing, not for general use
       IF (.FALSE.) THEN
       pft_t=(/ 13, 16, 1/)
       DO j=1, size(pft_t)
          ivm=pft_t(j)
          DO iele = 1,nelements
             WRITE(numout,*) 'veget_cov_max: ', veget_cov_max(test_grid,ivm), 'ipts:',test_grid, 'ivm:',ivm
             WRITE(numout,*) 'atm_to_bm: ', atm_to_bm_hist(test_grid,ivm,:,iele)*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'element ',iele
            WRITE(numout,*)  'biomass',SUM(SUM(circ_class_biomass(test_grid,ivm,:,:,iele),1)*circ_class_n(test_grid,ivm,:)) * &
                 veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'in bm_to_litter:',SUM(bm_to_litter(test_grid,ivm,:,iele))*veget_cov_max(test_grid,ivm)
             WRITE(numout,*)'in turnover_daily:',SUM(turnover_daily(test_grid,ivm,:,iele))*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'in litter:',SUM(SUM(litter(test_grid,:,ivm,:,iele),2))*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'in som:',SUM(som(test_grid,:,ivm,iele))*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'in harvest_pool:',SUM(harvest_pool(test_grid,ivm,:,iele),1)/pixel_area(test_grid)
             !wrong dimensions WRITE(numout,*) 'in prod_s:',prod_s(test_grid,ivm,iele)/pixel_area(test_grid)
             !wrong dimensions WRITE(numout,*) 'in prod_m:',prod_m(test_grid,ivm,iele)/pixel_area(test_grid)
             !wrong dimensions WRITE(numout,*) 'in prod_l:',prod_l(test_grid,ivm,iele)/pixel_area(test_grid)
             WRITE(numout,*) 'in gpp_daily:',gpp_daily(test_grid,ivm)*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'inresp_maint:',resp_maint(test_grid,ivm)*veget_cov_max(test_grid,ivm)
             WRITE(numout,*) 'in resp_growth:',resp_growth(test_grid,ivm)*veget_cov_max(test_grid,ivm)
             !wrong dimensions WRITE(numout,*) 'in flux_s:',flux_s(test_grid,ivm,iele)/pixel_area(test_grid)
             !wrong dimensions WRITE(numout,*) 'in flux_m:',flux_m(test_grid,ivm,iele)/pixel_area(test_grid)
             !wrong dimensions WRITE(numout,*) 'in flux_l:',flux_l(test_grid,ivm,iele)/pixel_area(test_grid)
             WRITE(numout,*) 'in plant_n_uptake_daily:',SUM(plant_n_uptake_daily(test_grid,ivm,:))
          ENDDO
        ENDDO
      ENDIF
      !!! END DEBUG !!!

    ! Initialize check for area conservation
    veget_cov_max_hist(:,:,ibeg) = veget_cov_max(:,:)
    atm_to_bm_hist(:,:,ibeg,:) = atm_to_bm(:,:,:)

       
    ! +++CHECK+++
    ! Check and document why we do this
    IF (EndOfYear) THEN
       forest_managed_lastyear(:,:) = forest_managed(:,:)
    ENDIF
    ! +++++++++++

    !! 1.6 Update stand age
    ! Here we increment the age of the stand, if it's the end of the 
    ! year. Notice that we need to do this before the forestry routines, 
    ! since the forest has had a chance to grow since the last time
    ! we called the routine.  Doing it this way may cause a small 
    ! problem since forestry is only done every year. For example, 
    ! let us say that a forest die-off occurs due to natural reasons 
    ! at the end of December. The age of that stand will be set to zero. 
    ! However, when the end of year happens the age will be incremented 
    ! by one here, even though the forest is only a couple weeks old. 
    ! There doesn't seem to be a better way to do this, though, so a 
    ! forest age of x should be taken to mean that the forest is 
    ! between x-1 and x years old.  I see this only being an issue
    ! in short rotation coppices, but those forests should never have
    ! die off because they are harvested every couple years.
    IF (EndOfYear) THEN
       DO ipts=1,npts
          DO ivm=1,nvm
             IF(veget_cov_max(ipts,ivm) .GT. min_stomate)THEN
                age_stand(ipts,ivm) = age_stand(ipts,ivm) + 1
                
                ! last_cut is similar to stand age, but it measures the
                ! time since any human intervention, either thinning or 
                ! clearcutting.
                last_cut(ipts,ivm) = last_cut(ipts,ivm) + 1
                
             ELSE
                   
                age_stand(ipts,ivm) = 0
                last_cut(ipts,ivm) = 0
                
             ENDIF
          ENDDO
       ENDDO
    ENDIF

    !! 1.7 Calculate some vegetation characteristics
    
    ! +++CHECK++++
    ! Seems useless except for the veget_cov_max issue which I don't
    ! understand.  Crown and height are now prognostic and can
    ! be calculated from biomass whenever needed. No need to
    ! call crown here.
    !! 1.7.1 Calculate some vegetation characteristics 
    !        Calculate cn_ind (individual crown mass) and individual height from
    !        state variables if running DGVM or dynamic mortality in static cover mode
    !??        Explain (maybe in the header once) why you mulitply with veget_cov_max in the DGVM
    !??        and why you don't multiply with veget_cov_max in stomate.
!!$    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort) THEN
!!$       IF(ok_dgvm) THEN
!!$          WHERE (ind(:,:).GT.min_stomate)
!!$             woodmass_ind(:,:) = &
!!$                  ((biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
!!$                  +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon)) & 
!!$                  *veget_cov_max(:,:))/ind(:,:)
!!$          ENDWHERE
!!$       ELSE
!!$          WHERE (ind(:,:).GT.min_stomate)
!!$             woodmass_ind(:,:) = &
!!$                  (biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
!!$                  +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$          ENDWHERE
!!$       ENDIF
!!$
!!$       CALL crown (npts,  PFTpresent, &
!!$            ind, biomass, woodmass_ind, &
!!$            veget_cov_max, cn_ind, height)
!!$    ENDIF

  !! 2. Prescribe vegetation characteristics if the vegetation is not dynamic
    !   At the first call this routine takes atmospheric CO2 and converts it
    !   into carbon pools to basically avoid modelling seed germination.
    !   When lpft_replant is used we ware not sure whether we want to replant
    !   with the same or another species. Replanting with another species is
    !   a lcchange and therefore when lpf_replant is used it will be dealt with
    !   at the end of the year in sapiens_lcchange. In this case replanting
    !   will only take place at the end of the year. If a PFT dies during the 
    !   year it will be left fallow until the last day of the year.
    ! +++CHECK+++
    ! Old issue, check whether the code can deal with it - I [SL] think this 
    ! should not be an issue. The model now always has density and individuals.
    !  IF the DGVM is not activated, the density of individuals and their crown
    !  areas don't matter, but they should be defined for the case we switch on
    !  the DGVM afterwards. At the first call, if the DGVM is not activated, 
    !  impose a minimum biomass for prescribed PFTs and declare them present.
    ! +++++++++++
    
    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before prescribe', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)
    !-

    dummy(:,:,:) = un
    CALL prescribe (npts, veget_cov_max, dt_days, PFTpresent, &
            everywhere, when_growthinit, leaf_frac, circ_class_dist, &  
            circ_class_n, circ_class_biomass, atm_to_bm, &
            forest_managed, KF, plant_status, age, npp_longterm, &
            lm_lastyearmax, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
            k_latosa_adapt, dummy, EndOfYear, lpft_replant, species_change_map)

    !We store intermediate veget_cov_max and atm_to_bm in order to calculate
    !the mass balance closure at the end odf the subroutine
    atm_to_bm_hist(:,:,ipre,:) = atm_to_bm(:,:,:)
    veget_cov_max_hist(:,:,ipre) =  veget_cov_max(:,:)
    ! atm_to_bm is always reset to zero  begore going to another subroutine
    atm_to_bm(:,:,:) = zero


  !! 2. Climatic constraints for PFT presence and regenerativeness

    ! Call this even when DGVM is not activated so that "adapted" 
    ! and "regenerate"
    ! are kept up to date for the moment when the DGVM is activated.
    ! +++CHECK+++
    ! Uncomment when working on the DGVM
!!$    CALL constraints (npts, dt_days, &
!!$         t2m_month, t2m_min_daily,when_growthinit, Tseason, &
!!$         adapted, regenerate)
    ! +++++++++++
    
  !! 3. Determine introduction and elimination of PTS based on climate criteria
    ! +++CHECK+++
    ! Uncomment when working on the DGVM
!!$    IF ( ok_dgvm ) THEN
!!$      
!!$       !! 3.1 Calculate introduction and elimination
!!$       CALL pftinout (npts, dt_days, adapted, regenerate, &
!!$            neighbours, veget_cov_max, &
!!$            biomass, ind, cn_ind, age, leaf_frac, npp_longterm, lm_lastyearmax, senescence, &
!!$            PFTpresent, everywhere, when_growthinit, need_adjacent, RIP_time, &
!!$            atm_to_bm, avail_tree, avail_grass)
!!$
!!$       !! 3.2 Reset attributes for eliminated PFTs.
!!$       !     This also kills PFTs that had 0 leafmass during the last year. The message
!!$       !     "... after pftinout" is misleading in this case.
!!$       CALL kill (npts, 'pftinout  ', lm_lastyearmax, &
!!$            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$            lai, age, leaf_age, leaf_frac, npp_longterm, &
!!$            when_growthinit, everywhere, veget_cov_max, bm_to_litter)
!!$       
!!$       !! 3.3 Calculate new crown area and diameter 
!!$       !      Calculate new crown area, diameter and maximum vegetation cover**[No longer used in the subroutine]
!!$       !      unsure whether this is really required
!!$       !      - in theory this could ONLY be done at the END of stomate_lpj
!!$       !      calculate woodmass of individual tree
!!$       WHERE ((ind(:,:).GT.min_stomate))
!!$          WHERE  ( veget_cov_max(:,:) .GT. min_stomate)
!!$             woodmass_ind(:,:) = &
!!$                  ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))*veget_cov_max(:,:))/ind(:,:)
!!$          ELSEWHERE
!!$             woodmass_ind(:,:) =(biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$          ENDWHERE
!!$
!!$       ENDWHERE
!!$       
!!$       ! Calculate crown area and diameter for all PFTs (including the newly established)
!!$       CALL crown (npts, PFTpresent, &
!!$            ind, biomass, woodmass_ind, &
!!$            veget_cov_max, cn_ind, height)
!!$
!!$    ENDIF
     ! +++++++++++
 
  !! 4. Phenology

    ! Forests and grasses or the so called natural vegetation in ORCHIDEE
    ! undergo a real phenology based on climatology. This means that the
    ! plants are planted the first day of the year and that they live from
    ! their reserves until the day of bud burst. Crops are planted 
    ! (and harvested) every year. We plant them the day of bud burst. The 
    ! biggest difference is that crops do not need reserves to live
    ! between January 1st and the day of bud burst.
    
    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before phenology', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)

    ! 
    CALL phenology (npts, dt_days, PFTpresent, &
         veget_cov_max, &
         t2m_longterm, t2m_month, t2m_week, gpp_daily, &
         maxmoiavail_lastyear, minmoiavail_lastyear, &
         moiavail_month, moiavail_week, &
         gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
         plant_status, time_hum_min, &
         leaf_frac, leaf_age, &
         when_growthinit, atm_to_bm, circ_class_n, &
         circ_class_biomass, KF, &
         tau_eff_leaf, tau_eff_sap, tau_eff_root, age, &
         everywhere, npp_longterm, lm_lastyearmax, k_latosa_adapt, &
         cn_leaf_min_season, plant_n_uptake_daily)

    !We store intermediate veget_cov_max and atm_to_bm in order to calculate
    !the mass balance closure at the end odf the subroutine
    atm_to_bm_hist(:,:,iphe,:) = atm_to_bm(:,:,:)
    veget_cov_max_hist(:,:,iphe) =  veget_cov_max(:,:)
    atm_to_bm(:,:,:) = zero

 !! 5. Allocate C to different plant parts

    ! Allometry based allocation and intra-stand competition (based on 
    ! Sitch et al 2003, Zaehle et al 2010 and Deleuze 2004)
    
    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before allocation', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)

    !-
    CALL growth_fun_all (npts, dt_days, veget_cov_max, veget_cov, PFTpresent, &
         plant_status, when_growthinit, t2m_week, &
         nstress_season, moiavail_growingseason, &
         gpp_daily, resp_maint_part, resp_maint, &
         resp_growth, npp_daily, bm_alloc, age, &
         leaf_age, leaf_frac, use_reserve, &
         lab_fac, rue_longterm, circ_class_n, &
         circ_class_biomass, KF, sigma, &
         gammas, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
         k_latosa_adapt, forest_managed, circ_class_dist, &
         cn_leaf_min_season, atm_to_bm, store_sum_delta_ba)     

    check_intern(:,:,iatm2land,icarbon) = gpp_daily(:,:) * veget_cov_max(:,:) * dt_days
    check_intern(:,:,iland2atm,icarbon) = &
        -un*(resp_maint(:,:) + resp_growth(:,:)) *veget_cov_max(:,:) * dt_days
 

    !We store intermediate veget_cov_max and atm_to_bm in order to calculate
    !the mass balance closure at the end odf the subroutine
    veget_cov_max_hist(:,:,igro) =  veget_cov_max(:,:)
    atm_to_bm_hist(:,:,igro,:) = atm_to_bm(:,:,:)
    atm_to_bm(:,:,:)=zero

 !! 6. Land cover change and land management

    ! We only want to do management at the end of the year. Whether 
    ! LCC takes place before land management appears to be a matter
    ! of taste but given that LCC results in the (partial) destruction 
    ! of the PFT it appears somehow logic to deal with LCC before FM. 
    ! After all, it seems unlikely that a land owner will first 
    ! carefully thin a forest and then cut it to turn it into a 
    ! grassland or cropland.
    IF ( EndOfYear) THEN
       
       ! Debug
       !printlev_loc=get_printlev('stomate_lpj')
       IF (printlev_loc>=4) CALL debug_write(npts,'before age class distr', &
            circ_class_biomass, circ_class_n, circ_class_kill, &
            plant_status, veget_cov_max)
       !- 

       !  Following allocation, the trees have grown and it needs to be
       !  checked whether they are still in the correct age class. If not,
       !  redistribute the age classes in line with a prescribe diameter
       !  threshold for each age class. This function could be applied on 
       !  daily time step but once per year is probably enough. This 
       !  routines distributes biomass within a species group so it can 
       !  be used in combination with species changes.
       CALL age_class_distr(npts, circ_class_n, circ_class_biomass, &
            veget_cov_max, circ_class_dist, wstress_season, &
            lm_lastyearmax, age, leaf_frac, atm_to_bm, &
            everywhere, litter, som, lignin_struc, &
            lignin_wood, bm_to_litter,turnover_daily, PFTpresent, when_growthinit,&
            forest_managed, KF, plant_status, &
            npp_longterm, gpp_daily, leaf_age, lab_fac, &
            gdd_from_growthinit, gdd_midwinter, time_hum_min, &
            hum_min_dormance, gdd_m5_dormance, &
            ncd_dormance, moiavail_month, moiavail_week, ngd_minus5, &
            resp_maint, resp_growth, npp_daily, &
            use_reserve, rue_longterm, mai, pai, &
            mai_count, previous_wood_volume, moiavail_growingseason,&
            matrixA, matrixV, VectorB, VectorU, age_stand, last_cut, &
            k_latosa_adapt, fm_change_map, lpft_replant, cn_leaf_min_season,&
            nstress_season, soil_n_min, p_O2, bact, store_sum_delta_ba, &
            CN_som_litter_longterm,nbp_pool_start)

      ! We store intermediate veget_cov_max and atm_to_bm in order to calculate
      ! the mass balance closure at the end odf the subroutine
      veget_cov_max_hist(:,:,iage) =  veget_cov_max(:,:)
      atm_to_bm_hist(:,:,iage,:) = atm_to_bm(:,:,:)
      atm_to_bm(:,:,:)=zero

 
      !! 5.2 Land cover change
      IF (do_now_stomate_lcchange) THEN
       
          ! When there is a loss in veget_cov_max (i.e., oaks are changed 
          ! to grasslands the) standard code will distribute this loss 
          ! proportionaly (i.e., all age classes with oak will be partly
          ! converted) to the populated age classes of the species group. 
          ! In case of a species change we only want to change the surface 
          ! area of the youngest age class (because we will only plant 
          ! trees in the youngest age class). set losses = proportional
          losses = 'proportional'
          CALL land_cover_change_main(npts, dt_days, veget_cov_max, veget_cov, &
               harvest_pool, harvest_type, harvest_cut, harvest_area, &
               litter, som, lignin_struc, lignin_wood, &
               PFTpresent, everywhere, when_growthinit, & 
               leaf_frac, circ_class_n, circ_class_biomass, &
               atm_to_bm, forest_managed, &
               KF, wstress_season, wstress_month, plant_status, npp_longterm, &
               age, lm_lastyearmax, harvest_pool_bound, resolution, &
               bm_to_litter, turnover_daily,leaf_age, circ_class_dist, &
               tau_eff_leaf, tau_eff_sap, &
               tau_eff_root, veget_cov_max_new, age_stand, last_cut, &
               k_latosa_adapt, losses, light_tran_to_level_season, &
               EndOfYear,  lpft_replant,  soil_n_min, bact, species_change_map)

          ! Set the flag done_stomate_lcchange to be used in the end of sechiba_main 
          ! to update the fractions.
          do_now_stomate_lcchange=.FALSE.
          done_stomate_lcchange=.TRUE.
          
          !We store intermediate veget_cov_max and atm_to_bm in order to
          !calculate
          !the mass balance closure at the end odf the subroutine
          atm_to_bm_hist(:,:,ilcc,:) = atm_to_bm(:,:,:)
          veget_cov_max_hist(:,:,ilcc) =  veget_cov_max(:,:)
          atm_to_bm(:,:,:) = zero
 
       ENDIF

       !! 5.3 Forest management
       !  Thin and harvest species. Flag opportunities for species
       !  changes.

       ! Debug
       !printlev_loc=get_printlev('stomate_lpj')
       IF (printlev_loc>=4) CALL debug_write(npts,'before forestry', &
            circ_class_biomass, circ_class_n, circ_class_kill, &
            plant_status, veget_cov_max)
       !- 
       CALL forestry (npts, age_stand, last_cut, &
            circ_class_n, circ_class_kill, forest_managed, rdi,&
            circ_class_biomass, mai, pai, previous_wood_volume, &
            mai_count, coppice_dens, veget_cov_max, rdi_target_upper,&
            rdi_target_lower, fm_change_map, species_change_map)

       !! 5.4 Crop harvest
       !  According to the logic of this module crop harvest should 
       !  take place here but it depends on :: turnover_daily and that
       !  variable is not calculated until the call to turnover. Also
       !  the implemented approach harvests biomass daily rather once
       !  per year (as it should be).
       
       !! 5.5 Grazing
       !  This is an excellent place to account for grazing.
       
       !! 5.6 Litter raking
       !  If you are NOT doing historical runs, litter raking almost certainly
       !  does not apply to you.
       !
       !  In order to feed animals through the winter, as well as absorb
       !  their manure to spread on the fields during the spring, farmers
       !  in historical Europe would collect litter from forests and place
       !  it in the stables during winter.  We will simulate this by
       !  looking at the litter pools at the end of every year and moving
       !  around a certain amount of litter from the forest to the crop PFTs.
       !  We do this after the forest management under the assumption that
       !  branches left on site after thinning operations would, historically,
       !  have been collected for this purpose. Of course, litter raking fell
       !  out of fashion in the mid-19th century, around the same time that
       !  scientific sylviculture became popular, so perhaps it doesn't 
       !  matter.
       IF(ok_litter_raking)THEN

          ! Calculate fluxes from litter raking
          CALL litter_raking(npts, veget_cov_max, resolution, litter_demand, &
               litter, lrake_frac)
          
       ENDIF

    ENDIF ! EndOfYear

!! 6. Fire, Wind and Pests

      !! 6.1 Call wind_damage module
       IF (ok_windthrow) THEN

       !  Calculate the critical wind speed for uprooting and breakage
       !  critical windspeeds are calculated every half-hour. A
       !  cummulated critical windspeed is then passed to stomate_lpj
       !  to calculate the actual stand damage and mortality
!       IF(ld_windfall)THEN
          WRITE(numout,*) 'Call wind_damage module for calculating critical wind speed'
          WRITE(numout,*) 'Daily maximum windspeed at pixel level:', &
                           wind_speed_daily
          WRITE(numout,*) 'Daily maximum soil temperature near 80 cm below ground  at pixel level:',&
                           soil_temp_daily

!       ENDIF

       CALL wind_damage(npts, nlevels_tot, circ_class_biomass, &
            veget_cov_max, circ_class_n, wind_speed_daily, plant_status, &
            resolution, circ_class_kill, harvest_5y_area, forest_managed, &
            soil_temp_daily)

       ENDIF
       !! 6.2 Call the fire module


 !! 7. Kill PFT's
   
    ! Kill slow growing PFTs in DGVM or STOMATE with dynamic or
    ! constant mortality. Mark trees that died from self-thinning
    ! or environmental mortality. The biomass pools are not touched.
    ! Vegetation is only marked for killing
   
    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before mark_to_kill', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)
    !- 
    CALL mark_to_kill (npts, 'growth    ', lm_lastyearmax, &
         PFTpresent, npp_longterm, circ_class_biomass, &
         circ_class_n, circ_class_kill, rue_longterm, turnover_longterm, &
         dt_days, veget_cov_max, st_dist, forest_managed)


     ! +++CHECK+++
     ! Uncomment when working on the DGVM
!!$    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort) THEN
!!$       CALL kill (npts, 'npp       ', lm_lastyearmax,  &
!!$            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$            lai, age, leaf_age, leaf_frac, npp_longterm, &
!!$            when_growthinit, everywhere, veget_cov_max, bm_to_litter)
!!$
!!$       !! 6.2.1 Update wood biomass      
!!$       !        For the DGVM
!!$       IF(ok_dgvm) THEN
!!$          WHERE (ind(:,:).GT.min_stomate)
!!$             woodmass_ind(:,:) = &
!!$                  ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon)) & 
!!$                  *veget_cov_max(:,:))/ind(:,:)
!!$          ENDWHERE
!!$
!!$       ! For all pixels with individuals
!!$       ELSE
!!$          WHERE (ind(:,:).GT.min_stomate)
!!$             woodmass_ind(:,:) = &
!!$                  (biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$          ENDWHERE
!!$       ENDIF ! ok_dgvm
!!$
!!$       !! 6.2.2 New crown area and maximum vegetation cover after growth
!!$       CALL crown (npts, PFTpresent, &
!!$            ind, biomass, woodmass_ind,&
!!$            veget_cov_max, cn_ind, height)
!!$
!!$    ENDIF ! ok_dgvm
     ! +++++++++++
    
  !! 7. fire

    !! 7.1. Burn PFTs
     ! +++CHECK+++
     ! Uncomment when working on the DGVM
!!$     CALL fire (npts, dt_days, &
!!$         litterhum_daily, t2m_daily, lignin_struc, lignin_wood, veget_cov_max, &
!!$         fireindex, firelitter, biomass, ind, &
!!$         litter, dead_leaves, bm_to_litter, &
!!$         co2_fire, matrixA)
    ! +++++++++++ 

    !! 7.2 Kill PFTs in DGVM
     ! +++CHECK+++
     ! Uncomment when working on the DGVM
!!$     IF ( ok_dgvm ) THEN
!!$
!!$        ! reset attributes for eliminated PFTs
!!$        CALL kill (npts, 'fire      ', lm_lastyearmax, &
!!$             ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$             lai, age, leaf_age, leaf_frac, npp_longterm, &
!!$             when_growthinit, everywhere, veget_cov_max, bm_to_litter)
!!$
!!$    ENDIF ! ok_dgvm
    ! +++++++++++ 

 !! 8. Tree mortality

    ! Here is where the trees actually die and biomass is moved around.  
    ! First, we kill all the trees that died because of human intervention.
    ! When we mark trees for killing the reason of their dead is also
    ! stored in circ_class_kill. This is where we make use of that 
    ! information.

    !+++CHECK+++
    ! This is called always. It is not clear whether this relates to
    ! the DGVM or not. The temperature dependencies suggest it does.
    ! check when working on the DGVM. It is not needed for when the
    ! PFTs are prescribed.
!!$    ! Does not depend on age, therefore does not change crown area.
!!$    CALL gap (npts, dt_days, &
!!$         npp_longterm, turnover_longterm, lm_lastyearmax, &
!!$         PFTpresent, t2m_min_daily, Tmin_spring_time, &
!!$         biomass, ind, bm_to_litter, mortality)
    !+++++++++++
    
    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts,'before anthrop. mortality', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)
    !-
    CALL anthropogenic_mortality (npts, &
         bm_to_litter, circ_class_biomass, circ_class_kill, &
         circ_class_n, harvest_pool_bound, harvest_pool, harvest_type, &
         harvest_cut, harvest_area, resolution, &
         veget_cov_max, age_stand, last_cut, mai_count, circ_class_dist, &
         coppice_dens, plant_status, biomass_cut)

!+++CHECK+++
    ! Code for the DGVM - in addition to the killing
!!$    IF ( ok_dgvm ) THEN
!!$       ! reset attributes for eliminated PFTs
!!$       CALL kill (npts, 'gap       ', lm_lastyearmax, &
!!$            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$            lai, age, leaf_age, leaf_frac, npp_longterm, &
!!$            when_growthinit, everywhere, veget_cov_max, bm_to_litter)
!!$    ENDIF
    !+++++++++++
       
    ! Now we kill all the plants which died due to natural causes. This comes
    ! after the plant died due to human causes, since in theory a forest
    ! that is thinned will not suffer as high of natural mortality.
    ! When we mark trees for killing the reason of their dead is also
    ! stored in circ_class_kill. This is where we make use of that information.

    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before natural mortality', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
        plant_status, veget_cov_max)
    !-
    
    CALL natural_mortality (npts, bm_to_litter, &
         circ_class_biomass, circ_class_kill, circ_class_n, &
         veget_cov_max, biomass_cut)

 !! 9. Change forestry
       
       ! PFTs have been managed and killed based on the outcome of 
       ! the operations and processes it is now decided whether we
       ! will replant the PFT possible with a different species.
       IF(ok_change_species)THEN
          CALL flag_species_change(npts, veget_cov_max, circ_class_biomass,&
            circ_class_n,lpft_replant, forest_managed)
       ENDIF
       
 !! 10. Clean the stands after mortality

    ! Now we need to do some checking. If there is no biomass on the site,
    ! some variables have to be reset. If some of the circumference classes
    ! are empty, we need to redistribute the biomass among them.  Notice this
    ! must be done after both the natural and human killing is done, because
    ! circ_class_kill for the natural killing is calculated based on 
    ! circ_class_biomass, and if we redistribute we change circ_class_biomass.
    ! If a PFT is found empty (= killed) it is getting replanted unless
    ! species change is used. In that case we wait for the end of the year
    ! to replant with a different species.

    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts, 'before mortality_clean', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)

    !-

    CALL mortality_clean (npts, circ_class_biomass, &
         circ_class_n, age, everywhere, leaf_age,&
         leaf_frac, plant_status, when_growthinit, circ_class_dist,&
         age_stand, PFTpresent, last_cut, mai_count, &
         veget_cov_max, npp_longterm, KF, atm_to_bm, &
         wstress_season, lm_lastyearmax, lignin_struc, lignin_wood, &
         som, litter, dt_days, forest_managed, bm_to_litter, lab_fac, &
         gpp_daily, resp_maint,&
         resp_growth, npp_daily, use_reserve, mai, pai,&
         rue_longterm, previous_wood_volume,species_change_map,&
         tau_eff_leaf, tau_eff_sap, tau_eff_root, moiavail_growingseason,&
         k_latosa_adapt, lpft_replant, cn_leaf_min_season, &
         nstress_season, soil_n_min, plant_n_uptake_daily, p_O2, bact, &
         CN_som_litter_longterm, matrixA, matrixV, vectorB, vectorU)
         
    atm_to_bm_hist(:,:,imor,:) = atm_to_bm(:,:,:)
    veget_cov_max_hist(:,:,imor) =  veget_cov_max(:,:)
    atm_to_bm(:,:,:) = zero

  !! 11. Leaf senescence and other turnover processes

    ! Calculate all turnover processes which are responsible
    ! for moving carbon/nitrogen to bifferent living biomass pools in 
    ! the case of sapwood to heartwood turnover or moving carbon/
    ! nitrogen from the living biomass to the litter pools. The
    ! turnover module also deals with leaf fall in autumn.

    ! Debug
    !printlev_loc=get_printlev('stomate_lpj')
    IF (printlev_loc>=4) CALL debug_write(npts,'before turnover', &
         circ_class_biomass, circ_class_n, circ_class_kill, &
         plant_status, veget_cov_max)
    !-

    CALL turn_over (npts, dt_days, PFTpresent, herbivores, &
         maxmoiavail_lastyear, minmoiavail_lastyear, moiavail_week, &
         moiavail_month, t2m_longterm, t2m_month, t2m_week, &
         veget_cov_max, gdd_from_growthinit, leaf_age, leaf_frac, age, &
         turnover_daily, plant_status, turnover_time, &
         circ_class_biomass, circ_class_n, &
         when_growthinit, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
         harvest_pool, harvest_type, harvest_cut, harvest_area, &
         resolution, wstress_month)

  !! 12. Product use
 
    !  +++CHECK+++
    !  Write the harvest pools to the history files here because they will
    !  be reset when the subroutine basic_product_use is called. Note that
    !  due to some peculiarities in the coding the units of the harvest 
    !  pool differ for grasses, crops and forests. For forest, the 
    !  harvest_pool is filled and emptied the last day of the year. Hence
    !  all daily values are 0, except for the last day. When writing,
    !  for example, monthly output files the mean monthly value will be 
    !  written. To obtain the absolute harvest, the value in the history
    !  file should be multiplied by 30 or 31 depending on whether a 360 or 
    !  365 day calendre is used. For grasslands the harvest_pool is filled
    !  daily with the turnover. So the mean value written to the history file
    !  is the correct value. Crops are typically harvested before December
    !  31st but the harvest_pool is only emptied on December 31st (just as
    !  for grasses and forests). So between the harvest date and December
    !  31st the same biomass remains in the harvest pools. If the harvest 
    !  takes place before December 1st the same average value is written for
    !  every day of December and therefore the last month contains the correct
    !  value for harvest.
    !  All of this would become straightforward if we would only write 
    !  harvest_pool the last day of the year. We tried using the operator 
    !  once in histwrite but were unhappy with the outcome. The IPSL team 
    !  no longer maintains histwrite as it should get replaced by the 
    !  ioxserver.
    CALL histwrite_p (hist_id_stomate, 'HARVEST_POOL_C', itime, &
         SUM(harvest_pool(:,:,:,icarbon),3), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HARVEST_POOL_N', itime, &
         SUM(harvest_pool(:,:,:,initrogen),3), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HARVEST_TYPE', itime, &
         harvest_type(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HARVEST_CUT', itime, &
         harvest_cut(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HARVEST_AREA', itime, &
         harvest_area(:,:), npts*nvm, horipft_index)

    CALL xios_orchidee_send_field("HARVEST_POOL_C",SUM(harvest_pool(:,:,:,icarbon),3))
    CALL xios_orchidee_send_field("HARVEST_POOL_N",SUM(harvest_pool(:,:,:,initrogen),3))
    CALL xios_orchidee_send_field("HARVEST_TYPE",harvest_type(:,:))
    CALL xios_orchidee_send_field("HARVEST_CUT",harvest_cut(:,:)) 
    CALL xios_orchidee_send_field("HARVEST_AREA",harvest_area(:,:))
    !+++++++++++
    
    !  Agricultural harvest now goes into the short-lived product pool.
    !  Hence, product use can only be estimated after crop_harvest which
    !  in turn depends on the calculation of turnover. In previous versions
    !  product use was an integral part of LCC. Maintaining this approach 
    !  would result in inconsistencies with harvest from forestry and 
    !  agriculture. Note that there is still no real harvest from grasslands. 
    !  Grasslands are thus wrongly considered natural PFTs without any human 
    !  use. To partly overcome this issue the turnover in grasslands is moved 
    !  into the harvest pool but this approach should be considered as a
    !  patch rather than a permanent solution.
    IF(EndofYear)THEN
       
       ! Debug
       !printlev_loc=get_printlev('stomate_lpj')
       IF (printlev_loc>=4) CALL debug_write(npts,'before wood use', &
            circ_class_biomass, circ_class_n, circ_class_kill,&
            plant_status, veget_cov_max)
       !- 

       IF(ok_dimensional_product_use)THEN

          ! Wood harvest with small dimensions is being used as fire wood
          ! the remaining wood goes into the short, medium and long product
          ! pools. This implies that when the dimensions of the harvest 
          ! change, the product pools will change as well
          CALL dim_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
               harvest_type, harvest_cut, harvest_area, &
               flux_prod_s, flux_prod_m, flux_prod_l, prod_s, &
               prod_m, prod_l, prod_s_total, prod_m_total, &
               prod_l_total, flux_prod_total, flux_s, flux_m, &
               flux_l, resolution, veget_cov_max) 
          
       ELSE

          ! The scheme does not account for the harvest dimensions. All
          ! wood is distributed across the short, medium and long product
          ! pools. This basically means that wood use in 1600, 1700, 1800,
          ! 1900 and 2000 was identical.
          CALL basic_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
               harvest_type, harvest_cut, harvest_area, &
               flux_prod_s, flux_prod_m, flux_prod_l, prod_s, &
               prod_m, prod_l, prod_s_total, prod_m_total, &
               prod_l_total, flux_prod_total, flux_s, flux_m, &
               flux_l, resolution, veget_cov_max)

       ENDIF ! dimensional product use

    ELSE ! EndOfYear
       
       ! These variables are all intent(out) so they should be initialized 
       ! if it's not the end of the year
       CALL product_init(flux_prod_s, flux_prod_m, flux_prod_l, &
            prod_s_total, prod_m_total, prod_l_total, flux_s, &
            flux_m, flux_l, flux_prod_total)
       
    ENDIF

 !! 13. Establishment of saplings/recruitment   
  
    IF(ok_recruitment .AND. EndOfYear)THEN

       ! Debug
       IF(printlev_loc.GT.4) WRITE(numout,*) 'We will call recruitment from stomate_lpj.f90  '
       !-

       !! 12.1 Call recruitment process in
       CALL prescribe (npts, veget_cov_max, dt_days, PFTpresent, &
            everywhere, when_growthinit, leaf_frac, circ_class_dist, & 
            circ_class_n, circ_class_biomass, atm_to_bm, &
            forest_managed, KF, plant_status, age, npp_longterm, &
            lm_lastyearmax, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
            k_latosa_adapt, light_tran_to_level_season, &
            EndOfYear, lpft_replant, species_change_map)

       !We store intermediate veget_cov_max and atm_to_bm in order to calculate
       !the mass balance closure at the end odf the subroutine
       atm_to_bm_hist(:,:,irec,:) = atm_to_bm(:,:,:)
       veget_cov_max_hist(:,:,irec) =  veget_cov_max(:,:)
       atm_to_bm(:,:,:) = zero

    ENDIF ! establishment/recruitment    
  
  !! 13. Establishment of saplings

!!       IF (.NOT. ok_constant_mortality) THEN

          ! Establish new plants based on the functional allocation
          ! +++CHECK+++
          ! The comments below were written during the first round of
          ! ORCHIDEE-CAN code development. In the mean time a recruitment
          ! module was added to prescribe. Some the functionality
          ! that was aimed for in the initial establish routine can now
          ! be accounted for in prescribe.

          ! Old comments:
          ! How does this intefer with self-thinning it seems that this 
          ! code tries to maintain a +/- high biomass by adding individuals 
          ! when the fpc (foliage) projected cover decreases). This is 
          ! exactely what we want to avoid as we want to introduce gaps 
          ! in the static approach. We could try to adjust the 
          ! proposed aproach for unmanaged systems but I doubt whether 
          ! the rule of Deleuze and Dhote could deal with such an extreme
          ! range of diameter sizes. Most unmanged forest are, however, 
          ! under a disturbance regime and may be aproximated 
          ! by an 'even-age' evolution.

          ! NOTE:  We have decided not to create this routine.  The 
          ! reasoning is that we really want gaps in the system and don't 
          ! want to continually establish. All of this should be taken 
          ! care of in prescribe at the end
          ! of the year.  If someone wants to run with DGVM and the new 
          ! allocation, this will require some effort here.

          !CALL establish_prognostic (npts, dt_days, PFTpresent, regenerate, &
          !     neighbours, resolution, need_adjacent, herbivores, &
          !     precip_lastyear, gdd0_lastyear, lm_lastyearmax, avail_tree, &
          !     avail_grass, npp_longterm, leaf_age, leaf_frac, &
          !     ind, biomass, age, everywhere, &
          !     co2_to_bm, veget_cov_max, circ_class_biomass, circ_class_n)
          ! ++++++++++++

!!       ELSE
          
          ! Plant establishment does not occur when constant mortality 
          ! is used outside the DGVM. Write the values of the flags to 
          ! make sure this condition is satisfied
!!          IF (firstcall) THEN
             
!!             WRITE(numout,*) 'Plant establishment does not occur because '
!!             WRITE(numout,*) 'constant mortality is used outside the DGVM'
!!             WRITE(numout,*) 'control%ok_dgvm, ', ok_dgvm
!!             WRITE(numout,*) 'control%ok_constant_mortality, ', &
!!                  ok_constant_mortality
!!             
!!          ENDIF

!!       ENDIF  ! constant mortality 

 !! 14. Replant PFTs with a different species.

    IF(EndOfYear)THEN

       IF(ok_change_species)THEN

          IF(printlev_loc>4)  WRITE(numout,*) 'species change lpft_replant'

          ! This is an ad-hoc method to change the species after they die.
          ! It is only meant to change species through human intervention.
          ! In theory, this is a function that should be done in a DGVM, which
          ! is why this code is considered only temporary. Because we passed
          ! the mortality routines the PFTs may have changed. For example, in 
          ! sapiens_forestry the oldest age class was marked for harvest. In
          ! kill the biomass is removed and the surface area taken by the 
          ! oldest age class was moved to the youngest age class of that group
          ! Here we will move the youngest age class of the intial group to
          ! the younest age class of the new species group. First create
          ! the vectors that are then used to calculate the losses and gains
          ! in surface area (veget_cov_max)
          CALL species_change(npts, lpft_replant, veget_cov_max_new, &
               forest_managed, species_change_map, veget_cov_max, fm_change_map)

          ! Use the land cover change code to move the biomass to the
          ! correct PFT. When veget_cov_max is lost (i.e., oaks are changed 
          ! to grasslandsthe) standard code will distribute this loss 
          ! proportionaly (i.e., all age classes with oak will be partly
          ! coverted) to the populated age classes of the species group. 
          ! In case of species change we only want to change the surface 
          ! area of the youngest age class (because we will only plant 
          ! trees in the youngest age class). set losses = youngest
          losses = 'youngest'

          CALL land_cover_change_main (npts, dt_days, veget_cov_max, &
               veget_cov, harvest_pool, &
               harvest_type, harvest_cut, harvest_area, &
               litter, som, lignin_struc, lignin_wood, &
               PFTpresent, everywhere, when_growthinit, & 
               leaf_frac, circ_class_n, circ_class_biomass, &
               atm_to_bm, forest_managed, &
               KF, wstress_season, wstress_month, plant_status, npp_longterm, &
               age, lm_lastyearmax, harvest_pool_bound, resolution, &
               bm_to_litter, turnover_daily,leaf_age, circ_class_dist, &
               tau_eff_leaf, tau_eff_sap, &
               tau_eff_root, veget_cov_max_new, age_stand, last_cut, &
               k_latosa_adapt, losses, light_tran_to_level_season, &
               EndOfYear, lpft_replant, soil_n_min, bact, species_change_map)    

          veget_cov_max_hist(:,:,ispc) = veget_cov_max(:,:)
          atm_to_bm_hist(:,:,ispc,:) = atm_to_bm(:,:,:)
          atm_to_bm(:,:,:) = zero

       ENDIF ! checking for species change  

    ENDIF ! checking for the end of the year

    !! 11. Light competition
     ! +++CHECK+++
     ! Uncomment when working on the DGVM

!!$    !! If not using constant mortality then kill with light competition
!!$!    IF ( ok_dgvm .OR. .NOT.(lpj_gap_const_mort) ) THEN
!!$    IF ( ok_dgvm ) THEN
!!$ 
!!$       !! 11.1 Light competition
!!$       CALL light (npts, dt_days, &
!!$            veget_cov_max, fpc_max, PFTpresent, cn_ind, maxfpc_lastyear, &
!!$            lm_lastyearmax, ind, biomass, veget_lastlight, bm_to_litter, mortality)
!!$       
!!$       !! 11.2 Reset attributes for eliminated PFTs
!!$       CALL kill (npts, 'light     ', lm_lastyearmax, &
!!$            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$            lai, age, leaf_age, leaf_frac, npp_longterm, &
!!$            when_growthinit, everywhere, veget_cov_max, bm_to_litter)
!!$
!!$    ENDIF
!!$
!!$   
!!$ 
!!$  !! 12. Establishment of saplings
!!$    
!!$    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort ) THEN
!!$
!!$       !! 12.1 Establish new plants
!!$       CALL establish (npts, dt_days, PFTpresent, regenerate, &
!!$            neighbours, resolution, need_adjacent, herbivores, &
!!$            precip_lastyear, gdd0_lastyear, lm_lastyearmax, &
!!$            cn_ind, lai, avail_tree, avail_grass, npp_longterm, &
!!$            leaf_age, leaf_frac, &
!!$            ind, biomass, age, everywhere, atm_to_bm, &
!!$            soil_n_min, plant_n_uptake_daily, nstress_season, veget_cov_max, woodmass_ind, &
!!$            mortality, bm_to_litter)
!!$
!!$       IF (printlev_loc>=3) WRITE (numout,*) 'after establish soil_n_min(test_grid,test_pft,:):',soil_n_min(test_grid,test_pft,:)
!!$       !! 12.2 Calculate new crown area (and maximum vegetation cover)
!!$       CALL crown (npts, PFTpresent, &
!!$            ind, biomass, woodmass_ind, &
!!$            veget_cov_max, cn_ind, height)
!!$
!!$    ENDIF
    ! +++++++++++


  !! 13. Calculate final LAI and vegetation cover
    ! +++CHECK+++
    ! Move to DGVM - not needed when using prescribed veget_cov_max
!!$    CALL cover (npts, cn_ind, ind, biomass, &
!!$         veget_cov_max, veget_cov_max_tmp, lai, &
!!$         litter, som, turnover_daily, bm_to_litter, &
!!$         atm_to_bm, co2_fire, resp_hetero, resp_maint, resp_growth, gpp_daily, &
!!$         lignin_struc, lignin_wood, soil_n_min)
    ! +++++++++++



  !! 16. Calculate vcmax

    CALL vmax (npts, dt_days, leaf_age, leaf_frac, vcmax, nue)

  !! 17. Compute the effective leaf area index. 
     
    ! Need to put in a function here to compute the height of the
    ! photosynthesis levels given the height of the energy levels and the 
    ! vegetation on the grid square.  The hightest levels will be a 
    ! function of the height of the vegetation so that we don't waste 
    ! computational time on empty levels.

    ! Debug
    ! The ipslerr_p issue is most likely causing the crash here. 
    ! printlev_loc=get_printlev('stomate_lpj')
    ! IF (printlev_loc>=4) 
    ! CALL debug_write(npts,'before lai effective', &
    !      circ_class_biomass, circ_class_n, circ_class_kill, &
    !      plant_status, veget_cov_max)

    !- 
    CALL calculate_z_level_photo(npts, circ_class_biomass, circ_class_n, &
         z_level_photo)
    
    ! Finding the true LAI per level is different from finding the
    ! effective LAI per level, so we'll do that here.  This only
    ! changes once every day so hopefully it is not too expensive.
    ! It will eventually be needed by the energy budget.  It is
    ! also needed by effective_lai for grasses and crops.
    CALL find_lai_per_level(npts, z_level_photo, &
         circ_class_biomass, circ_class_n, lai_per_level, &
         max_height_store)
    
    ! This is a function of the solar angle, and needed for the albedo
    ! and the photosynthesis.  It was very expensive to compute, so instead
    ! of computing it at every timestep we compute it at a couple points
    ! throughout the day and fit a function to it.  We do this now because
    ! the canopy as it appears now will be used for the next day.
    ! Now we actually find the effective LAI and fit the function 
    ! we'll use later on 
    CALL fitting_laieff(npts, z_level_photo, circ_class_biomass, & 
         circ_class_n, veget_cov_max, lai_per_level, laieff_fit, &
         h_array_out, z_array_out)
 
 !! 18. Check numerical consistency of this routine

    !  This test is always performed. If err_act.EQ.1 then 
    !  the value of the mass balance error -if any- is written 
    !  to the history file.
!!$    printlev_loc=get_printlev('stomate_lpj')

    ! 14.2 Check surface area
    CALL check_surface_area("stomate_lpj", npts, veget_cov_max_hist(:,:,ibeg), &
         veget_cov_max,'pixel')

    IF (printlev_loc .GE. 4) THEN  
       DO ivm=1,nvm
          WRITE(numout,*) "step, veget_cov_max, atm_to_bm for ivm:", ivm 
          WRITE(numout,*)"IBEG",veget_cov_max_hist(test_grid,ivm,ibeg),&
               atm_to_bm_hist(test_grid,ivm,ibeg,1)
          WRITE(numout,*)"IPRE",veget_cov_max_hist(test_grid,ivm,ipre),& 
               atm_to_bm_hist(test_grid,ivm,ipre,1)
          WRITE(numout,*)"IPHE",veget_cov_max_hist(test_grid,ivm,iphe),&
               atm_to_bm_hist(test_grid,ivm,iphe,1)
          WRITE(numout,*)"IGRO",veget_cov_max_hist(test_grid,ivm,igro),&
               atm_to_bm_hist(test_grid,ivm,igro,1)
          WRITE(numout,*)"IAGE",veget_cov_max_hist(test_grid,ivm,iage),&
               atm_to_bm_hist(test_grid,ivm,iage,1)
          WRITE(numout,*)"ILCC",veget_cov_max_hist(test_grid,ivm,ilcc),&
               atm_to_bm_hist(test_grid,ivm,ilcc,1)
          WRITE(numout,*)"IMOR",veget_cov_max_hist(test_grid,ivm,imor),&
               atm_to_bm_hist(test_grid,ivm,imor,1)
          WRITE(numout,*)"IREC",veget_cov_max_hist(test_grid,ivm,irec),&
               atm_to_bm_hist(test_grid,ivm,irec,1)
          WRITE(numout,*)"ISPC",veget_cov_max_hist(test_grid,ivm,ispc),&
               atm_to_bm_hist(test_grid,ivm,ispc,1)
       ENDDO
    ENDIF

    ! 14.3 Mass balance closure 
    ! 14.3.1 Calculate final biomass
    pool_end(:,:,:) = zero 
    DO iele = 1,nelements
       DO ipar = 1,nparts
          DO icir=1,ncirc
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  circ_class_biomass(:,:,icir,ipar,iele)* &
                  circ_class_n(:,:,icir) * veget_cov_max(:,:)
          ENDDO
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (bm_to_litter(:,:,ipar,iele)  + &
               turnover_daily(:,:,ipar,iele)) * veget_cov_max(:,:)
       ENDDO
       pool_end(:,1,iele) = pool_end(:,1,iele) + &
            ( SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
            SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
            SUM(prod_s(:,:,iele),2) ) / pixel_area(:)
    ENDDO

    ! Common processes. Atm_to_bm should be empty when the model enters
    ! stomate_lpj but its value can change at several call in stomate_lpj.
    ! for this reason the history of atm_to_bm needs to be stored and
    ! it has be multiplied by the history of veget_cov_max within stomate_lpj
    ! to track all the carbon down.
    DO iele = 1,nelements
       check_intern(:,:,iatm2land,iele) = &
            check_intern(:,:,iatm2land,iele) + &
            SUM(atm_to_bm_hist(:,:,:,iele) * veget_cov_max_hist(:,:,:) * dt_days,3)
       check_intern(:,1,iland2atm,iele) = check_intern(:,1,iland2atm,iele) &
            -un * ( SUM(flux_s(:,:,iele),2) + SUM(flux_m(:,:,iele),2) + &
            SUM(flux_l(:,:,iele),2) ) / pixel_area(:)
       check_intern(:,:,ilat2out,iele) = zero
       check_intern(:,:,ilat2in,iele) = -un * zero
       check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
            pool_start(:,:,iele))
    ENDDO

    closure_intern = zero
    DO ivm=1, nvm
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             ! Debug
             IF (printlev_loc>=4) THEN              
                IF(iele .EQ. 1) WRITE(numout,"(9999(G12.5,:,','))") &
                     'imbc, ivm, check_intern', test_grid , imbc, &
                     ivm, check_intern(test_grid,ivm,imbc,iele)
             ENDIF
             closure_intern(:,ivm,iele) = closure_intern(:,ivm,iele) + &
                  check_intern(:,ivm,imbc,iele)
          ENDDO
       ENDDO
    ENDDO
    CALL check_mass_balance("stomate_lpj", closure_intern, npts, &
         pool_end, pool_start, veget_cov_max, 'pixel')

    ! Add to the mass balance errors of stomate_dt_sechiba
    DO iele = 1,nelements
       mass_balance_closure(:,iele) = mass_balance_closure(:,iele) + &
            closure_intern(:,1,iele)
    ENDDO
 

    !+++CHECK+++
    ! This does not look essential. Just making daily totals.
    ! Shouldn we do this where we calculate heterothropic respiration?
!!$  !! 16. Total heterotrophic respiration
!!$     
!!$    tot_soil_carb(:,:) = zero
!!$    tot_litter_carb(:,:) = zero
!!$    DO j=2,nvm
!!$
!!$       tot_litter_carb(:,j) = tot_litter_carb(:,j) + (litter(:,istructural,j,iabove,icarbon) + &
!!$            &          litter(:,imetabolic,j,iabove,icarbon) + litter(:,iwoody,j,iabove,icarbon) + &
!!$            &          litter(:,istructural,j,ibelow,icarbon) + litter(:,imetabolic,j,ibelow,icarbon)+ &
!!$            &          litter(:,iwoody,j,ibelow,icarbon))
!!$
!!$       tot_soil_carb(:,j) = tot_soil_carb(:,j) + (som(:,iactive,j,icarbon) + &
!!$            &          som(:,islow,j,icarbon)+  som(:,ipassive,j,icarbon)+  som(:,isurface,j,icarbon))
!!$
!!$    ENDDO
!!$    tot_litter_soil_carb(:,:) = tot_litter_carb(:,:) + tot_soil_carb(:,:)
!!$
!!$!     DO k = 1, nelements ! Loop over # elements
!!$!        tot_live_biomass(:,:,k) = biomass(:,:,ileaf,k) + biomass(:,:,isapabove,k) + biomass(:,:,isapbelow,k) +&
!!$!             &                    biomass(:,:,iheartabove,k) + biomass(:,:,iheartbelow,k) + &
!!$!             &                    biomass(:,:,iroot,k) + biomass(:,:,ifruit,k) + biomass(:,:,icarbres,k)
!!$!    END DO ! Loop over # elements
!!$
!!$    tot_live_biomass(:,:,:) = biomass(:,:,ileaf,:) + biomass(:,:,isapabove,:) + biomass(:,:,isapbelow,:) +&
!!$             &                    biomass(:,:,iheartabove,:) + biomass(:,:,iheartbelow,:) + &
!!$             &                    biomass(:,:,iroot,:) + biomass(:,:,ifruit,:) + biomass(:,:,icarbres,:) + biomass(:,:,ilabile,:)
!!$
!!$
!!$    tot_turnover(:,:,:) = turnover_daily(:,:,ileaf,:) + turnover_daily(:,:,isapabove,:) + &
!!$         &         turnover_daily(:,:,isapbelow,:) + turnover_daily(:,:,iheartabove,:) + &
!!$         &         turnover_daily(:,:,iheartbelow,:) + turnover_daily(:,:,iroot,:) + &
!!$         &         turnover_daily(:,:,ifruit,:) + turnover_daily(:,:,icarbres,:) + turnover_daily(:,:,ilabile,:) 
!!$
!!$    tot_bm_to_litter(:,:,:) = bm_to_litter(:,:,ileaf,:) + bm_to_litter(:,:,isapabove,:) +&
!!$         &             bm_to_litter(:,:,isapbelow,:) + bm_to_litter(:,:,iheartbelow,:) +&
!!$         &             bm_to_litter(:,:,iheartabove,:) + bm_to_litter(:,:,iroot,:) + &
!!$         &             bm_to_litter(:,:,ifruit,:) + bm_to_litter(:,:,icarbres,:) + bm_to_litter(:,:,ilabile,:)
!!$
!!$    carb_mass_variation(:)=-carb_mass_total(:)
!!$    carb_mass_total(:)=SUM((tot_live_biomass(:,:,icarbon)+tot_litter_carb+tot_soil_carb)*veget_cov_max,dim=2) + &
!!$         &                 (prod10_total + prod100_total)
!!$    carb_mass_variation(:)=carb_mass_total(:)+carb_mass_variation(:)
   ! +++++++++++
   
 
    !! 19. Write history

    !! 19.1 Calculate the latest values before writing
    lai(:,1) = zero
    qm_dia(:,1) = zero
    qm_height(:,1) = zero

    vcmax_new=zero

    DO ipts = 1,npts
       DO ivm=2,nvm

           lai(ipts,ivm) = cc_to_lai(circ_class_biomass(ipts,ivm,:,ileaf,icarbon),&
            circ_class_n(ipts,ivm,:),ivm)
           qm_dia(ipts,ivm)=wood_to_qmdia(circ_class_biomass(ipts,ivm,:,:,icarbon), &
                circ_class_n(ipts,ivm,:), ivm)
           qm_height(ipts,ivm)=wood_to_qmheight(circ_class_biomass(ipts,ivm,:,:,icarbon), &
                circ_class_n(ipts,ivm,:), ivm)

           IF(SUM(circ_class_biomass(ipts,ivm,:,ileaf,icarbon)*&
                circ_class_n(ipts,ivm,:)) .GT. min_stomate)THEN

             vcmax_new(ipts,ivm)=nue(ipts,ivm)*SUM(circ_class_biomass(ipts,ivm,:,ileaf,initrogen)*&
                circ_class_n(ipts,ivm,:))* ext_coeff_N(ivm) / &
                ( 1.-exp(-ext_coeff_N(ivm) * lai(ipts,ivm)))

           ENDIF
       ENDDO
    ENDDO

    ! Soil, litter, biomass and llc
    ! Total living biomass (gC m-2)i
    DO l=1,nelements
        tot_live_biomass(:,:,l) = SUM((circ_class_biomass(:,:,:,ileaf,l) +& 
             circ_class_biomass(:,:,:,isapabove,l) + &
             circ_class_biomass(:,:,:,isapbelow,l) + &
             circ_class_biomass(:,:,:,iheartabove,l) + &
             circ_class_biomass(:,:,:,iheartbelow,l) + &
             circ_class_biomass(:,:,:,iroot,l) + &
             circ_class_biomass(:,:,:,ifruit,l) + &
             circ_class_biomass(:,:,:,icarbres,l))*circ_class_n(:,:,:),3)
    ENDDO

    !! Calculate the wood volume change at pixel level by ncuts
    DO ivm = 1,nvm
       IF(is_tree(ivm))THEN
          DO icut= 1,ncut_times

             ! This variable is used to monitor the change in biomass due
             ! to different reasons for cutting trees. For this variable we
             ! don't care about the exact forest management but we do care
             ! whether the forest was managed or not. If the forest is
             ! managed the cuttings will be exported. If the forest is not
             ! managed the cuttings are left on-site.
             wood_volume_pix_cut(:,icut) = wood_volume_pix_cut(:,icut) + &
                  ((biomass_cut(:,ivm,isapabove,icarbon,ifm_none,icut) + &
                  biomass_cut(:,ivm,iheartabove,icarbon,ifm_none,icut)) * &
                  (1-branch_ratio(ivm)) / pipe_density(ivm) + &
                  (biomass_cut(:,ivm,isapabove,icarbon,ifm_thin,icut) + &
                  biomass_cut(:,ivm,iheartabove,icarbon,ifm_thin,icut)) * &
                  (1-branch_ratio(ivm)) / pipe_density(ivm) + &
                  (biomass_cut(:,ivm,isapabove,icarbon,ifm_cop,icut) + &
                  biomass_cut(:,ivm,iheartabove,icarbon,ifm_cop,icut)) * &
                  (1-branch_ratio(ivm)) / pipe_density(ivm) + &
                  (biomass_cut(:,ivm,isapabove,icarbon,ifm_src,icut) + &
                  biomass_cut(:,ivm,iheartabove,icarbon,ifm_src,icut)) * &
                  (1-branch_ratio(ivm)) / pipe_density(ivm)) * &
                  veget_cov_max(:,ivm)
                   
          ENDDO !icut
       ENDIF ! is_tree
    ENDDO !ivm

    CALL xios_orchidee_send_field("WOOD_VOL_PIX_CUT",wood_volume_pix_cut)
    
    !! 19.2 Write using XIOS
    !! 19.2.1 stomate history
    CALL xios_orchidee_send_field("RESOLUTION_X",resolution(:,1))
    CALL xios_orchidee_send_field("RESOLUTION_Y",resolution(:,2))
    CALL xios_orchidee_send_field("VEGET_COV_MAX",veget_cov_max)
    CALL xios_orchidee_send_field("CONTFRAC_STOMATE",contfrac(:))

    ! climatic conditions
    CALL xios_orchidee_send_field("T2M_MONTH",t2m_month)
    CALL xios_orchidee_send_field("T2M_WEEK",t2m_week)
    CALL xios_orchidee_send_field("TSEASON",Tseason)
    CALL xios_orchidee_send_field("TMIN_SPRING_TIME",Tmin_spring_time)
    CALL xios_orchidee_send_field("FPC_MAX",fpc_max)
    CALL xios_orchidee_send_field("MAXFPC_LASTYEAR",maxfpc_lastyear)
    !CALL xios_orchidee_send_field("ONSET_DATE",onset_date(:,:,2))
    CALL xios_orchidee_send_field("LITTERHUM",litterhum_daily)
    CALL xios_orchidee_send_field("FIREINDEX",fireindex(:,:))
    
    ! Fluxes
    CALL xios_orchidee_send_field("NPP_STOMATE",npp_daily)
    CALL xios_orchidee_send_field("GPP",gpp_daily)
    CALL xios_orchidee_send_field("HET_RESP",resp_hetero(:,:))
    CALL xios_orchidee_send_field("CO2_TAKEN",SUM(atm_to_bm_hist(:,:,:,initrogen),3))
    IF(ok_ncycle) CALL xios_orchidee_send_field("N_TAKEN",SUM(atm_to_bm_hist(:,:,:,initrogen),3))
    CALL xios_orchidee_send_field("RDI",rdi(:,:))
    CALL xios_orchidee_send_field("RDI_TARGET_UPPER",rdi_target_upper(:,:))
    CALL xios_orchidee_send_field("RDI_TARGET_LOWER",rdi_target_lower(:,:))
    CALL xios_orchidee_send_field("MAINT_RESP",resp_maint)
    CALL xios_orchidee_send_field("GROWTH_RESP",resp_growth)
    CALL xios_orchidee_send_field("PLANT_STATUS",plant_status)

    ! Fire results 
    CALL xios_orchidee_send_field("CO2_FIRE",co2_fire)

    ! Soil, litter, biomass and llc
    ! Total living biomass (gC m-2)
    DO l=1,nelements
        tot_live_biomass(:,:,l) = SUM((circ_class_biomass(:,:,:,ileaf,l) +&
            circ_class_biomass(:,:,:,isapabove,l) + &
            circ_class_biomass(:,:,:,isapbelow,l) + &
            circ_class_biomass(:,:,:,iheartabove,l) + &
            circ_class_biomass(:,:,:,iheartbelow,l) + &
            circ_class_biomass(:,:,:,iroot,l) + &
            circ_class_biomass(:,:,:,ifruit,l) + &
            circ_class_biomass(:,:,:,icarbres,l))*circ_class_n(:,:,:),3)
    ENDDO

    DO l=1,nelements 
       IF     (l == icarbon) THEN 
          element_str(l) = '_c' 
       ELSEIF (l == initrogen) THEN 
          element_str(l) = '_n' 
       ELSE 
          STOP 'Define element_str' 
       ENDIF 

       !! The writting could be more compact, but writting all the components to
       !! seperate variables might make it more userfriendly i.e. the user need not to
       !! remember the order of the indexes of nelement and nparts.(asl)      
       ! Biomass
       CALL xios_orchidee_send_field("TOTAL_M"//TRIM(element_str(l)),tot_live_biomass(:,:,l))
       CALL xios_orchidee_send_field("LEAF_M"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,ileaf,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("SAP_M_AB"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,isapabove,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("SAP_M_BE"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,isapbelow,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("HEART_M_AB"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,iheartabove,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("HEART_M_BE"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,iheartbelow,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("ROOT_M"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,iroot,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("FRUIT_M"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,ifruit,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("LABILE_M"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,ilabile,l)*circ_class_n(:,:,:),3))
       CALL xios_orchidee_send_field("RESERVE_M"//TRIM(element_str(l)),&
            SUM(circ_class_biomass(:,:,:,icarbres,l)*circ_class_n(:,:,:),3))

       !CALL xios_orchidee_send_field("TOTAL_TURN"//TRIM(element_str(l)),tot_turnover(:,:,l))
       CALL xios_orchidee_send_field("LEAF_TURN"//TRIM(element_str(l)),turnover_daily(:,:,ileaf,l))
       CALL xios_orchidee_send_field("SAP_AB_TURN"//TRIM(element_str(l)),turnover_daily(:,:,isapabove,l))
       CALL xios_orchidee_send_field("ROOT_TURN"//TRIM(element_str(l)),turnover_daily(:,:,iroot,l))
       CALL xios_orchidee_send_field("FRUIT_TURN"//TRIM(element_str(l)),turnover_daily(:,:,ifruit,l))

       !CALL xios_orchidee_send_field("TOTAL_BM_LITTER"//TRIM(element_str(l)),tot_bm_to_litter(:,:,l))
       CALL xios_orchidee_send_field("LEAF_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ileaf,l))
       CALL xios_orchidee_send_field("SAP_AB_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,isapabove,l))
       CALL xios_orchidee_send_field("SAP_BE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,isapbelow,l))
       CALL xios_orchidee_send_field("HEART_AB_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iheartabove,l))
       CALL xios_orchidee_send_field("HEART_BE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iheartbelow,l))
       CALL xios_orchidee_send_field("ROOT_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iroot,l))
       CALL xios_orchidee_send_field("FRUIT_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ifruit,l))
       CALL xios_orchidee_send_field("LABILE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ilabile,l))
       CALL xios_orchidee_send_field("RESERVE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,icarbres,l))

       ! Litter
       CALL xios_orchidee_send_field("LITTER_STR_AB"//TRIM(element_str(l)),litter(:,istructural,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_MET_AB"//TRIM(element_str(l)),litter(:,imetabolic,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_WOD_AB"//TRIM(element_str(l)),litter(:,iwoody,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_STR_BE"//TRIM(element_str(l)),litter(:,istructural,:,ibelow,l))
       CALL xios_orchidee_send_field("LITTER_MET_BE"//TRIM(element_str(l)),litter(:,imetabolic,:,ibelow,l))
       CALL xios_orchidee_send_field("LITTER_WOD_BE"//TRIM(element_str(l)),litter(:,iwoody,:,ibelow,l))

       ! Soil
       CALL xios_orchidee_send_field("SOIL_ACTIVE"//TRIM(element_str(l)),som(:,iactive,:,l))
       CALL xios_orchidee_send_field("SOIL_SLOW"//TRIM(element_str(l)),som(:,islow,:,l))
       CALL xios_orchidee_send_field("SOIL_PASSIVE"//TRIM(element_str(l)),som(:,ipassive,:,l))
       CALL xios_orchidee_send_field("SOIL_SURF"//TRIM(element_str(l)),som(:,isurface,:,l))

       ! land cover change
       CALL xios_orchidee_send_field('FLUX_PROD_S'//TRIM(element_str(l)),flux_prod_s(:,l))
       CALL xios_orchidee_send_field('FLUX_PROD_M'//TRIM(element_str(l)),flux_prod_m(:,l))
       CALL xios_orchidee_send_field('FLUX_PROD_L'//TRIM(element_str(l)),flux_prod_l(:,l))
       CALL xios_orchidee_send_field('PROD_S'//TRIM(element_str(l)),prod_s(:,:,l))
       CALL xios_orchidee_send_field('PROD_M'//TRIM(element_str(l)),prod_m(:,:,l))
       CALL xios_orchidee_send_field('PROD_L'//TRIM(element_str(l)),prod_l(:,:,l))
       CALL xios_orchidee_send_field('FLUX_S'//TRIM(element_str(l)),flux_s(:,:,l))
       CALL xios_orchidee_send_field('FLUX_M'//TRIM(element_str(l)),flux_m(:,:,l))
       CALL xios_orchidee_send_field('FLUX_L'//TRIM(element_str(l)),flux_l(:,:,l))

    ENDDO

    !CN longterm ratios
    IF (spinup_analytic) THEN
       CALL xios_orchidee_send_field("CN_LONGTERM_ACTIVE",CN_som_litter_longterm(:,:,iactive_pool))
       CALL xios_orchidee_send_field("CN_LONGTERM_SLOW",CN_som_litter_longterm(:,:,islow_pool))
       CALL xios_orchidee_send_field("CN_LONGTERM_PASSIVE",CN_som_litter_longterm(:,:,ipassive_pool))
       CALL xios_orchidee_send_field("CN_LONGTERM_SURF",CN_som_litter_longterm(:,:,isurface_pool))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_STR_AB",CN_som_litter_longterm(:,:,istructural_above))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_STR_BE",CN_som_litter_longterm(:,:,istructural_below))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_MET_AB",CN_som_litter_longterm(:,:,imetabolic_above))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_MET_BE",CN_som_litter_longterm(:,:,imetabolic_below))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_WOD_AB",CN_som_litter_longterm(:,:,iwoody_above))
       CALL xios_orchidee_send_field("CN_LONGTERM_LITTER_WOD_BE",CN_som_litter_longterm(:,:,iwoody_below))
    ENDIF
 
    ! Stand structure
    CALL xios_orchidee_send_field("DIAMETER",qm_dia)
    CALL xios_orchidee_send_field("LAI_PER_LEVEL",lai_per_level)
    CALL xios_orchidee_send_field("CLEVEL_HEIGHT",z_level_photo)
    CALL xios_orchidee_send_field("lai_ipcc",SUM(lai*veget_cov_max,dim=2)*contfrac)
    CALL xios_orchidee_send_field("LAI",lai) ! this is written twice!
    CALL xios_orchidee_send_field("HEIGHT",qm_height)
    CALL xios_orchidee_send_field("IND",SUM(circ_class_n(:,:,:),3))
    CALL xios_orchidee_send_field("WOODMASS_IND",woodmass_ind)
    CALL xios_orchidee_send_field("AGE",age)
    CALL xios_orchidee_send_field("DEADLEAF_COVER",deadleaf_cover)
    CALL xios_orchidee_send_field("TOTAL_SOIL_CARB",tot_litter_soil_carb)
    CALL xios_orchidee_send_field("CN_IND",cn_ind)

    ! Growth stress and factors
    CALL xios_orchidee_send_field("MOISTRESS",moiavail_week)
    CALL xios_orchidee_send_field("VCMAX",vcmax)
    CALL xios_orchidee_send_field("VCMAX_NEW",vcmax_new)
    CALL xios_orchidee_send_field("TURNOVER_TIME_LEAF",turnover_time(:,:,ileaf))
    CALL xios_orchidee_send_field("TURNOVER_TIME_ROOT",turnover_time(:,:,iroot))
    CALL xios_orchidee_send_field("TURNOVER_TIME_SAP_AB",turnover_time(:,:,isapabove))
    CALL xios_orchidee_send_field("TURNOVER_TIME_FRUIT",turnover_time(:,:,ifruit))


    ! for functional allocation, we have all of this information
    ! at every step so we can write it out.
    var_real=REAL(age_stand)
    CALL xios_orchidee_send_field("AGE_STAND",var_real)
    var_real=REAL(forest_managed)
    CALL xios_orchidee_send_field("FOREST_MANAGED",var_real)
    var_real=REAL(last_cut)
    CALL xios_orchidee_send_field("LAST_CUT",var_real)
    var_real=REAL(rotation_n)
    CALL xios_orchidee_send_field("ROTATION_N",var_real)
    CALL xios_orchidee_send_field("LITTER_RAKE_FRAC",lrake_frac)
    CALL xios_orchidee_send_field("MAI",mai)
    CALL xios_orchidee_send_field("PAI",pai)
    CALL xios_orchidee_send_field("SIGMA",sigma)
    CALL xios_orchidee_send_field("GAMMA",gammas)


   !! 19.2.2 ipcc history
    CALL xios_orchidee_send_field('RESOLUTION_X',resolution(:,1))
    CALL xios_orchidee_send_field('RESOLUTION_Y',resolution(:,2))
    CALL xios_orchidee_send_field('CONTFRAC_STOMATE',contfrac(:)) 
  
    !CALL xios_orchidee_send_field("cVeg",SUM(tot_live_biomass(:,:,icarbon)*veget_cov_max,dim=2)/1e3)
    !CALL xios_orchidee_send_field("cLitter",SUM(tot_litter_carb*veget_cov_max,dim=2)/1e3)
    !CALL xios_orchidee_send_field("cSoil",SUM(tot_soil_carb*veget_cov_max,dim=2)/1e3)
!!$    CALL xios_orchidee_send_field("cProduct",(prod10_total + prod100_total)/1e3)
   ! CALL xios_orchidee_send_field("cMassVariation",carb_mass_variation/1e3/one_day)

    CALL xios_orchidee_send_field("lai_ipcc",SUM(lai*veget_cov_max,dim=2))  ! m2/m2

    ! Carbon fluxes transformed from gC/m2/d into kgC/m2/s
    CALL xios_orchidee_send_field("gpp_ipcc",SUM(gpp_daily*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("ra",SUM((resp_maint+resp_growth)*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("npp_ipcc",SUM(npp_daily*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("rh",SUM(resp_hetero*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("fFire",SUM(co2_fire*veget_cov_max,dim=2)/1e3/one_day)
!   As LLC is not acitivated yet, flux_prod_total is not calculated.
!   Likewise, harvest_above is no longer used. Possibly, it can  be replaced
!   by harvest_pool? If yes, be careful with the units, and should be done further 
!   up the code, as harvest_pool is cleared in basic_product_use.
!    CALL xios_orchidee_send_field("fHarvest",harvest_above/1e3/one_day)
!    CALL xios_orchidee_send_field("fLuc",flux_prod_total/1e3/one_day)
!    CALL xios_orchidee_send_field("nbp",(SUM((gpp_daily + atm_to_bm(:,:,icarbon) - &
!        (resp_maint+resp_growth+resp_hetero)-co2_fire) *veget_cov_max,dim=2)- &
!        flux_prod_total-harvest_above)/1e3/one_day)
    !CALL xios_orchidee_send_field("fVegLitter",SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*&
    !     veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("fLitterSoil",SUM(SUM(som_input(:,:,:,icarbon),dim=2)*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("cLeaf",SUM(SUM(circ_class_biomass(:,:,:,ileaf,icarbon)*&
        circ_class_n(:,:,:),3)*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cWood",SUM(SUM((circ_class_biomass(:,:,:,isapabove,icarbon)+&
        circ_class_biomass(:,:,:,iheartabove,icarbon))*circ_class_n(:,:,:),3)*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cRoot",SUM(SUM((circ_class_biomass(:,:,:,iroot,icarbon) + &
        circ_class_biomass(:,:,:,isapbelow,icarbon) + &
        circ_class_biomass(:,:,:,iheartbelow,icarbon) )*circ_class_n(:,:,:),3)*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cMisc",SUM(SUM((circ_class_biomass(:,:,:,icarbres,icarbon) + &
        circ_class_biomass(:,:,:,ifruit,icarbon))*circ_class_n(:,:,:),3)*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cLitterAbove",SUM((litter(:,istructural,:,iabove,icarbon)+&
         litter(:,imetabolic,:,iabove,icarbon)+ litter(:,iwoody,:,iabove,icarbon))*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cLitterBelow",SUM((litter(:,istructural,:,ibelow,icarbon)+&
         litter(:,imetabolic,:,ibelow,icarbon)+ litter(:,iwoody,:,ibelow,icarbon))*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cSoilFast",SUM((som(:,iactive,:,icarbon)+som(:,isurface,:,icarbon)) * &
         veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cSoilMedium",SUM(som(:,islow,:,icarbon)*veget_cov_max,dim=2)/1e3)
    CALL xios_orchidee_send_field("cSoilSlow",SUM(som(:,ipassive,:,icarbon)*veget_cov_max,dim=2)/1e3)

    ! Vegetation fractions [0,100] 
    CALL xios_orchidee_send_field("landCoverFrac",veget_cov_max*100) 
    vartmp(:)=zero 
    DO j = 2,nvm 
       IF (is_deciduous(j)) THEN 
          vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100 
       ENDIF 
    ENDDO 

    CALL xios_orchidee_send_field("treeFracPrimDec",vartmp) 
    vartmp(:)=zero 
    DO j = 2,nvm 
       IF (is_evergreen(j)) THEN 
          vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100 
       ENDIF 
    ENDDO
    
    CALL xios_orchidee_send_field("treeFracPrimEver",vartmp) 
    vartmp(:)=zero 
    DO j = 2,nvm 
       IF ( .NOT.(is_c4(j)) ) THEN 
          vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100 
       ENDIF 
    ENDDO
 
    CALL xios_orchidee_send_field("c3PftFrac",vartmp) 
    vartmp(:)=zero 
    DO j = 2,nvm 
       IF ( is_c4(j) ) THEN 
          vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100 
       ENDIF 
    ENDDO 

    CALL xios_orchidee_send_field("c4PftFrac",vartmp)

    ! Carbon fluxes transformed from gC/m2/d into kgC/m2/s
    CALL xios_orchidee_send_field("rGrowth",SUM(resp_growth*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("rMaint",SUM(resp_maint*veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("nppLeaf",SUM(bm_alloc(:,:,ileaf,icarbon)*&
         veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("nppWood",SUM(bm_alloc(:,:,isapabove,icarbon)*&
         veget_cov_max,dim=2)/1e3/one_day)
    CALL xios_orchidee_send_field("nppRoot",SUM(( bm_alloc(:,:,isapbelow,icarbon) + &
         bm_alloc(:,:,iroot,icarbon) )*veget_cov_max,dim=2)/1e3/one_day)     

    ! 19.3 Write to file using histwrite
    CALL histwrite_p (hist_id_stomate, 'RESOLUTION_X', itime, &
         resolution(:,1), npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'RESOLUTION_Y', itime, &
         resolution(:,2), npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'VEGET_COV_MAX', itime, &
         veget_cov_max, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'CONTFRAC', itime, &
         contfrac(:), npts, hori_index)
    
    ! Climatic conditions
    CALL histwrite_p (hist_id_stomate, 'T2M_MONTH', itime, &
         t2m_month, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'T2M_WEEK', itime, &
         t2m_week, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'TSEASON', itime, &
         Tseason, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'TMIN_SPRING_TIME', itime, &
         Tmin_spring_time, npts*nvm, horipft_index)
!    CALL histwrite_p (hist_id_stomate, 'ONSET_DATE', itime, &
!         onset_date(:,:,2), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'FPC_MAX', itime, &
         fpc_max, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'MAXFPC_LASTYEAR', itime, &
         maxfpc_lastyear, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'FIREINDEX', itime, &
         fireindex(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTERHUM', itime, &
         litterhum_daily, npts, hori_index)

    ! Fire results
    CALL histwrite_p (hist_id_stomate, 'CO2_FIRE', itime, &
         co2_fire, npts*nvm, horipft_index)
    
    ! Stand structure
    CALL histwrite_p (hist_id_stomate, 'LAI', itime, &
         lai, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'DIAMETER', itime, &
         qm_dia, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HEIGHT', itime, &
         qm_height, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'IND', itime, &
         SUM(circ_class_n(:,:,:),3), npts*nvm, horipft_index)
    
    ! Calculate lai per level.  Also print out the real height of
    ! the canopy level.
    DO ivm=1,nvm
       
       WRITE(var_name,'(A,I3.3)') 'LAI_PER_LEVEL_',ivm
       CALL histwrite_p (hist_id_stomate, var_name, itime, &
            lai_per_level(:,ivm,:), npts*nlevels_tot, horican_index)
       
       WRITE(var_name,'(A,I3.3)') 'CLEVEL_HEIGHT_',ivm
       CALL histwrite_p (hist_id_stomate, var_name, itime, &
            z_level_photo(:,ivm,:), npts*nlevels_tot, horican_index) 
       
    ENDDO
    
!!$     CALL histwrite_p (hist_id_stomate, 'CN_IND', itime, &
!!$          cn_ind, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'WOODMASS_IND', itime, &
         woodmass_ind, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'AGE', itime, &
         age, npts*nvm, horipft_index)
    
    ! Growth stress and factors
    CALL histwrite_p (hist_id_stomate, 'MOISTRESS', itime, &
         moiavail_week, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'VCMAX', itime, &
         vcmax, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'VCMAX_NEW', itime, &
         vcmax_new, npts*nvm, horipft_index)

     !+++CHECK+++
     ! New dimensions
     !! CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_LEAF', itime, &
     !!      turnover_time(:,:,ileaf), npts*nvm, horipft_index)
     !! CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_ROOT', itime, &
     !!      turnover_time(:,:,iroot), npts*nvm, horipft_index)
     !! CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_SAP', itime, &
     !!      turnover_time(:,:,isapabove), npts*nvm, horipft_index)
     !! CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_FRUIT', itime, &
     !!      turnover_time(:,:,ifruit), npts*nvm, horipft_index) 
     !+++++++++++

    ! for functional allocation, we have all of this information
    ! at every step so we can write it out.
    var_real=REAL(age_stand)
    CALL histwrite_p (hist_id_stomate, 'AGE_STAND', itime, &
         var_real, npts*nvm, horipft_index) 
    CALL histwrite_p (hist_id_stomate, 'MAI', itime, &
         mai, npts*nvm, horipft_index) 
    CALL histwrite_p (hist_id_stomate, 'PAI', itime, &
         pai, npts*nvm, horipft_index) 
    var_real=REAL(rotation_n)
    CALL histwrite_p (hist_id_stomate, 'ROTATION_N', itime, &
         var_real, npts*nvm, horipft_index)
    var_real=REAL(last_cut)
    CALL histwrite_p (hist_id_stomate, 'LAST_CUT', itime, &
         var_real, npts*nvm, horipft_index)
    var_real=REAL(forest_managed)
    CALL histwrite_p (hist_id_stomate, 'FOREST_MANAGED', itime, &
         var_real, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_RAKE_FRAC', itime, &
         lrake_frac, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SIGMA', itime, &
         sigma, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'GAMMA', itime, &
          gammas, npts*nvm, horipft_index)
    
    
    ! Fluxes
    CALL histwrite_p (hist_id_stomate, 'NPP', itime, &
         npp_daily, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'GPP', itime, &
         gpp_daily, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HET_RESP', itime, &
         resp_hetero(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'CO2_TAKEN', itime, &
         SUM(atm_to_bm_hist(:,:,:,icarbon),3), npts*nvm, horipft_index)
    IF(ok_ncycle) CALL histwrite_p (hist_id_stomate, 'N_TAKEN', itime, &
         SUM(atm_to_bm_hist(:,:,:,initrogen),3), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'RDI', itime, &
         rdi, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'RDI_TARGET_UPPER', itime, &
         rdi_target_upper, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'RDI_TARGET_LOWER', itime, &
         rdi_target_lower, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'MAINT_RESP', itime, &
         resp_maint, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'GROWTH_RESP', itime, &
         resp_growth, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'PLANT_STATUS', itime, &
         plant_status, npts*nvm, horipft_index)
    
    ! Soil, litter and biomass
    CALL histwrite_p (hist_id_stomate, 'DEADLEAF_COVER', itime, &
         deadleaf_cover, npts, hori_index)
    ! CALL histwrite_p (hist_id_stomate, 'TOTAL_SOIL_CARB', itime, &
    !     tot_litter_soil_carb, npts*nvm, horipft_index)
     ! +++CHECK+++
    ! If needed declare in ioipscontrol first
!!$     CALL histwrite_p (hist_id_stomate, 'LIGNIN_STRUC_AB', itime, &
!!$          lignin_struc(:,:,iabove), npts*nvm, horipft_index)
!!$     CALL histwrite_p (hist_id_stomate, 'LIGNIN_STRUC_BE', itime, &
!!$          lignin_struc(:,:,ibelow), npts*nvm, horipft_index)
!!$     CALL histwrite_p (hist_id_stomate, 'LIGNIN_WOOD_AB', itime, &
!!$          lignin_wood(:,:,iabove), npts*nvm, horipft_index)
!!$     CALL histwrite_p (hist_id_stomate, 'LIGNIN_WOOD_BE', itime, &
!!$          lignin_wood(:,:,ibelow), npts*nvm, horipft_index)
     ! +++++++++++
  
!!$     WRITE(numout,*) 'Back in stomate_lpj - 4'
     
     DO l=1,nelements 
        IF     (l == icarbon) THEN 
           element_str(l) = '_c' 
        ELSEIF (l == initrogen) THEN 
           element_str(l) = '_n' 
        ELSE 
           STOP 'Define element_str' 
        ENDIF
       
        ! Soil and litter per element 
        CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AB'//TRIM(element_str(l)), itime, & 
             litter(:,istructural,:,iabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AB'//TRIM(element_str(l)), itime, & 
             litter(:,imetabolic,:,iabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LITTER_STR_BE'//TRIM(element_str(l)), itime, & 
             litter(:,istructural,:,ibelow,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LITTER_MET_BE'//TRIM(element_str(l)), itime, & 
             litter(:,imetabolic,:,ibelow,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_AB'//TRIM(element_str(l)), itime, & 
             litter(:,iwoody,:,iabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_BE'//TRIM(element_str(l)), itime, & 
             litter(:,iwoody,:,ibelow,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SOIL_ACTIVE'//TRIM(element_str(l)), itime, & 
             som(:,iactive,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SOIL_SLOW'//TRIM(element_str(l)), itime, & 
             som(:,islow,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SOIL_PASSIVE'//TRIM(element_str(l)), itime, & 
             som(:,ipassive,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SOIL_SURF'//TRIM(element_str(l)), itime, & 
             som(:,isurface,:,l), npts*nvm, horipft_index) 

        ! Biomass per element  (g C m-2)
        CALL histwrite_p (hist_id_stomate, 'TOTAL_M'//TRIM(element_str(l)), itime, & 
             tot_live_biomass(:,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LEAF_M'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,ileaf,l)*circ_class_n(:,:,:),3),&
             npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SAP_M_AB'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,isapabove,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SAP_M_BE'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,isapbelow,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'HEART_M_AB'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,iheartabove,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'HEART_M_BE'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,iheartbelow,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'ROOT_M'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,iroot,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'FRUIT_M'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,ifruit,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'RESERVE_M'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,icarbres,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LABILE_M'//TRIM(element_str(l)), itime, & 
             SUM(circ_class_biomass(:,:,:,ilabile,l)*circ_class_n(:,:,:),3), npts*nvm, horipft_index) 
        !CALL histwrite_p (hist_id_stomate, 'TOTAL_TURN'//TRIM(element_str(l)), itime, & 
        !     tot_turnover(:,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LEAF_TURN'//TRIM(element_str(l)), itime, & 
             turnover_daily(:,:,ileaf,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SAP_AB_TURN'//TRIM(element_str(l)), itime, & 
             turnover_daily(:,:,isapabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'ROOT_TURN'//TRIM(element_str(l)), itime, & 
             turnover_daily(:,:,iroot,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'FRUIT_TURN'//TRIM(element_str(l)), itime, & 
             turnover_daily(:,:,ifruit,l), npts*nvm, horipft_index) 
        !CALL histwrite_p (hist_id_stomate, 'TOTAL_BM_LITTER'//TRIM(element_str(l)), itime, & 
        !     tot_bm_to_litter(:,:,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LEAF_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,ileaf,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SAP_AB_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,isapabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'SAP_BE_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,isapbelow,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'HEART_AB_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,iheartabove,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'HEART_BE_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,iheartbelow,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'ROOT_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,iroot,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'FRUIT_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,ifruit,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'RESERVE_BM_LITTER'//TRIM(element_str(l)), itime, & 
             bm_to_litter(:,:,icarbres,l), npts*nvm, horipft_index) 
        CALL histwrite_p (hist_id_stomate, 'LABILE_BM_LITTER'//TRIM(element_str(l)), itime, &
             bm_to_litter(:,:,ilabile,l), npts*nvm, horipft_index)

        ! land cover change
        CALL histwrite_p (hist_id_stomate, 'FLUX_PROD_S'//TRIM(element_str(l)), itime, &
             flux_prod_s(:,l), npts, hori_index)
        CALL histwrite_p (hist_id_stomate, 'FLUX_PROD_M'//TRIM(element_str(l)), itime, &
             flux_prod_m(:,l), npts, hori_index)
        CALL histwrite_p (hist_id_stomate, 'FLUX_PROD_L'//TRIM(element_str(l)), itime, &
             flux_prod_l(:,l), npts, hori_index)
        CALL histwrite_p (hist_id_stomate, 'PROD_S'//TRIM(element_str(l)), itime, &
             prod_s(:,:,l), npts*(nshort+1), horip_ss_index)
        CALL histwrite_p (hist_id_stomate, 'PROD_M'//TRIM(element_str(l)), itime, &
             prod_m(:,:,l), npts*(nmedium+1), horip_mm_index)
        CALL histwrite_p (hist_id_stomate, 'PROD_L'//TRIM(element_str(l)), itime, &
             prod_l(:,:,l), npts*(nlong+1), horip_ll_index)
        CALL histwrite_p (hist_id_stomate, 'FLUX_S'//TRIM(element_str(l)), itime, &
             flux_s(:,:,l), npts*nshort, horip_s_index)
        CALL histwrite_p (hist_id_stomate, 'FLUX_M'//TRIM(element_str(l)), itime, &
             flux_m(:,:,l), npts*nmedium, horip_m_index)
        CALL histwrite_p (hist_id_stomate, 'FLUX_L'//TRIM(element_str(l)), itime, &
             flux_l(:,:,l), npts*nlong, horip_l_index)
        
     ENDDO

     ! Wind throw
     CALL histwrite_p (hist_id_stomate, 'WOOD_VPCUT', itime, &
          wood_volume_pix_cut, npts*ncut_times, horicut_index)
     
     ! IPCC
     IF ( hist_id_stomate_IPCC > 0 ) THEN
        vartmp(:)=SUM(tot_live_biomass(:,:,icarbon)*veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cVeg", itime, &
             vartmp, npts, hori_index)
        !vartmp(:)=SUM(tot_litter_carb*veget_cov_max,dim=2)/1e3
        !CALL histwrite_p (hist_id_stomate_IPCC, "cLitter", itime, &
        !     vartmp, npts, hori_index)
        !vartmp(:)=SUM(tot_soil_carb*veget_cov_max,dim=2)/1e3
        !CALL histwrite_p (hist_id_stomate_IPCC, "cSoil", itime, &
        !     vartmp, npts, hori_index)
!!$        vartmp(:)=(prod10_total + prod100_total)/1e3
!!$        CALL histwrite_p (hist_id_stomate_IPCC, "cProduct", itime, &
!!$             vartmp, npts, hori_index)
        !vartmp(:)=carb_mass_variation/1e3/one_day
        !CALL histwrite_p (hist_id_stomate_IPCC, "cMassVariation", itime, &
        !     vartmp, npts, hori_index)
        vartmp(:)=SUM(lai*veget_cov_max,dim=2)
        CALL histwrite_p (hist_id_stomate_IPCC, "lai", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(gpp_daily*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "gpp", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM((resp_maint+resp_growth)*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "ra", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(npp_daily*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "npp", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(resp_hetero*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "rh", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(co2_fire*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "fFire", itime, &
             vartmp, npts, hori_index)
!!$        vartmp(:)=harvest_above/1e3/one_day
!!$        CALL histwrite_p (hist_id_stomate_IPCC, "fHarvest", itime, &
!!$             vartmp, npts, hori_index)
!!$        vartmp(:)=cflux_prod_total/1e3/one_day
!!$        CALL histwrite_p (hist_id_stomate_IPCC, "fLuc", itime, &
!!$             vartmp, npts, hori_index)
!!$        vartmp(:)=(SUM((gpp_daily + atm_to_bm(:,:,icarbon) -(resp_maint+resp_growth+resp_hetero)-co2_fire) &
!!$             & * veget_cov_max,dim=2)-cflux_prod_total-harvest_above)/1e3/one_day
!!$        CALL histwrite_p (hist_id_stomate_IPCC, "nbp", itime, &
!!$             vartmp, npts, hori_index)
        !vartmp(:)=SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*&
        !     veget_cov_max,dim=2)/1e3/one_day
        !CALL histwrite_p (hist_id_stomate_IPCC, "fVegLitter", itime, &
        !     vartmp, npts, hori_index)
        vartmp(:)=SUM(SUM(som_input(:,:,:,icarbon),dim=2)*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "fLitterSoil", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(SUM(circ_class_biomass(:,:,:,ileaf,icarbon)*&
            circ_class_n(:,:,:),3)*veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cLeaf", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(SUM((circ_class_biomass(:,:,:,isapabove,icarbon)+&
            circ_class_biomass(:,:,:,iheartabove,icarbon))*circ_class_n(:,:,:),3)* &
             veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cWood", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(SUM(( circ_class_biomass(:,:,:,iroot,icarbon) + &
            circ_class_biomass(:,:,:,isapbelow,icarbon) + &
            circ_class_biomass(:,:,:,iheartbelow,icarbon) )*circ_class_n(:,:,:),3)*& 
            veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cRoot", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(SUM((circ_class_biomass(:,:,:,icarbres,icarbon) + &
            circ_class_biomass(:,:,:,ilabile,icarbon) + &
            circ_class_biomass(:,:,:,ifruit,icarbon))*circ_class_n(:,:,:),3)*&
            veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cMisc", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM((litter(:,istructural,:,iabove,icarbon)+ &
             litter(:,imetabolic,:,iabove,icarbon)+litter(:,iwoody,:,iabove,icarbon))*&
             veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cLitterAbove", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM((litter(:,istructural,:,ibelow,icarbon)+ &
             litter(:,imetabolic,:,ibelow,icarbon)+litter(:,iwoody,:,ibelow,icarbon))*&
             veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cLitterBelow", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM((som(:,iactive,:,icarbon)+som(:,isurface,:,icarbon))* &
             veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cSoilFast", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(som(:,islow,:,icarbon)*veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cSoilMedium", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(som(:,ipassive,:,icarbon)*veget_cov_max,dim=2)/1e3
        CALL histwrite_p (hist_id_stomate_IPCC, "cSoilSlow", itime, &
             vartmp, npts, hori_index)
        DO j=1,nvm
           histvar(:,j)=veget_cov_max(:,j)*100
        ENDDO
        CALL histwrite_p (hist_id_stomate_IPCC, "landCoverFrac", itime, &
             histvar, npts*nvm, horipft_index)
        !-
        vartmp(:)=zero
        DO j = 2,nvm
           IF (is_deciduous(j)) THEN
              vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100
           ENDIF
        ENDDO
        CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimDec", itime, &
             vartmp, npts, hori_index)
        !-
        vartmp(:)=zero
        DO j = 2,nvm
           IF (is_evergreen(j)) THEN
              vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100
           ENDIF
        ENDDO
        CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimEver", itime, &
             vartmp, npts, hori_index)
        !-
        vartmp(:)=zero
        DO j = 2,nvm
           IF ( .NOT.(is_c4(j)) ) THEN
              vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100
           ENDIF
        ENDDO
        CALL histwrite_p (hist_id_stomate_IPCC, "c3PftFrac", itime, &
             vartmp, npts, hori_index)
        !-
        vartmp(:)=zero
        DO j = 2,nvm
           IF ( is_c4(j) ) THEN
              vartmp(:) = vartmp(:) + veget_cov_max(:,j)*100
           ENDIF
        ENDDO
        CALL histwrite_p (hist_id_stomate_IPCC, "c4PftFrac", itime, &
             vartmp, npts, hori_index)
        !-
        vartmp(:)=SUM(resp_growth*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "rGrowth", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(resp_maint*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "rMaint", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(bm_alloc(:,:,ileaf,icarbon)*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "nppLeaf", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(bm_alloc(:,:,isapabove,icarbon)*veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "nppWood", itime, &
             vartmp, npts, hori_index)
        vartmp(:)=SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*&
             veget_cov_max,dim=2)/1e3/one_day
        CALL histwrite_p (hist_id_stomate_IPCC, "nppRoot", itime, &
             vartmp, npts, hori_index)

        CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_X', itime, &
             resolution(:,1), npts, hori_index)
        CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_Y', itime, &
             resolution(:,2), npts, hori_index)
        CALL histwrite_p (hist_id_stomate_IPCC, 'CONTFRAC', itime, &
             contfrac(:), npts, hori_index)

    ENDIF

    IF (printlev_loc>=3) WRITE (numout,*) 'soil_n_min(test_grid,test_pft,:):', &
         soil_n_min(test_grid,test_pft,:)

    IF (printlev>=3) WRITE(numout,*) 'Leaving stomate_lpj'

  END SUBROUTINE stomate_lpj_vegetation


!! ================================================================================================================================
!! SUBROUTINE   : debug_write
!!
!>\BRIEF        Write a set of variables to the out_orchidee file
!!
!! DESCRIPTION  : Write a set of variables to the out_orchidee file after a routine 
!!                a routine was called.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE debug_write(npts, check_point, circ_class_biomass, &
    circ_class_n, circ_class_kill, plant_status, veget_cov_max)

 !! 0. Variable and parameter description

    !! 0.1 Input variables
     INTEGER(i_std), INTENT(in)                   :: npts               !! Domain size (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)       :: plant_status       !! Growth and phenological status of the plant
                                                                        !! istatus = Phases defined in constantes
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in) :: circ_class_biomass !! Biomass of the componets of the model  
                                                                        !! tree within a circumference
                                                                        !! class @tex $(gC ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in) :: circ_class_kill    !! Number of trees within a circ class that needs
                                                                        !! to be killed @tex $(ind m^{-2})$ @endtex
                                                                        !! IMPORTANT: See the note in constantes.f90
                                                                        !! regarding the indicies for this variable.
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)     :: circ_class_n       !! Number of individuals in each circ class
                                                                        !! @tex $(m^{-2})$ @endtex
    CHARACTER(*),INTENT(in)                       :: check_point        !! A flag to indicate at which
                                                                        !! point in the code we're doing
    REAL(r_std), DIMENSION(:,:), INTENT(in)       :: veget_cov_max          !! "maximal" coverage fraction of a PFT 
                                                                        !! (LAI -> infinity) on ground. May sum to
                                                                        !! less than unity if the pixel has
                                                                        !! nobio area. (unitless; 0-1)

    !! 0.4 Local variables
    INTEGER                                       :: icut, ifm, ipts, ivm
!_ ================================================================================================================================

    WRITE(numout,*) check_point
    WRITE(numout,*) 'test_pft, ', test_pft
    WRITE(numout,*) 'circ_class_n, ', SUM(circ_class_n(test_grid,test_pft,:))
    WRITE(numout,*) 'circ_class_biomass - labile N, ',&
         SUM(circ_class_biomass(test_grid,test_pft,:,ilabile,initrogen)*&
         circ_class_n(test_grid,test_pft,:),1)
    WRITE(numout,*) 'circ_class_biomass - C, ',&
         SUM(SUM(circ_class_biomass(test_grid,test_pft,:,:,icarbon),2)),&
         SUM(SUM(circ_class_biomass(test_grid,test_pft,:,:,icarbon),2)*&
         circ_class_n(test_grid,test_pft,:),1)
    WRITE(numout,*) 'circ_class_biomass - N, ',&
         SUM(SUM(circ_class_biomass(test_grid,test_pft,:,:,initrogen),2)),&
         SUM(SUM(circ_class_biomass(test_grid,test_pft,:,:,initrogen),2)*&
         circ_class_n(test_grid,test_pft,:),1)
    WRITE(numout,*) 'plant_status, ', plant_status(:,test_pft)
!    DO ifm = 1, nfm_types
!       DO icut = 1, ncut_times
!          WRITE(numout,*) 'circ_class_kill, ifm, icut, ', ifm, icut, &
!               circ_class_kill(test_grid,test_pft,:,ifm,icut)
!       ENDDO
!    ENDDO
    WRITE(numout,*) 'veget_cov_max, ', veget_cov_max(:,test_pft), SUM(veget_cov_max(:,:),2)

    DO ipts = 1,npts
       DO ivm = 1,nvm
          IF (veget_cov_max(ipts,ivm) .GT. min_stomate .AND. &
               SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1)) .LT. min_stomate .AND.&
               SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1)) .NE. zero) THEN
             WRITE(numout,*) check_point
             WRITE(numout,*) 'veget_cov_max, ',ivm, veget_cov_max(ipts,ivm)
             WRITE(numout,*) 'biomass -C, ', &
                  SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1))
             WRITE(numout,*) 'biomass -N, ', &
                  SUM(SUM(circ_class_biomass(ipts,ivm,:,:,initrogen),1))
             WRITE(numout,*) 'WARNING - some very small amount of C left'
             CALL ipslerr_p(2,'stomate_lpj','Very small amount of C left',&
                  'Could cause problems elsewhere','')
          ENDIF
       ENDDO
    ENDDO
    

  END SUBROUTINE debug_write

END MODULE stomate_lpj
