! =================================================================================================================================
! MODULE        : stomate_wind
!
! CONTACT	: yiyingchen@gate.sinica.edu.tw
!
! LICENCE      	: IPSL (2006). This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        : Calculates critical wind speed for tree damage for each half-hourly period of the simulation.
!                 This wind speed can be compared to the actual wind speed to decide whether a tree damage occurs.
!!
!!\streamlining_n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN           :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_windthrow.f90 $ 
!! $Date: 2015               -01-22 13:41:13 +0100 (mer., 22 janv. 2014) $
!! $Revision:                   1831 $
!! \streamlining_n
!_ ================================================================================================================================

MODULE stomate_windthrow

  ! modules used:
  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE pft_parameters_var
  USE function_library, ONLY: wood_to_dia, wood_to_height, &
       wood_to_cv_eff, cc_to_lai, cc_to_biomass


  IMPLICIT NONE

  ! private & public routines

  PRIVATE 
  PUBLIC wind_damage
!!$  wind_clear, streamlining, calculate_force, calculate_bending_moment, &
!!$       calculate_cws, calculate_wind_process, elevate

  LOGICAL, SAVE                   :: firstcall = .TRUE.        !! first call 

CONTAINS


!! ================================================================================================================================
!!  SUBROUTINE   : wind_clear 
!!
!>\BRIEF        Set the firstcall flag to .TRUE. and activate initialization
!! 
!_ ================================================================================================================================

  SUBROUTINE wind_clear
    firstcall = .TRUE.
  END SUBROUTINE wind_clear


!! ================================================================================================================================
!! SUBROUTINE     : wind_damage
!!
!>\BRIEF          This subroutine calculates the critical wind speeds for both uprooting and stem breakage
!!                for each half-hourly period of the simulation. The algorithm follows Barry Gardiner's wind 
!!                damage risk model called GALES (Hale et al. 2015).
!! 
!! DESCRIPTION	: The purpose of this module is to describe the actual sensitivity of a given PFT to the wind that is present
!! at the same pixel. For each circumference class in each PFT, two critical wind speed values (CWS) are calculated in every
!! half-hourly time step by this subroutine, following the approach of differentiating two types of wind damage of trees:
!! - uprooting (the whole tree is removed from the soil together with its root plate)
!! - stem breakage (the root plate and the bottom part of the stem remains attached to the ground).
!! Whichever threshold (i.e. CWS) is reached first for a tree, the according damage type occurs in the model.
!!
!! The module completes the following tasks:
!! 1. Calculates...
!! 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): :: Critical wind speed for stem breakage 10 m above the zero-plane displacement (m/s): U_Break_10
!!                             Critical wind speed for tree overturn 10 m above the zero-plane displacement (m/s): U_Overturn_10
!!
!! REFERENCES    :
!! - Hale, S., Gardiner, B., Nicoll, B., Taylor, P., and Pizzirani, S. (2015). Comparison and validation of three versions
!! of a forest wind risk model. Environmental Modelling and Software. In Press.
!! ...
!!
!! FLOWCHART     : None
!! \streamlining_n 
!_ ================================================================================================================================
  
  SUBROUTINE wind_damage (npts, nlevels_tot, circ_class_biomass, &  
       veget_cov_max, circ_class_n, wind_speed_daily, plant_status, &
       resolution, circ_class_kill, harvest_5y_area, forest_managed, &
       soil_temp_daily)
    
!root_dens, senescence, &
    !   circ_class_biomass, circ_class_n, snowdepth, sph, gap_size, snow_depth) 
    ! OUTPUT VARIABLES SHOULD BE LISTED HERE, AS WELL.

  IMPLICIT NONE



 !! 0. Variable declarations

 !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                       :: npts                         !! Domain size @tex $(unitless)$ @endtex
!    INTEGER(i_std), DIMENSION(:), INTENT(in)         :: njsc                         !! Index of the dominant soil textural class in
                                                                                     !! the grid cell @tex $(unitless)$ @endtex
    INTEGER(i_std), INTENT(in)                       :: nlevels_tot                  !! Total level numbers in photosynthesis
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)      :: forest_managed               !! forest management flag (is the forest
                                                                                     !! being managed?)
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)    :: circ_class_biomass           !! Biomass in of an individual tree in each circ class
                                                                                     !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: circ_class_n                 !! Number of trees within each circumference
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget_cov_max                    !! "maximal" coverage fraction of a PFT on the ground
                                                                                     !! May sum to less than unity if the pixel has nobio 
                                                                                     !! area. (unitless, 0-1)
!    REAL(r_std), DIMENSION(:), INTENT(in)            :: snow                         !! Snow mass @tex $ (kg m^{-2})$ @endtex
!    LOGICAL, DIMENSION(:,:), INTENT(in)              :: senescence                   !! The PFT is senescent 
    
! ++++ the senescence is no longer use in CN-CAN and it is replaced by plant_status
     REAL(r_std), DIMENSION(:,:), INTENT(in)          :: plant_status                !! Growth and phenological status of the plant
! +++

!    REAL(r_std), DIMENSION (:,:,:), INTENT(in)       :: z_level_photo               !! Level height of photosynthesis layers (m)
!                                                                                    !! Dimension in (npts,nvm, nlevels_tot)
!    REAL(r_std), DIMENSION (:,:,:,:), INTENT(in)     :: biomass                      !! Biomass @tex $(gC m^{-2})$ @endtex  
!
!    CN-CAN didn't use biomass anymore, so we need to calculate the biomass from circ_class_biomass
 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: resolution                   !! The length in m of each grid-box in X and Y axis
                                                                                     !! Dimension in (npts,2) 1:X and 2:Y
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: harvest_5y_area              !! total harvested area in the last 5 years
                                                                                     !! Dimension(npts,nvm,wind_years)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: soil_temp_daily              !! Daily maximum soil temperature at 0.8 meter below ground (K) 
 ! 0.2 Output variables
    
    ! OUTPUT VARIABLES TO BE DECLARED HERE
    ! We need to output a flag to indicate the damage type which is classified
    ! into no damage or damaged (overturning or branch damage)
 
 
 !! 0.3 Modified variables (INTENT(inout))

    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                         INTENT(inout) :: circ_class_kill        !! Number of trees within a circ that needs
                                                                                 !! to be killed @tex $(ind m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts), INTENT(inout)  :: wind_speed_daily             !! Daily maximum wind speed at 2 meter (ms-1)
!! 0.4 Local variables
    CHARACTER(30)                                :: var_name                     !! To store variable names for I/O
    INTEGER(i_std)                               :: ibdl,ipts,ivm,icir,i,iele    !! Index for the loops 
    INTEGER(i_std), DIMENSION(npts)              :: soil_type                    !! Soil types: 
                                                                                 !!  1. Free-draining mineral soils;
                                                                                 !!  2. Gleyed mineral soils; 
                                                                                 !!  3. Peaty mineral soils; and 
                                                                                 !!  4. Deep Peats
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: virtual_circ_class_n         !! Virtual number of trees within each circumference
    REAL(r_std), DIMENSION(ncirc,60)             :: section_height               !! The height of the bottom of the 1-m sections 
                                                                                 !! of the mean tree @tex $(m)$ @endtex. 60 is the
                                                                                 !! max number of sections thus max tree height is 60m
!    REAL(r_std), DIMENSION(npts,nvm,ncirc,60)    :: section_diam                 !! The diameter of the 1-m sections of the mean  
    REAL(r_std), DIMENSION(ncirc,60)             :: branch_width                 !! Width of canopy) of the 1-m sections of the mean 
                                                                                 !! tree @tex $(m)$ @endtex 60 is the max number of 
                                                                                 !! sections thus max tree height is 60m
    REAL(r_std), DIMENSION(nbdl,nvm)             :: root_dens                    !! Root density per soil layer (what proportion
                                                                                 !! of the root system is found at the given layer)
    REAL(r_std), DIMENSION(nbdl+1)               :: z_soil                       !! Variable to store depth of the different
                                                                                 !! soil layers (m)
    REAL(r_std), DIMENSION(nvm)                  :: rpc                          !! scaling factor for integral
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: mean_dbh                     !! Diameter at breast height (dbh) of the mean tree
                                                                                 !! of the circumference-class @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: mean_height                  !! Height of the mean tree of the given dbh-class 
                                                                                 !! @tex $(m)$ @endtex
!    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: zp_disp_height               !! zero-plane displacement height (m)
!    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: roughness_length             !! roughness_length (m)

    REAL(r_std), DIMENSION(ncirc)                :: lai                          !! Leaf area index @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: canopy_breadth               !! Canopy breadth (width of canopy) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: canopy_depth                 !! Canopy depth (height of canopy) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: canopy_height                !! The height of the starting point of the canopy from 
                                                                                 !! below @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mid_canopy                   !! The middle height of the canopy @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_dbh_height              !! The height of the diameter (by default it is at 
                                                                                 !! breast height = 1.3 m) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: snow_mass                    !! Weight of the snow on the tree @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: diameter_c2                  !! Variable for calculating diameters 
                                                                                 !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: diameter_canopy_base         !! Diameter at the base of the canopy 
                                                                                 !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: diameter_c1                  !! Variable for calculating diameters 
                                                                                 !! @tex $(unitless)$ @endtex
!!$    REAL(r_std), DIMENSION(ncirc)                :: no_of_sections               !! The number of one-meter sections that the mean tree 
!!$                                                                                 !! has @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: crown_depth                  !! Crown depth of the tree in each circumference class 
    REAL(r_std), DIMENSION(ncirc)                :: crown_width                  !! Crown width of the mean tree (m)  
    REAL(r_std), DIMENSION(ncirc)                :: crown_volume                 !! Crown volume of the mean tree @tex $(m^{3})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: crown_mass                   !! Mass of the mean tree crown @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: stem_mass                    !! Mass of the mean tree stem @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: current_spacing              !! Spacing between trees in the given dbh-class 
                                                                                 !! @tex $(m)$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm)          :: rooting_depth                !! Rooting depth of the mean tree (three categories;
                                                                                 !! 1: shallow, 2: deep, 3:average 
                                                                                 !! See constantes_soil_var.f90 for the definitions
!!$    REAL(r_std)                                  :: canopy_density               !! Density of the tree canopy, i.e. of the
!!$                                                                                 !! branches and leaves @tex $(kg/m^{3})$ @endtex
    REAL(r_std)                                  :: streamlining_c               !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                 !! Streamlining is the change of shape of the crowns 
                                                                                 !! due to wind.
    REAL(r_std)                                  :: streamlining_n               !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                 !! Streamlining is the change of shape of the crowns 
                                                                                 !! due to wind.
    REAL(r_std)                                  :: streamlining_rb              !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                 !! Streamlining is the change of shape of the crowns 
                                                                                 !! due to wind.    

    REAL(r_std), DIMENSION(ncirc)                :: area_timber_removals_5_years !! Area of the total timber removal in the
                                                                                 !! previous 5 year @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(npts)                 :: area_icir                    !! The total area of the circumference class @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: tree_heights_from_edge       !! Width of the forest edge area, i.e. area_closer (in number of tree heights)
                                                                                 !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts)                 :: n_gap                        !! The number of gaps with an area of clear_cut_max @tex $(unitless)$ @endtex
    REAL(r_std)                                  :: length_side_gap              !! The length of one side of the square-shaped gap @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_around_gap              !! The framing edge area around the gap with different gustiness
                                                                                 !! @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_total_closer            !! The total area of forest edge areas affected by wind @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_total_further           !! The total area of forest without edge areas @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: ratio_spacing_height         !! Ratio of tree height and tree spacing @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: gap_size                     !! The length (width of the cross-section) of the gap @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: ratio_gap_height             !! Ratio of gap size and tree height @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_gap_factor              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_gap_factor               !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: term_a                       !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: term_b                       !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_closer              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_further             !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_edge                !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_edge_gap            !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_closer              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_further             !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge                !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge_gap_closer     !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge_gap_further    !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_gust_factor_closer   !! Mean bending moment of the forest edge area (for calculating
                                                                                 !! the edge factor) @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_gust_factor_further  !! Mean bending moment of the inner forest area (for calculating
                                                                                 !! the edge factor) @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: edge_factor                  !! The ratio of the mean bending moment in the forest edge area 
                                                                                 !! and mean bending moment in the inner forest area
                                                                                 !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: gust_factor_closer           !! Gust factor for calculating the critical wind speed of the forest
                                                                                 !! edge area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: gust_factor_further          !! Gust factor for calculating the critical wind speed of the inner
                                                                                 !! forest area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: g_closer                     !! Gustiness of the forest edge area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: g_further                    !! Gustiness of the inner forest area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)             :: overturning_moment_multiplier !! Overturning moment multiplier representing the effects of species,
                                                                                  !! soil type, soil depths and senescence @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: max_overturning_moment       !! The maximum overturning moment the mean tree can withstand
                                                                                 !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: max_breaking_moment          !! The maximum breaking moment the mean tree can withstand
                                                                                 !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: overturning_moment_closer    !! The maximum overturning moment the mean tree can withstand in the
                                                                                 !! forest edge area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: overturning_moment_further   !! The maximum overturning moment the mean tree can withstand in the
                                                                                 !! inner forest area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: breaking_moment_closer       !! The maximum breaking moment the mean tree can withstand in the
                                                                                 !! forest edge area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: breaking_moment_further      !! The maximum breaking moment the mean tree can withstand in the
                                                                                 !! inner forest area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_break_closer             !! Critical wind speed for stem breakage in the forest edge area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_break_closer_10          !! Critical wind speed for stem breakage in the forest edge area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_overturn_closer          !! Critical wind speed for overturning in the forest edge area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_overturn_closer_10       !! Critical wind speed for overturning in the forest edge area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_final_closer             !! The critical wind speed of the final form of damage (breakage / uprooting)
                                                                                 !! in the forest edge area @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: d_h_ratio                    !! Array for storing the stand spacing and tree height ratio for output (-)
    REAL(r_std), DIMENSION(npts)                 :: wind_speed_actual            !! The actual wind speed over the pixel
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    LOGICAL                                      :: wind_damage_type_uprooting_closer   !! The final form of damage (breakage / uprooting) in the forest edge area
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: wind_damage_rate_closer      !! Damage rate (%) in the forest edge area in case of damage
                                                                                 !! @tex $(0-1)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_break_further            !! Critical wind speed for stem breakage in the inner forest area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_break_further_10         !! Critical wind speed for stem breakage in the inner forest area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_overturn_further         !! Critical wind speed for overturning in the inner forest area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_overturn_further_10      !! Critical wind speed for overturning in the inner forest area
                                                                                 !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: cws_final_further            !! The critical wind speed of the final form of damage (breakage / uprooting)
                                                                                 !! in the inner forest area @tex $(m/s; mean hourly wind speed)$ @endtex
    LOGICAL                                      :: wind_damage_type_uprooting_further  !! The final form of damage (breakage / uprooting) in the inner forest area
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: wind_damage_rate_further     !! Damage rate (%) in the forest edge area in case
                                                                                 !! of damage in the inner forest area @tex $(0-1)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: z0_wind                      !! surface roughness calculated from the subroutine streamlining()
    REAL(r_std), DIMENSION(npts,nvm,ncirc)       :: d_wind                       !! zero plane displacement height calculated from the subroutine streamlining() 
    INTEGER(i_std), DIMENSION(npts,nvm,ncirc)    :: wind_damage_type_closer      !! Integer store the wind damage type for closer area  ibreakage=1, ioverturning=2
    INTEGER(i_std), DIMENSION(npts,nvm,ncirc)    :: wind_damage_type_further     !! Integer store the wind damage type for further area
    INTEGER(i_std)                               :: printlev_loc                 !! Integer for print out for windthrow module   
    REAL(r_std)                                  :: crown_top                    !! Temporal variable for storing the tallest tree crown height in different PFTs  
    REAL(r_std)                                  :: crown_bottom                 !! Temporal variable for storing the smallest tree crown height in different PFTs 
    REAL(r_std)                                  :: distance_from_force          !! The distance from the force was set to 1.3 for breakage and 0.0 for overturning
    REAL(r_std), DIMENSION(ncirc)                :: closer_area_fraction         !! Closer area fraction 
    REAL(r_std), DIMENSION(ncirc)                :: further_area_fraction        !! Further area fraction 
  
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements) :: biomass  
               
! Temp output Variables for GALES model intercomparision
   REAL(r_std), DIMENSION(npts,nvm,ncirc)        :: canopy_depth_out
   REAL(r_std), DIMENSION(npts,nvm,ncirc)        :: tree_h_edge_out
   REAL(r_std), DIMENSION(npts,nvm,ncirc)        :: mean_gap_f_out
   REAL(r_std), DIMENSION(npts,nvm,ncirc)        :: gap_size_out
   REAL(r_std), DIMENSION(npts,nvm,ncirc)        :: edge_factor_out  
  
!_ ============================================================================================================================================================================

   IF (printlev>=2) WRITE(numout,*) 'Entering stomate_windthrow.f90'

   ! 1.1 Initialize local printlev
   !printlev_loc=get_printlev('windthrow')
  
   printlev_loc=5 
!! 0. Initialized the values of variables
       wind_damage_rate_closer(:,:,:) = 0.0d0
       wind_damage_rate_further(:,:,:) = 0.0d0
       cws_final_further(:,:,:) = 0.0d0 
       cws_final_closer(:,:,:) = 0.0d0
       d_h_ratio(:,:,:) = 0.0d0

    !-Debug-
    IF (printlev_loc .GT. 4) THEN
      DO ipts = 1, npts
         WRITE(numout,*) 'Daily maximum windspeed passed from stomate to stomate_winthrow:', wind_speed_daily(ipts)
         WRITE(numout,*) 'Daily maximum soil temperature at 80cm below ground:', soil_temp_daily(ipts)
    !   WRITE(numout,*) 'Initial wind damage rate closer, ', wind_damage_rate_closer(:,:,:)
    !   WRITE(numout,*) 'Initial wind damage rate futther, ',wind_damage_rate_further(:,:,:)
      ENDDO
    ENDIF
    !-
       
 !! 1. Getting the input variables in the required form for the actual sub-routine 
       
    ! The code is specific for CRU-NCEP. If a different forcing
    ! is used, the Max Wind Ratio parameters should be adjusted
    ! CALL ipslerr_p(2,'Wind throw module assumes that CRU-NCEP forcing is used',&
    !   '','','')
   
 ! ++++++
 ! calculate the total biomass  

      
    biomass(:,:,:,:) = cc_to_biomass(npts,nvm,circ_class_biomass,circ_class_n)



    ! Looping over pixels
    DO ipts = 1, npts 
    
       !! 1.1 Convert 6-hr CRU-NCEP into half-hourly wind speed     
       ! The max_wind_ratio parameters were determined by relating fluxnet
       ! half-hourly data to 6h CRU-NCEP data. For this reason the
       ! parameters should be considered specific to the CRU-NCEP forcing.
       ! The max_wind_ratio parameters were also found to depend on the 
       ! spatial aggregation, hence, they are specific to the 0.5 degree
       ! CRU-NCEP data (see Chen et al for details). 

       ! The following relationship to convert CRU-NCEP into half-hourly
       ! wind speeds was found:
       ! +++CHANGE+++
       ! externalise parameters as max_wind_ratio        
       !wind_speed_daily(ipts) = wind_speed_daily(ipts) - &
       !    0.077* wind_speed_daily(ipts)**2. + &
       !    0.015 * wind_speed_daily(ipts)**3.
       ! ++++++++++++ 
       wind_speed_daily(ipts) = -5.922*wind_speed_daily(ipts)  &
           + 2.051*wind_speed_daily(ipts)**2.  &
           - 0.191*wind_speed_daily(ipts)**3. &
           + 0.006*wind_speed_daily(ipts)**4.
 
       ! In case this convertion did not result in the observed wind
       ! damage, we kept a linear tuning parameter. Ideally, no tuning
       ! should be applied and therefore daily_max_tune should be 1.0
!       wind_speed_daily(ipts) = wind_speed_daily(ipts) * daily_max_tune 

       wind_speed_daily(ipts) = wind_speed_daily(ipts) 

     

       !! 1.2 Calculate root profile 
!       z_soil(1) = 0.
!       z_soil(2:nbdl+1) = diaglev(1:nbdl)

       ! Scaling factor for integration
!       rpc(:) = un / ( un - EXP( -z_soil(nbdl) / humcste(:) ) )

       ! Calculate root density per layer per pft
       ! Loop over # soil layers
!       DO ibdl = 1, nbdl

!          root_dens(nbdl-ibdl,:) = rpc(:) * &
!               ( EXP( -z_soil(ibdl)/humcste(:)) - &
!               EXP( -z_soil(ibdl+1)/humcste(:)) )

!       ENDDO ! Loop over # soil layers
 
       !! 1.3 Defining the soil type
       ! This sub-routine uses 4 types of soil for calculating the 
       ! critical wind speed, so 12 different soil types present in 
       ! other modules of ORCHIDEE need to be grouped into these
       ! 4 generic types.

       ! +++CHECK+++
       ! For the moment only 3 soil types are distinguished.
       ! Needs to be checked with all 12 USDA soil types. 
       ! If more soil type are used it will become possible
       ! to make use of the different soils distinguished in
       ! the windthrow module. The code could then look as
       ! follow:
!        CASE (x:y)
!          soil_type(ipts) = ifreedrainage  
!        CASE (x:y)
!          soil_type(ipts) = igleyed
!        CASE (x:y)
!          soil_type(ipts) = ipeaty
!       CASE (x:y)
!          soil_type(ipts) = ipeat 
 
!      SELECT CASE (njsc(ipts))
!       CASE (1:3) 
          ! For the moment all soils are assumed to be freely draining
          soil_type(ipts) = ifree_draining

!       CASE DEFAULT
!          CALL ipslerr_p(3, 'stomate_windthrow.f90',&
!               'soil type is not defined','','')
!       END SELECT
!       !++++++++++++
      
       ! Looping over PFT-s
       DO ivm = 2, nvm

          ! Don't do the calculations if there is no vegetation
          ! or the vegetation is bare soil, grass or crop
          IF ((veget_cov_max(ipts,ivm).LT.min_stomate) .OR. .NOT.is_tree(ivm)) THEN
             CYCLE
          ENDIF

          !! 1.4 Calculating the rooting depth
          !  If root density (proportion of root mass) is higher 
          !  than 3% in the 11th soil layer (1.024-2.056 m) below 
          !  the soil surface, then we treat rooting depth as 
          !  "deep". Otherwise, it's "shallow". Note that the 
          !  threshold of 3% is arbitrary and it might be needed to
          !  justify at some point in the future.
          
          ! +++CHANGE+++
          ! solution: change nbdl in the index for the layer that is 
          ! closest to 2 meters.
          ! It was hard coded (index = 10). We don't like hard coded
          ! even if it is unlikely that the 11-layers will change
          ! any time soon. The depth of the 10th layer depends on
          ! the total depth of the soil (dpu_max). Would be better
          ! to use ::dpu_max rather than assuming the 2 m.
          ! Also, there is new scheme being tested where nbdl may 
          ! have changed. CHECK when using the new scheme.
          IF (root_dens(nbdl-1,ivm) > 0.10) THEN

             rooting_depth(ipts,ivm) = ideep

          ELSE

             rooting_depth(ipts,ivm) = ishallow

          END IF
          ! +++++++++++
       !--Debug        
          IF (printlev_loc >= 4) THEN
              WRITE(numout,*) 'rooting depth ', rooting_depth(ipts,ivm)
          ENDIF 
 

          !!  1.5 Set crown parameters
          !   To set crown parameters leaf mass needs to be available
          IF (SUM(circ_class_biomass(ipts,ivm,:,ileaf,icarbon)) .GT. min_stomate) THEN
  
             ! There is a canopy
             streamlining_c = streamlining_c_leaf(ivm)
             streamlining_n = streamlining_n_leaf(ivm)
             streamlining_rb = streamlining_rb_leaf(ivm)
          ELSE

             ! If the tree is leafless, the following parameters 
             ! become different. The weight of leaves will be absent.
             ! Streamlining changes as the surface of leaves is absent.
             streamlining_c = streamlining_c_leafless(ivm)
             streamlining_n = streamlining_n_leafless(ivm)
             streamlining_rb = streamlining_rb_leafless(ivm)

          END IF

          !-Debug-
          IF(printlev_loc .GT. 4)THEN
             WRITE(numout,*) 'Initialisation of critical windspeed'
             WRITE(numout,*) 'ipts, ivm, ',ipts, ivm
             WRITE(numout,*) 'humcste, ', humcste
             WRITE(numout,*) 'rooting depth(nbdl+1), ', z_soil
             WRITE(numout,*) 'root_dens(nbdl,ivm), ',root_dens(:,ivm)
             WRITE(numout,*) 'soil_type(ipts), ',soil_type(ipts)
             WRITE(numout,*) 'rooting_depth_type,(1:deep, 2:shallow,3:average), ',&
                  rooting_depth(ipts,ivm)
             WRITE(numout,*) 'streamlining_c, ', streamlining_c
             WRITE(numout,*) 'streamlining_n, ', streamlining_n
             WRITE(numout,*) 'streamlining_rb, ', streamlining_rb
          ENDIF
          !-

          !! 1.6 Calculate stand characteristics
          !  Calculating diameter at breast height (dbh) in m of the
          !  mean tree of all circ classes.
          mean_dbh(ipts,ivm,:) = wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

          ! Calculating the height in m of the mean tree of the given 
          ! dbh-class using
          mean_height(ipts,ivm,:) = wood_to_height(&
               circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)
          
          ! Crown_volume is calculated as the woody biomass of an individual tree 
          crown_volume(:) = wood_to_cv_eff( circ_class_biomass(ipts,ivm,:,:,icarbon), ivm)

          ! Crown_width is calculated as(crown_vloume * 6.0/pi)^(1/3)
          crown_width(:) =  ((crown_volume(:)*6.0)/pi)**(1.0/3.0)      

          ! ORCHIDEE assumes that the crowns are spherical. Hence, width
          ! breath and depth are all the same. Set canopy_breadth as the 
          ! crown_width for all circumference classes          
          canopy_breadth(:) = crown_width(:) 
          canopy_depth(:) = crown_width(:)  
          canopy_height(:) = mean_height(ipts,ivm,:)

          !-Debug-
          IF (printlev_loc .GT. 4) THEN
              WRITE(numout,*) 'crown_volumn, ', crown_volume(:)
              WRITE(numout,*) 'crown_width, ', crown_width(:)
              WRITE(numout,*) 'crown_depth, ', canopy_depth(:)
              WRITE(numout,*) 'canopy_height, ', mean_height(:,:,:) 
          ENDIF   
          !-
          ! loop over the circ class 
          DO icir = 1, ncirc

             !! 1.6 Calculating the current tree spacing in the PFT
             ! The current tree spacing between tree stems are calculated 
             ! from the number of stems per hectare. circ_class_n is the 
             ! number of stems per m2; it's converted to stems per hectare 
             ! with m2_to_ha which has a value of 10000.
             ! The virtual tree spacing assumes that the LAI remains
             ! constant and the calculates how many trees of a given 
             ! circumference class would be needed to result in the set LAI.
             ! Because lai changes throughout the season because of leaf
             ! fall, wood mass was used instead. Because LAI is related to 
             ! wood mass (see allocation). The principle remains the same 

! this part of code need to revised for CN-CAN for using circ

          !-Debug-
          IF (printlev_loc .GT. 4) THEN
              WRITE(numout,*) 'biomass, isapabove, ', biomass(ipts,ivm,isapabove,icarbon)
              WRITE(numout,*) 'biomass, iheartabove, ', biomass(ipts,ivm,iheartabove,icarbon)

              WRITE(numout,*) 'circ_class_biomass, isapabove, ', &
                               circ_class_biomass(ipts,ivm,icir,isapabove,icarbon)
              WRITE(numout,*) 'circ_class_biomass, iheatabove, ', &
                               circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon) 
          ENDIF   
 

             IF (  (circ_class_biomass(ipts,ivm,icir,isapabove,icarbon) + &
                circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon)) >= zero ) THEN 
             

             virtual_circ_class_n(ipts,ivm,icir) =  &
               (biomass(ipts,ivm,isapabove,icarbon) + &
                biomass(ipts,ivm,iheartabove,icarbon)) / &
               (circ_class_biomass(ipts,ivm,icir,isapabove,icarbon) + &
                circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon))
             ELSE 

              virtual_circ_class_n(ipts,ivm,icir) = zero 
 
             ENDIF
             




             ! Now the spacing is calculated based on virtual tree numbers ( virtual_circ_class_n)      
             IF (virtual_circ_class_n(ipts,ivm,icir) .GT. min_stomate) THEN
                ! The original equation is current_spacing(icir) = 100.0 / &
                ! SQRT(m2_to_ha * virtual_circ_class_n(ipts,ivm,icir))
                ! given that m2_to_ha = 10000 it reduces to:
                current_spacing(icir) = 1.0 / SQRT(virtual_circ_class_n(ipts,ivm,icir))
             ELSE
                ! In principle, we should never enter this condition. The
                ! spacing between 2 trees equals the size of the pixel. It is 
                ! thus empty. We defined this case such that if we do end here,
                ! a very large cws will be calculated and the code should not
                ! suffer from divide by zero.
                current_spacing(icir) = (resolution(ipts,1) + resolution(ipts,2) )/2.0
                WRITE(numout,*) 'WARNING from WINDFALL module:'
                WRITE(numout,*) 'the current spacing set to pixel spacing:', current_spacing(icir)
                CALL ipslerr_p(2,'Wind throw module',&
                   'current spacing was set to pixel dimensions',&
                   'it is therefore assumed to be empty','') 
             ENDIF

             !! 1.7 Calculate green stem mass
             ! Stem mass is calculated as the aboveground woody biomass
             ! minus the branches. circ_class_biomass is the biomass of
             ! an individual tree (gC/tree)
             ! Here, we convert the unit to (kgC/tree) by dividing kilo_to_unit (1000)
             stem_mass(ipts,ivm,icir) = (circ_class_biomass(ipts,ivm,icir,isapabove,icarbon) + &
                 circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon)) * &
                 (un - branch_ratio(ivm)) / kilo_to_unit
          WRITE(numout,*) 'stem_mass,',stem_mass(ipts,ivm,icir)
          WRITE(numout,*) 'green_density,', green_density(ivm)
          WRITE(numout,*) 'pipe_sensity,', pipe_density(ivm)
          WRITE(numout,*) 'kilo to unit,', kilo_to_unit
          WRITE(numout,*) 'virtual_circ_class', virtual_circ_class_n(ipts,ivm,icir)
          WRITE(numout,*) 'sla,',sla(ivm)
             ! Wind throw occurs on living trees so the stem mass should be
             ! converted from dry biomass in gC to wet biomass in kg to be used
             ! in the GALES equations. Stem_mass thus converted as follows:
             ! (green_density / (pipe_density/1000)) from dry wood mass
             ! (kgC/tree) to wet wood mass (Kg/trees)
             stem_mass(ipts,ivm,icir) = stem_mass(ipts, ivm,icir) * &
                 green_density(ivm)/(pipe_density(ivm)/kilo_to_unit)
   
             !! 1.8 Calculating leaf area index 
             ! We need to use the virtual biomass in each circumference class
             ! to calculate lai, and tree_heights_from_edge was calculated as
             ! function of lai.
             lai(icir) = circ_class_biomass(ipts,ivm,icir,ileaf,icarbon)*&
                virtual_circ_class_n(ipts,ivm,icir) * sla(ivm)

!++++
! CNCAN cann't find the variable " green_density "
!++++

             !-Debug-
             IF (printlev_loc .GT. 4) THEN           
                WRITE(numout,*) 'check spacing and circ_number in a diameter class'
                WRITE(numout,*) 'virtual_tree_numbers:', &
                     virtual_circ_class_n(ipts,ivm,icir), ' for circ class:', icir
                WRITE(numout,*) 'current spacing: ',&
                     current_spacing(icir),'mean diameter: ',  mean_dbh(ipts,ivm,icir) 
            !    WRITE(numout,*) 'PFT No.:',ivm,'green_density (Kg/m^3):', &
            !         green_density(ivm) 
            !    WRITE(numout,*) 'dry_density (Kg/m^3):', pipe_density(ivm)/kilo_to_unit
                WRITE(numout,*) 'resolution_x:', resolution(ipts,1)
                WRITE(numout,*) 'resolution_y:', resolution(ipts,2)
                WRITE(numout,*) 'ipts, ',ipts,'ivm, ',ivm,'icir, ',icir
                WRITE(numout,*) 'mean dbh, ', mean_dbh(ipts,ivm,icir)
                WRITE(numout,*) 'mean height, ', mean_height(ipts,ivm,icir)
                WRITE(numout,*) 'stem_mass, ', stem_mass(ipts,ivm,icir)
                WRITE(numout,*) 'crown_volume, ',crown_volume(icir)
                WRITE(numout,*) 'crown_mass, ',crown_mass(icir)
                WRITE(numout,*) 'lai, ',lai(icir)
                WRITE(numout,*) 'virtual_circ_n, ', virtual_circ_class_n(ipts,ivm,icir) 
                WRITE(numout,*) 'real_circ_n, ', circ_class_n(ipts,ivm,icir) 
                WRITE(numout,*) 'current_spacing, ',current_spacing(icir)
                WRITE(numout,*) 'End of Section 1:  Calculated canopy structure &
                     & in difffernt circumferences for each PFT' 
             ENDIF
             !

             !! 2. Calculating the gustiness of wind in the PFTs

             !! 2.1 Calculate gaps and the surrounding areas 
             ! IMPORTANT: AS WE DO NOT HAVE INFORMATION ON STAND EDGES IN THE 
             ! ORCHIDEE SIMULATIONS, A POSSIBLE PROXY TO USE HERE IS A VARIABLE 
             ! THAT REPRESENTS EARLIER HARVESTS IN THE PREVIOUS 5 YEARS. AFTER 
             ! SUCH A PERIOD, THE FOREST EDGE IS ASSUMED TO GET ACCLIMATIZED
             ! TO THE NEW CIRCUMSTANCES, AND IT'S VULNERABILITY DIMINISHES 
             ! (see Persson, 1975; Valinger and Fridman, 2011)
             ! There is a big increase in gustiness after a certain distance from 
             ! the edge of the forest, hence we divide the area of the PFT into two 
             ! areas:
             ! - the area which is further than X tree heights from any edge that 
             ! was created in the previous five years (area_total_further)
             ! - the area which is closer than X tree heights from any edge that 
             ! was created in the previous five years (area_total_closer).
             ! Of course, these smaller areas depend on the fragmentation of the 
             ! gaps (created in the last five years). As we have no information 
             ! on this, we assume a fixed gap size representing the most likely 
             ! size of clear cuts (clear_cut_max) and that the  
             ! gaps are square-shaped. Then, we divide the area of harvest in the 
             ! previous five years (area_timber_removals_5_years) by clear_cut_max. 
             ! Hence, from the number of gaps we can calculate the area_total_closer. 
             ! As each circumference class will have its forest area divided, we  
             ! calculate two gustiness values and hence, two critical wind speeds.
             ! The value of X depends on the leaf area index of the PFT and is 
             ! calculated as follows:
             ! tree_heights_from_edge is defined as "X/h", X sets to 28/LAI and h
             ! is set to meant_tree_height.
             ! This comes from the equation of calculating canopy penetration depth
             ! in Pan et al. 2017?. Ying Pan work on the large eddy simulation and 
             ! is now at Pennsilvania State University   

             ! Here, lai(icir) for each diameter class is calculated by virtual_class_n 
             ! a threshold "0.1" is used to avoid the issue of divided by small lai < 0.1 
             ! for the cases of deciduous tree species 
             IF (lai(icir) .GT. 0.1) THEN
                tree_heights_from_edge(icir) = (28.0/lai(icir)) / mean_height(ipts,ivm,icir) 
             ELSE
                tree_heights_from_edge(icir) = (28.0/0.1) / mean_height(ipts,ivm,icir) 
             ENDIF
         
             ! As suggested by Barry (the father of GALES), the tree height from edge 
             ! should be limited to 9.0     
             IF (tree_heights_from_edge(icir) .GT. 9.0)  tree_heights_from_edge(icir) = 9.0
     

             ! Calculating the total area of timber removals in the previous five years:
             area_timber_removals_5_years(icir) = SUM(harvest_5y_area(ipts,ivm,1:wind_years)) 
             area_icir(ipts) = resolution(ipts,1)*resolution(ipts,2) !* veget_cov_max(ipts,ivm) 
 
             ! Number of gaps with the area of the maximum allowed clearfelling:
             n_gap  = INT(area_timber_removals_5_years(icir)) / clear_cut_max 
         
             ! The length of the sides of the squared-shaped gaps:
             length_side_gap = SQRT(clear_cut_max)
         
             ! The frame-shaped area around the clear cut gap with X as the width 
             ! of the frame:
             area_around_gap(icir) = ((length_side_gap + &
                tree_heights_from_edge(icir)*mean_height(ipts,ivm,icir)*2.0)**2.0) - &
                clear_cut_max

             ! Total area of the "frames" in the particular dbh-class of the particula 
             ! PFT:the multiplication by 0.25 is because of the assumption that wind 
             ! direction does not change during the half-hourly time step, therefore 
             ! only one of the four sides of the gaps are affected.
             area_total_closer(icir) = n_gap(ipts) * area_around_gap(icir) * 0.25


! add a if condiction to check the area_total_closer <= area_icir


             ! The total area which is nor gap neither "frame":
             area_total_further(icir) = area_icir(ipts) - area_total_closer(icir) - &
                (n_gap(icir) * clear_cut_max)

             ! Get the area fraction
             closer_area_fraction(icir) = area_total_closer(icir) / area_icir(ipts)
             further_area_fraction(icir) = area_total_further(icir) / area_icir(ipts)  
 

             !-Debug-
             IF(printlev_loc .GT. 4)THEN 
                 WRITE(numout,*) 'Section 2.0: Calculating the gustiness of wind &
                      & in the PFT and area fractions in the grid.'
                 WRITE(numout,*) 'area_closer_fraction, ', closer_area_fraction(icir)
                 WRITE(numout,*) 'area_further_fraction, ', further_area_fraction(icir)
             ENDIF
             !-

             !! 2.2 Calculate ratio to spacing height 
             ! The ratio of spacing to height describes the functional (for
             ! wind) stand density. If there are many tall trees, the spacing 
             ! is low and thus ratio_spacing_height will be low as well.  
             ! Values of the ratio of mean tree spacing and height are limited
             ! between 0.075 and 0.45. Bigger values don't make any difference,
             ! smaller values are not really possible in real life.
             ! The D/h  value are presented in Hale et al., 2015 
             IF (current_spacing(icir) / mean_height(ipts,ivm,icir) < 0.075) THEN
                ! Many tall trees, ratio has saturated
                ratio_spacing_height(icir) = 0.075
             ELSE
                IF (current_spacing(icir) / mean_height(ipts,ivm,icir) > 0.45) THEN
                   ! Few short trees, ratio has saturated
                   ratio_spacing_height(icir) = 0.45
                ELSE
                   ! Calculate ratio_spacing_height  
                   ratio_spacing_height(icir) = current_spacing(icir) / &
                      mean_height(ipts,ivm,icir)
                END IF
             ENDIF

             ! Copy the ratio_spacing_height to the D_H_ratio for output 
             d_h_ratio(ipts,ivm,icir) = ratio_spacing_height(icir) 
         
             !! 2.3 Calculate functional gap size
             ! Calculate whether the gap is large enough to increase
             ! the sensitivity of the remaining trees to wind throw.
             ! GALES uses the length of the edge of the square-shaped
             ! gaps calculated in the previous section.
             gap_size(icir) = length_side_gap  
 
             ! If the gap is wider than 10 times the tree height,
             ! then wind loading on the stand edge does not increase any 
             ! more. The threshold of 10 is used in the equations below
             IF (gap_size(icir) > 10.0 * mean_height(ipts,ivm,icir)) THEN
                mean_gap_factor(icir) = 1.
                max_gap_factor(icir) = 1. 
             ELSE
                ratio_gap_height(icir) = gap_size(icir) / &
                    mean_height(ipts,ivm,icir)
                mean_gap_factor(icir) = (0.001 * ratio_gap_height(icir)**0.562) / &
                    (0.001 * 10.**0.562)
                max_gap_factor(icir) = (0.0064 * ratio_gap_height(icir)**0.3467) / &
                    (0.0064 * 10.**0.3467)
             END IF
         
             ! In the following section, there are three types of terms for forest edges:
             ! the edge (edge), the area around the edge (closer)  and the area further 
             ! away from the edge (further). Only closer and further are used in
             ! the calculation of cws. Gustiness is different in both areas. 

             ! Calculating term_a and term_b for the new gust method. The definition of
             ! the term_a and term_b are in the Hale  et al. (2015) eq(7) 
             ! D/h is the ratio_spacing_height
             ! x/h is the tree_heights_from_edge 
             IF (-2.1 * ratio_spacing_height(icir) + 0.91 < zero) THEN
                 term_a(icir) = zero 
             ELSE
                 term_a(icir) = -2.1 * ratio_spacing_height(icir) + 0.91
             END IF
             term_b(icir) = 1.0611 * LOG(ratio_spacing_height(icir)) + 4.2
        
             ! New gust factors:
             ! The gustiness away from the edge is always higher than at the edge.
             ! We applied term_a*tree_heights_from_edge  + term_b for gustiness calculation.
             ! The gustiness away from the edge is always higher than at the edge.
             ! For the further area, tree_height_from_edge is always greater than 9.0
             ! thus, the above calculation will produce an even higher value for gustiness
             ! calculation. So, We set tree_heights_from_edge as 9.0.
             ! The gustiness at the further area will than be higher then the gustiness at 
             ! the closer area.        
             new_gust_closer(icir)  = term_a(icir) * tree_heights_from_edge(icir) + term_b(icir)
             new_gust_further(icir) = term_a(icir) * 9.0  + term_b(icir)
             new_gust_edge(icir)    = term_a(icir) * zero + term_b(icir)
       
             ! Calculating gustiness:
             g_closer(ipts,ivm,icir) = new_gust_closer(icir)
             g_further(ipts,ivm,icir) = new_gust_further(icir)

        
             ! The following section is used to calculate the edge_factor. This section calculates
             ! how the mean loading on a tree changes as a function of tree height, spacing, distance
             ! from edge and size of any upwind gap. An upwind gap greater than 10 tree heights
             ! is assumed to be an infinite gap. The output (edge_factor) is used to modify the
             ! calculated wind loading on the tree, which assumes a steady wind and a position well
             ! inside the forest.
             mean_bm_gust_factor_closer(icir) = (0.68 * ratio_spacing_height(icir) - &
                0.0385) + (-0.68 * ratio_spacing_height(icir) + 0.4785) * ((1.7239 * &
                ratio_spacing_height(icir) + 0.0316)**tree_heights_from_edge(icir)) * &
                mean_gap_factor(icir)
         
             mean_bm_gust_factor_further(icir) = (0.68 * ratio_spacing_height(icir) - &
                0.0385) + (-0.68 * ratio_spacing_height(icir) + 0.4785) * ((1.7239 * &
                ratio_spacing_height(icir) + 0.0316)**9.0) * mean_gap_factor(icir)
         
             
             edge_factor(icir) = mean_bm_gust_factor_closer(icir) / mean_bm_gust_factor_further(icir)
             ! Copy array for output 
             edge_factor_out(ipts,ivm,icir)  = edge_factor(icir)
             ! Calculating gust_factor accounting for gustiness and edge_factor:
             gust_factor_closer(ipts,ivm,icir) = g_closer(ipts,ivm,icir) * edge_factor(icir) 
           
             ! Suggested by Barry
             ! For further we force edge_factor = 1. This is because everything is
             ! compared to wind load on tree in the middle of the forest
             edge_factor(icir) = 1.0 
             gust_factor_further(ipts,ivm,icir) = g_further(ipts,ivm,icir) *  edge_factor(icir) 
         
         
             !-Debug-
             IF(printlev_loc .GT. 4)THEN 
                 WRITE(numout,*) 'End of Section 2.0: Calculating the &
                      & gustiness of wind in the PFT'
                 WRITE(numout,*) 'term_a, ', term_a(icir)
                 WRITE(numout,*) 'term_b, ', term_b(icir)
                 WRITE(numout,*) 'new_gust_closer, ', new_gust_closer(icir)
                 WRITE(numout,*) 'ratio_spacing_height, ', ratio_spacing_height(:) 
                 WRITE(numout,*) 'new_gust_further, ', new_gust_further(icir)
                 WRITE(numout,*) 'new_gust_edge, ', new_gust_edge(icir) 
                 WRITE(numout,*) 'tree_heights_from_edge, ', tree_heights_from_edge(icir)
                 WRITE(numout,*) 'Correction factor for the forest &
                      & edge dimension in number of circumference'
                 WRITE(numout,*) 'edge_factor, ', edge_factor(icir)
                 WRITE(numout,*) 'gustness_closer, ', g_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'gustness_futher, ', g_further(ipts,ivm,icir) 
                 WRITE(numout,*) 'gust_factor_closer, ' , gust_factor_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'gust_factor_further, ', gust_factor_further(ipts,ivm,icir)
                 WRITE(numout,*) 'Edge factor:', edge_factor(icir)
                 WRITE(numout,*) 'Tree height from edge:', tree_heights_from_edge(icir)
                 WRITE(numout,*) 'Mean bm_gust_factor_closer:',  &
                      mean_bm_gust_factor_closer(icir)
                 WRITE(numout,*) 'Mean bm_gust_factor_further:', &
                      mean_bm_gust_factor_further(icir)
                 WRITE(numout,*) 'Mean gap_factor:', mean_gap_factor(icir)
             ENDIF
             !-
                  
             !! 3. Calculating critical moments (tree mechanics section)
             ! overturning_moment_multiplier: this comes form the species parameters file and is related to the
             ! depth and type of the soil.
             ! Rooting depth has three categories: Shallow (less then 0.8 m deep); Deep (deeper than 0.8 m);
             ! Average (average values to be used when rooting depth is unknown).
             ! Soil type has four categories: Free-draining mineral soils; Gleyed mineral soils; Peaty mineral
             ! soils; Deep peats
             ! As these are generic soil types, a modified soil map of Europe would be needed for the optimal use
             ! of WINDFALL.

! ++++ This part of code need to revised for changing senescence to plant_status 
 
         
!              IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_shallow(ivm)
!              ELSE IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_shallow_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_deep(ivm)
!              ELSE IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_deep_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_average(ivm)
!              ELSE IF (soil_type(ipts) == ifree_draining .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_average_leafless(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_shallow(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_shallow_leafless(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_deep(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_deep_leafless(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_average(ivm)
!              ELSE IF (soil_type(ipts) == igleyed .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_average_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_shallow(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_shallow_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_deep(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_deep_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_average(ivm)
!              ELSE IF (soil_type(ipts) == ipeaty .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peaty_average_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_shallow(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == ishallow &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_shallow_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_deep(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == ideep &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_deep_leafless(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. .NOT. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_average(ivm)
!              ELSE IF (soil_type(ipts) == ipeat .AND. rooting_depth(ipts,ivm) == iaverage &
!                 .AND. senescence(ipts,ivm)) THEN
!                 overturning_moment_multiplier(ipts,ivm) = overturning_peat_average_leafless(ivm)
!              ELSE
                 !average value if parameters are missing
                 overturning_moment_multiplier(ipts,ivm) = 125.0
!              END IF

!  +++++ just apply a constant value 125.0  for testing


   
              ! Maximum moment calculated for tree overturning.
              max_overturning_moment(ipts,ivm,icir) = overturning_moment_multiplier(ipts,ivm) * &
                 stem_mass(ipts,ivm,icir)
           
              ! Maximum moment calculated for stem breakage.
              max_breaking_moment(ipts,ivm,icir) = modulus_rupture(ivm) * f_knot(ivm) * pi * &
                 (mean_dbh(ipts,ivm,icir)**3.0) / 32.0
           
              ! This is where the gustiness of wind is accounted for so that half hourly mean 
              ! wind speeds can be used in the simulations.
              ! In the end, moments and hence, critical wind speeds are calculated
              ! for both areas (closer & further) with different gustiness.
              overturning_moment_closer(ipts,ivm,icir) = max_overturning_moment(ipts,ivm,icir) / &
                  gust_factor_closer(ipts,ivm,icir)
              overturning_moment_further(ipts,ivm,icir) = max_overturning_moment(ipts,ivm,icir) / &
                  gust_factor_further(ipts,ivm,icir)
              breaking_moment_closer(ipts,ivm,icir) = max_breaking_moment(ipts,ivm,icir) / &
                  gust_factor_closer(ipts,ivm,icir)
              breaking_moment_further(ipts,ivm,icir) = max_breaking_moment(ipts,ivm,icir) / &
                  gust_factor_further(ipts,ivm,icir)
      
              !-Debug-
              IF(printlev_loc .GT. 4)THEN
                 WRITE(numout,*) 'PFT type, ', ivm 
                 WRITE(numout,*) 'POINT id, ', ipts
                 WRITE(numout,*) 'SENSCENCE type,', plant_status(ipts,ivm)
                 WRITE(numout,*) 'Overturning multiplier for free drainage average root leafless all PFTs,', &  
                                  overturning_free_draining_average_leafless(:)
                 WRITE(numout,*) 'Overturning multiplier for free drainage average root leaf all PFTs,', &  
                                  overturning_free_draining_average(:)
                 WRITE(numout,*) 'Overturning multiplier for free drainage shallow root leafless all PFTs,', &  
                                  overturning_free_draining_shallow_leafless(:)
                 WRITE(numout,*) 'Overturning multiplier for free drainage shallow root leaf all PFTs,', &  
                                  overturning_free_draining_shallow(:)
                 WRITE(numout,*) 'Overturning multiplier for free drainage deep root leafless all PFTs,', &  
                                  overturning_free_draining_deep_leafless(:)
                 WRITE(numout,*) 'Overturning multiplier for free drainage deep root leaf all PFTs,', &  
                                  overturning_free_draining_deep(:)
 
                 WRITE(numout,*) 'End of Section 3 : Calculating critical moments (tree mechanics section)'
                 WRITE(numout,*) 'Diameter class:', icir
                 WRITE(numout,*) 'Overturning moment multiplier, ', &
                                  overturning_moment_multiplier(ipts,ivm)
                 WRITE(numout,*) 'maximum moment for overturning, ', max_overturning_moment(ipts,ivm,icir)
                 WRITE(numout,*) 'maximum moment for stem breakag, ', max_breaking_moment(ipts,ivm,icir)
                 WRITE(numout,*) 'overturning closer,', overturning_moment_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'breaking closer, ', breaking_moment_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'overturning further, ', overturning_moment_further(ipts,ivm,icir)
                 WRITE(numout,*) 'breaking further, ', breaking_moment_further(ipts,ivm,icir)
              ENDIF
              !-
      
              !! 4. Calculating critical wind speed and damage rate

              !! 4.1 Calculating critical wind speed in the forest and 10 m above the zero-plane displacement
           
              ! +++CHECK+++
              ! Ideally ORCHIDEE should use only one  surface roughness (z0) and displacement 
              ! height. This is not straightforward because here surface roughness and 
              ! displacement height are calculated as a function of wind speed to account for 
              ! streamlining of the canopy under strong winds. This is done in the subroutine 
              ! streamlining(...) in the calculate_wind_process(). Nevertheless, the subroutine 
              ! streamlining uses the canopy structure from ORCHIDEE and when storm damage
              ! occurs, the canopy structure is adjusted and this new structure will be used 
              ! in condveg.f90 . The inconsistency is thus between the roughness length 
              ! calculations in condveg.f90 and stomate_windthrow.f90. 
              ! +++++++++++
      
              ! for the edge area
              ! Critical wind speed for stem breakage in the edge area (m/s)
              distance_from_force = 0.0 
              call calculate_wind_process(breaking_moment_closer(ipts,ivm,icir),    &
                      current_spacing(icir), canopy_depth(icir), &
                      canopy_breadth(icir), canopy_height(icir), &
                      streamlining_n, streamlining_c, &                      
                      cws_break_closer(ipts,ivm,icir),          &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir),&
                      distance_from_force) 
              
              ! Critical wind speed for stem breakage 10 m above 
              ! the zero-plane displacement in the edge area(m/s)
              call elevate(cws_break_closer(ipts,ivm,icir), & 
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir), canopy_height(icir), &
                      cws_break_closer_10(ipts,ivm,icir))
       
              ! Critical wind speed for tree overturning in the edge area (m/s)
              distance_from_force = 0.0
              call calculate_wind_process(overturning_moment_closer(ipts,ivm,icir),  &
                      current_spacing(icir), canopy_depth(icir), &
                      canopy_breadth(icir), canopy_height(icir), &
                      streamlining_n, streamlining_c, &
                      cws_overturn_closer(ipts,ivm,icir),        &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir),&
                      distance_from_force)
             
              ! Critical wind speed for tree overturning 10 m above
              ! the zero-plane displacement in the edge area (m/s)
              call elevate(cws_overturn_closer(ipts,ivm,icir), &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir), canopy_height(icir), &
                      cws_overturn_closer_10(ipts,ivm,icir))
         
              ! IF the soil at 80 below ground is frozen, the module only allows the stem breakage
              IF ( soil_temp_daily(ipts) .GT. 273.15 ) THEN 
              ! Calculating damage severity in case any of the critical wind speeds is reached 
              ! in the area around gaps (forest edge area). Whichever of the two damage types 
              ! (overturning, stem breakage) has the lowest critical wind speed, it will be the 
              ! occurring one.
                 IF (cws_overturn_closer_10(ipts,ivm,icir) < cws_break_closer_10(ipts,ivm,icir)) THEN
                    cws_final_closer(ipts,ivm,icir) = cws_overturn_closer_10(ipts,ivm,icir)
                    wind_damage_type_closer(ipts,ivm,icir) = ioverturning
                 ELSE
                    cws_final_closer(ipts,ivm,icir) = cws_break_closer_10(ipts,ivm,icir)
                    wind_damage_type_closer(ipts,ivm,icir) = ibreakage
                 END IF
              ELSE
              ! when the soil at 80 cm  is frozen, the only damage type is limited to the
              ! stem_breakage
                    cws_final_closer(ipts,ivm,icir) = cws_break_closer_10(ipts,ivm,icir)
                    wind_damage_type_closer(ipts,ivm,icir) = ibreakage           
              ENDIF     
 
              ! Based on observations of wind damage in Scotland and how much we had to
              ! adjust the critical wind speed to get agreement between the wind speed
              ! above the canopy, the calculated critical wind speed and the presence
              ! of damage (Hale et al, 2015). I am suggesting the relationship below to
              ! calculate the percentage of damage at a grid point as a function of the
              ! meteorological wind speed at 10m above the zero-plane displacement
              ! (pred_wind), and the critical wind speed for damage (crit_wind)
              ! calculated by ForestGALES. I would set maximum to 80% for the moment
              ! (sets the maximum total amount of damage = 80%) and the scaling factor
              ! (s) to 0.8 (this affects the rate at which damage percentage increases
              ! with increasing wind speed). We can adjust these if the seem to be
              ! wrong or I find other data that I can use to make a better assessment.
              ! We apply a sigmodial function to tune the "damage level" by using a
              ! function which is dependent on the difference between the actual wind 
              ! speed and the final_cws (the lower one of the cws from overturning or 
              ! stem_breakage. 
              wind_damage_rate_closer(ipts,ivm,icir) = max_damage_closer(ivm) * ( &
                  (1./(1.+exp(-((wind_speed_daily(ipts)- cws_final_closer(ipts,ivm,icir)) / &
                  sfactor_closer(ivm))))) - &
                  (1./(1.+exp(cws_final_closer(ipts,ivm,icir)/sfactor_closer(ivm)))) ) 
           
              !-Debug-
              IF (printlev_loc .GT. 4) THEN 
                 WRITE(numout,*) 'max_damage_closer:', max_damage_closer(ivm)
                 WRITE(numout,*) 'sfactor_closer:', sfactor_closer(ivm)
                 WRITE(numout,*) 'daily_max_wind_speed (m/s):', wind_speed_daily(ipts)
                 WRITE(numout,*) 'cws_closer (m/s)', cws_final_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'wind_damage_rate:', wind_damage_rate_closer(ipts,ivm,icir)
              ENDIF
              !-
  
              !! 4.2 Calculating critical wind speed in the forest and 10 m above the 
              !! zero-plane displacement for the further forest area
      
              ! Whichever of the two damage types (overturning, stem breakage) has the 
              ! lowest critical wind speed, it will be the occurring one.
              ! Critical wind speed for stem breakage in the inner forest area (m/s)
              distance_from_force = 0.0 
              call calculate_wind_process(breaking_moment_further(ipts,ivm,icir), &
                      current_spacing(icir), canopy_depth(icir), &
                      canopy_breadth(icir), canopy_height(icir), &
                      streamlining_n, streamlining_c, &
                      cws_break_further(ipts,ivm,icir), &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir),&
                      distance_from_force)
           
              ! Critical wind speed for stem breakage 10 m above the zero-plane 
              ! displacement in the inner forest area (m/s)
              call elevate(cws_break_further(ipts,ivm,icir), &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir), canopy_height(icir), &
                      cws_break_further_10(ipts,ivm,icir))
             
              ! Critical wind speed for tree overturning in the inner forest area (m/s)
              distance_from_force = 0.0
              call calculate_wind_process(overturning_moment_further(ipts,ivm,icir),&
                      current_spacing(icir), canopy_depth(icir), &
                      canopy_breadth(icir), canopy_height(icir), &
                      streamlining_n, streamlining_c, &
                      cws_overturn_further(ipts,ivm,icir),&
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir),&
                      distance_from_force)
    
              ! Critical wind speed for tree overturning 10 m above the zero-plane 
              ! displacement in the inner forest area (m/s)
              call elevate(cws_overturn_further(ipts,ivm,icir), &
                      z0_wind(ipts,ivm,icir), d_wind(ipts,ivm,icir), canopy_height(icir), &
                      cws_overturn_further_10(ipts,ivm,icir))
       
              ! Whichever of the two damage types (overturning, stem breakage) has the 
              ! lowest critical wind speed, it will be the occurring one.
              IF (cws_overturn_further_10(ipts,ivm,icir) < cws_break_further_10(ipts,ivm,icir)) THEN
                  cws_final_further(ipts,ivm,icir) = cws_overturn_further_10(ipts,ivm,icir)
                  wind_damage_type_further(ipts,ivm,icir) = ioverturning
              ELSE
                  cws_final_further(ipts,ivm,icir) = cws_break_further_10(ipts,ivm,icir)
                  wind_damage_type_further(ipts,ivm,icir) = ibreakage
              END IF
      
              ! Wind damage level is considered as a logistic function with sigmoidal
              ! shape, which is calculated as function of a critical wind speed, an actual
              ! wind speed, a scaling factor(s)=0.8 and a maximum damage rate(80%)=0.8.
              wind_damage_rate_further(ipts,ivm,icir) = max_damage_further(ivm) * ( &
                    (1./(1.+exp(-((wind_speed_daily(ipts)- cws_final_further(ipts,ivm,icir)) / &
                    sfactor_further(ivm))))) - &
                    (1./(1.+exp(cws_final_further(ipts,ivm,icir)/sfactor_further(ivm)))) ) 
      
              !-Debug-
              IF (printlev_loc .GT. 4) THEN 
                 WRITE(numout,*) 'max_damage_further:', max_damage_further(ivm)
                 WRITE(numout,*) 'sfactor_further:', sfactor_further(ivm)
                 WRITE(numout,*) 'daily_max_wind_speed (m/s):', wind_speed_daily(ipts)
                 WRITE(numout,*) 'cws_further (m/s)', cws_final_further(ipts,ivm,icir)
                 WRITE(numout,*) 'wind_damage_rate:', wind_damage_rate_further(ipts,ivm,icir)  
              ENDIF
              !-
      
              !! 5. Determine which trees are killed by wind damage

              ! We only do the damage for the virtual_circ_class_n GT min_stomate        
              IF (virtual_circ_class_n(ipts,ivm,icir) .GT. min_stomate) THEN
  
                  !! 5.1. Kill trees in closer area  
                  IF (wind_speed_daily(ipts) >= cws_final_closer(ipts,ivm,icir) ) THEN
                      IF (wind_damage_type_closer(ipts,ivm,icir) == ibreakage) THEN
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = &
                         wind_damage_rate_closer(ipts,ivm,icir) * circ_class_n(ipts,ivm,icir) * &
                         closer_area_fraction(icir)
               
                         ! We assume that there is Only one damage type will occur due to
                         ! its lower limiting wind speed
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = zero

                      ELSE

                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = &
                             wind_damage_rate_closer(ipts,ivm,icir) * &
                             circ_class_n(ipts,ivm,icir) *  closer_area_fraction(icir)

                         ! We assume that there is Only one damage type will occur due to
                         ! its lower limited wind speed
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = zero

                      ENDIF    
                  ENDIF 

                  !! 5.2 Kill trees in further area  
                  IF (wind_speed_daily(ipts) >= cws_final_further(ipts,ivm,icir) ) THEN
                      IF (wind_damage_type_further(ipts,ivm,icir) == ibreakage) THEN
                         ! Total damage = closer + further  
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = &
                             circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) + & 
                             wind_damage_rate_further(ipts,ivm,icir) * &
                             circ_class_n(ipts,ivm,icir) * further_area_fraction(icir)

                         ! We assume that there is only one damage type will occur due to
                         ! its lower limited wind speed
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = zero

                      ELSE

                         
                         ! Total damage = closer + further
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = &
                             circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) + &
                             wind_damage_rate_further(ipts,ivm,icir) * circ_class_n(ipts,ivm,icir) * &
                             further_area_fraction(icir)

                         ! We assume that there is only one damage type will occur due to
                         ! its lower limited wind speed
                         circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = zero
         
                      ENDIF    
                  ENDIF 
  
              ELSE

                  ! set no damage for the no vegetation area 
                  circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = zero
                  circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = zero  
      
              END IF
      
              ! For the tree height below five meters (7.0m), we assumed that the damage will not
              ! occur. At the moment, we don't have good solution for the calculation
              ! of CWS for the small trees, so we just simply keep the small trees. 
              IF ( mean_height(ipts,ivm,icir) .LT. 7.0 ) THEN
                   circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) = zero
                   circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) = zero  
              ENDIF 
  
              !-Debug-
              IF (printlev_loc .GT. 4 .AND. ipts==test_grid .AND. ivm==test_pft) THEN
                 WRITE(numout,*) 'End of Section 4: Calculating critical wind speed and damage rate'
                 WRITE(numout,*) 'critical wind speed further, ', cws_final_further(ipts,ivm,icir)
                 WRITE(numout,*) 'critical wind speed closer, ',  cws_final_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'wind damage type further, ', wind_damage_type_further(ipts,ivm,icir)
                 WRITE(numout,*) 'wind damage type closer, ',  wind_damage_type_closer(ipts,ivm,icir)
                 WRITE(numout,*) 'forest managed type, ', forest_managed(ipts,ivm) 
                 WRITE(numout,*) 'circ_class_n(ipts,ivm,icir),',  circ_class_n(ipts,ivm,icir) 
                 WRITE(numout,*) 'circ_class_kill(ipts,ivm,icir,:,icut_storm_break,', &
                    circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_break) 
                 WRITE(numout,*) 'circ_class_kill(ipts,ivm,icir,:,icut_storm_uproot,', &
                    circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),icut_storm_uproot) 
                 WRITE(numout,*) 'circ_class_kill, for icut_1:9_:'
                 WRITE(numout,*) circ_class_kill(ipts,ivm,icir,forest_managed(ipts,ivm),1:9)
              ENDIF
              !-
      
              ! Copy some values for the output files
              canopy_depth_out(ipts,ivm,icir) = canopy_depth(icir)
              tree_h_edge_out(ipts,ivm,icir)  = tree_heights_from_edge(icir)
              mean_gap_f_out(ipts,ivm,icir)   = mean_gap_factor(icir) 
              gap_size_out(ipts,ivm,icir)     = gap_size(icir)  
              !edge_factor_out(ipts,ivm,icir)  = edge_factor(icir)
      
           END DO ! loop ncirc

        END DO ! loop nvm

     END DO ! loop npts
   
     !! 5. Writing out history variables

!++++ SET lev 5  history files 
     IF ( printlev_loc .GT. 5 ) THEN
!+++
     ! Write the CWS to stomate_history_file
     DO icir = 1,ncirc
        !CWS at further area for each diameter class  
        WRITE(var_name,'(A,I3.3)') 'CWS_FURTHER_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_final_further(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at closer area for each diameter class
        WRITE(var_name,'(A,I3.3)') 'CWS_CLOSER_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_final_closer(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at closer area in each diameter class for stem breakage
        WRITE(var_name,'(A,I3.3)') 'CWS_CLOSER_BK_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_break_closer_10(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at closer area in each diameter class for overturning
        WRITE(var_name,'(A,I3.3)') 'CWS_CLOSER_OV_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_overturn_closer_10(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at further  area in each diameter class for stem break
        WRITE(var_name,'(A,I3.3)') 'CWS_FURTHER_BK_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_break_further_10(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at further area in each diameter class for overturning
        WRITE(var_name,'(A,I3.3)') 'CWS_FURTHER_OV_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_overturn_further_10(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at closer area in each diameter class for stem breakage
        WRITE(var_name,'(A,I3.3)') 'CWS_TOP_CLO_BK_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_break_closer(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at closer area in each diameter class for overturning
        WRITE(var_name,'(A,I3.3)') 'CWS_TOP_CLO_OV_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_overturn_closer(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at further  area in each diameter class for stem break
        WRITE(var_name,'(A,I3.3)') 'CWS_TOP_FUR_BK_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_break_further(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !CWS at further area in each diameter class for overturning
        WRITE(var_name,'(A,I3.3)') 'CWS_TOP_FUR_OV_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             cws_overturn_further(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        !D_H_ratio  for each diameter class
        WRITE(var_name,'(A,I3.3)') 'D_H_RATIO_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             d_h_ratio(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! MEAN_HEIGHT for each diameter class
        WRITE(var_name,'(A,I3.3)') 'MEAN_HEIGHT_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             mean_height(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! MEAN_DBH for each diameter class
        WRITE(var_name,'(A,I3.3)') 'MEAN_DBH_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             mean_dbh(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! V_IND virtual individual tree numbers for each diameter class
        WRITE(var_name,'(A,I3.3)') 'V_IND_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             virtual_circ_class_n(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! G_CLOSER gustiness for closer area in each diameter class
        WRITE(var_name,'(A,I3.3)') 'G_CLOSER_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             gust_factor_closer(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! G_FURTHER gustiness for further area in  each diameter class
        WRITE(var_name,'(A,I3.3)') 'G_FURTHER_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             gust_factor_further(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! G_FURTHER gustiness for further area in  each diameter class
        WRITE(var_name,'(A,I3.3)') 'STEM_MASS_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             stem_mass(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! roughness length
        WRITE(var_name,'(A,I3.3)') 'WIND_Z0_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             z0_wind(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! displacement height
        WRITE(var_name,'(A,I3.3)') 'WIND_D_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             d_wind(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! canopy depth
        WRITE(var_name,'(A,I3.3)') 'CANOPY_DEPTH_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             canopy_depth_out(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! tree heights from edge 
        WRITE(var_name,'(A,I3.3)') 'TREE_H_EDGE_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             tree_h_edge_out(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! mean gap factor
        WRITE(var_name,'(A,I3.3)') 'MEAN_GAP_F_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             mean_gap_f_out(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! gap size
        WRITE(var_name,'(A,I3.3)') 'GAP_SIZE_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             gap_size_out(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! edge factor
        WRITE(var_name,'(A,I3.3)') 'EDGE_FACTOR_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             edge_factor_out(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! maximum moment for overturning
        WRITE(var_name,'(A,I3.3)') 'MAX_OV_MOMENT_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             max_overturning_moment(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     DO icir =1,ncirc        
        ! maximum moment breakage 
        WRITE(var_name,'(A,I3.3)') 'MAX_BK_MOMENT_',icir
        CALL histwrite_p(hist_id_stomate, var_name, itime, &
             max_breaking_moment(:,:,icir), npts*nvm, horipft_index)
     ENDDO
     !
     ! Daily Maximum wind speed
     CALL histwrite_p(hist_id_stomate, 'DAILY_MAX_WIND', itime, &
          wind_speed_daily, npts, hori_index)
ENDIF! 
 
END SUBROUTINE wind_damage


  !! =============================================================================================
  !! SUBROUTINE      : calculate_wind_process
  !!
  !>\BRIEF           : This subroutine calculates the breaking and overturning critical wind
  !!                   speeds
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : 
  !!
  !! REFERENCES      :
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
  !_ =============================================================================================
     
    SUBROUTINE calculate_wind_process(critical_moment,&
               a_current_spacing, a_canopy_depth, a_canopy_breadth, a_canopy_height, &
               a_streamlining_n, a_streamlining_c, &
               guess_wind_speed_sub, surf_rough, zp_displ, a_distance_from_force)
  
    IMPLICIT NONE
  
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
      REAL, INTENT(in)  :: a_current_spacing
      REAL, INTENT(in)  :: a_canopy_depth
      REAL, INTENT(in)  :: a_canopy_breadth 
      REAL, INTENT(in)  :: a_canopy_height 
      REAL, INTENT(in)  :: critical_moment
      REAL, INTENT(in)  :: a_streamlining_n
      REAL, INTENT(in)  :: a_streamlining_c
      REAL, INTENT(in)  :: a_distance_from_force
    
      !! 0.2 Output variables
  
      REAL, INTENT(out) :: guess_wind_speed_sub       !! A guess for the
                                                      !! critical wind speed
      REAL, INTENT(out) :: surf_rough
      REAL, INTENT(out) :: zp_displ                       
      !! 0.3 Modified variables
  
      !! 0.4 Local variables
      INTEGER           :: loop_count                 !! Counting the iteration
      INTEGER           :: max_count=20               !! Max iteration times set to 20 iterations              
      REAL              :: delta                      !! Difference between the guessed
                                                      !! and the calculated wind speed
      REAL              :: wind_precision             !! How close we should
                                                      !! get with the iterations
      REAL              :: lambda                     !!
      REAL              :: lambda_capital 
      REAL              :: gamma_solved
      REAL              :: psih
      REAL              :: critical_moment_cws
      
      !! 0.5 Variables used in  calculate_force subroutine and  calculate_bending_moment subroutine                  
      REAL              :: gammaSolved
      REAL              :: d
      REAL              :: force_on_tree
      REAL              :: height_of_force
      REAL              :: bending_moment  
      REAL              :: porosity
      REAL              :: crit_wind_speed
      
!_ =============================================================================================
  
  !! 1. Subroutine
  
  ! This bit belongs to the original form of doing the iterations. For more info, see the notes below.
  !  guess_wind_speed_sub = 64.0
  !  delta = guess_wind_speed_sub / 2.0
 
     guess_wind_speed_sub = 25.0
     delta = guess_wind_speed_sub / 2.0 
     wind_precision = 0.01
     loop_count = 0
     DO WHILE (delta > wind_precision)
        CALL streamlining(guess_wind_speed_sub, &
             a_current_spacing, a_canopy_breadth, a_canopy_depth, a_canopy_height, & 
             a_streamlining_n, a_streamlining_c, &
             porosity, lambda, lambda_capital, gamma_solved, &
             zp_displ, psih, surf_rough)
        ! The input variable of the subroutine is passed towards the call of another subroutine:
        critical_moment_cws = critical_moment
  
        CALL calculate_cws(critical_moment_cws, zp_displ, &
             gamma_solved, a_current_spacing, &
             crit_wind_speed, a_distance_from_force)

        ! We use the absolute error as the criteria for stopping the iteration
        ! loop    
        delta = abs( guess_wind_speed_sub - crit_wind_speed ) 
  
        guess_wind_speed_sub = crit_wind_speed
        
        loop_count = loop_count  +1      
        ! Add the maximum iteration times condition for CWS calculation
        IF (loop_count .GE. max_count) THEN
            WRITE(numout,*) 'WARANNING from stomate_windthrow.f90: The maximum iteration times for CWS claculation!!'  
            EXIT
        ENDIF
    END DO
     
    END SUBROUTINE calculate_wind_process
  
  
   
  
  !! ===========================================================================================
  !! SUBROUTINE      : streamlining
  !!
  !>\BRIEF           : This subroutine calculates the streamlining of tree crowns. Streamlining is
  !!                   the change of shape of the crowns due to wind.
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : Surface roughness and zero-plane displacement among other variables that
  !!                   are used in the other subroutines
  !! REFERENCES      :
  !!                   Raupach (1994) Simplified expressions for vegetation roughness length and
  !!                   zero-plane displacement as functions of canopy height and area index, Boundary-Layer Meteorology 
  !!                   October 1994, Volume 71, Issue 1, pp 211-216, doi: 10.1007/BF00709229
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
  !_ ============================================================================================
     
    SUBROUTINE streamlining(a_guess_wind_speed, a_current_spacing, &
               a_canopy_breadth, a_canopy_depth, a_mean_height, &
               a_streamlining_n, a_streamlining_c, &
               porosity_sub, lambda_sub, lambda_capital_sub, &
               gamma_solved_sub, zp_displ_sub, psih_sub, surf_rough_sub)
  
    IMPLICIT NONE
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
  
      REAL,  INTENT(in)  :: a_guess_wind_speed  !! the guess wind speed (m s{-1})
      REAL,  INTENT(in)  :: a_current_spacing   !! the average spacing between trees (m)
      REAL,  INTENT(in)  :: a_canopy_breadth    !! the maximum width of the canopy   (m)
      REAL,  INTENT(in)  :: a_canopy_depth      !! the length of the live tree crown (m)
      REAL,  INTENT(in)  :: a_mean_height       !! the mean canopy height (m) 
      REAL,  INTENT(in)  :: a_streamlining_n    !! streamlining parameter n (-) see Eq. A12 in Hale et al 2015
      REAL,  INTENT(in)  :: a_streamlining_c    !! streamlining parameter c (-) see Eq. A12 in Hale et al 2015  
 
      !! 0.2 Output variables
  
      REAL, INTENT(out)  :: porosity_sub       !! the effective drag ?? (-)
                                               !! see equation A12 in the supplementary material of Hale et al (2015)
      REAL, INTENT(out)  :: lambda_sub         !! roughness density, the frontal
                                               !! area of roughness elements
      REAL, INTENT(out)  :: lambda_capital_sub !! canopy area index 
      REAL, INTENT(out)  :: gamma_solved_sub   !! a function for solving surface roughness (-)   
      REAL, INTENT(out)  :: zp_displ_sub       !! zero plane of displacement height (m)
      REAL, INTENT(out)  :: psih_sub           !! stability correction function for heat transfer (-)  
      REAL, INTENT(out)  :: surf_rough_sub     !! surface roughness (m)
  
      !! 0.3 Modified variables
  
      !! 0.4 Local variables
!      REAL :: current_spacing                 !! the average spacing between trees (m)
!      REAL :: canopy_breadth                  !! the maximum width of the canopy   (m)
!      REAL :: canopy_depth                    !! the length of the live tree crown (m)

      !! The aerodynamic parameter values used here are based on the reference of Raupach (1994)
      !! --- TEMP ----
      !! Need to be discussed for the consistency in the ORCHIDEE
      !! ------------         
      REAL, PARAMETER :: c_displacement = 7.5   
      REAL, PARAMETER :: c_surface = 0.003             
      REAL, PARAMETER :: c_drag = 0.3               
      REAL, PARAMETER :: c_roughness = 2.0      
      REAL, PARAMETER :: lambda_capital_max = 0.6
      REAL, PARAMETER :: wind_streamlining_max = 25.0 
      REAL, PARAMETER :: wind_streamlining_min = 10.0
   !_ =============================================================================================
  
  !! 1. Subroutine
  
     IF (a_guess_wind_speed > wind_streamlining_max) THEN
     ! Above 25 m/s, tree crowns cannot streamline any further.
     ! The porosity set to its minimum value 2.35*25^-0.51 ~ 0.46
        porosity_sub = a_streamlining_c * wind_streamlining_max**(-a_streamlining_n)            
        ELSE IF ( a_guess_wind_speed < wind_streamlining_min ) THEN
        ! Below 10 m/s, tree crowns do not streamline and porosity set to its
        ! maximum value 2.35*10^-0.51 ~ 0.73
        porosity_sub = a_streamlining_c * wind_streamlining_min**(-a_streamlining_n)            
        ELSE
        ! Wind speed between 10 m/s and 25 m/s,  tree crown do the streamlining 
        porosity_sub = a_streamlining_c * a_guess_wind_speed**(-a_streamlining_n)
     END IF
 
     ! lambda_sub = (a_canopy_breadth/2.0) * a_canopy_depth *  porosity_sub / a_current_spacing **2.0
     ! Here, we directly calculated the mean width of canopy from the value of "a_canopy_breadth"  
     ! The shape of tree crown is not rhomboid anymore. In the ORCHIDEE-CAN
     ! the shape of tree crown is circle, so the frontal area should be
     ! calculated as (pi/4 * a_canopy_breadth * a_canopy_depth). 
     lambda_sub = a_canopy_breadth * a_canopy_depth *  porosity_sub / a_current_spacing **2.0
     
     lambda_capital_sub = lambda_sub

     IF (lambda_capital_sub > lambda_capital_max) THEN
        ! Gamma is needed for the calculation of the roughness length.
        gamma_solved_sub = 1.0 / ((c_surface + c_drag * lambda_capital_max / 2.0)**0.5)
        ELSE
        gamma_solved_sub = 1.0 / ((c_surface + c_drag * lambda_capital_sub / 2.0)**0.5)
     END IF
  
     ! Calculation of the zero-plane displacement: 
     ! see equations from A7 to A11 in the supplementary material of Hale et al. (2015)  
     zp_displ_sub = a_mean_height * (1.0 - ((1.0 - EXP(-(c_displacement * &
     lambda_capital_sub)**0.5)) / (c_displacement * lambda_capital_sub)**0.5))
      
     ! Calculation of the aerodynamic roughness length:
     ! psih = "Profile influence function" at h (height of the roughness elements).
     psih_sub = LOG(c_roughness) -1.0 + c_roughness**(-1.0)

     surf_rough_sub = (a_mean_height - zp_displ_sub) * EXP(psih_sub - ct_karman * gamma_solved_sub)

  !!++++++++++++++++++
 
    END SUBROUTINE streamlining

  
  
  
  !! ============================================================================================
  !! SUBROUTINE      : calculate_force
  !!
  !>\BRIEF           : This subroutine calculates the force affecting the tree.
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : 
  !!
  !! REFERENCES      :
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
  !_ ============================================================================================
     
    SUBROUTINE calculate_force(a_guess_wind_speed, a_gamma_solved, &
                               a_zp_displ, a_current_spacing, a_mean_height, & 
                               force_on_tree_sub, height_of_force_sub)
  
    IMPLICIT NONE
  
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
  
      REAL, INTENT(in)  :: a_guess_wind_speed
      REAL, INTENT(in)  :: a_gamma_solved
      REAL, INTENT(in)  :: a_zp_displ
      REAL, INTENT(in)  :: a_current_spacing
      REAL, INTENT(in)  :: a_mean_height
      !! 0.2 Output variables
  
      REAL, INTENT(out) :: force_on_tree_sub
      REAL, INTENT(out) :: height_of_force_sub
  
      !! 0.3 Modified variables
  
      !! 0.4 Local variables
      REAL,PARAMETER    :: air_density = 1.2226 !!This should call from constant.f90 (kg m^{-3}) 
  !_ =============================================================================================
  
  !! 1. Subroutine
  
     force_on_tree_sub = air_density * (a_guess_wind_speed * &
                                        a_current_spacing / a_gamma_solved)**2.0
!! --- TEMP ---
!!   I checked with the reference the force of wind can be calculated as 
!!   air_density*ustar^{2}*D^{2} as the gamma is defined as  U/ustar , D is the current spacing
!!   the force of wind should be as 
!!   force_on_tree_sub = air_density * (U/gamma)^{2} * D^{2}
!! ------------

     height_of_force_sub = a_zp_displ
  
     ! The maximum of the zero-plane displacement height is locked at 0.8 tree height.
     ! IF (height_of_force_sub > 0.8 * a_mean_height) THEN
     !    height_of_force_sub = 0.8 * a_mean_height
     ! END IF
 
    END subroutine calculate_force
  
  
  !! =============================================================================================
  !! SUBROUTINE      : calculate_bending_moment
  !!
  !>\BRIEF           : Subroutine to calculate the bending moment
  !!                   (to compare with critical moments in the iterative calculation of the critical
  !!                   wind speeds)
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : 
  !!
  !! REFERENCES      :
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
  !_ =============================================================================================
     
    SUBROUTINE calculate_bending_moment (a_force_on_tree, a_height_of_force, bending_moment_sub)
  
    IMPLICIT NONE
  
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
  
      REAL, INTENT(in)  :: a_force_on_tree 
      REAL, INTENT(in)  :: a_height_of_force
  
      !! 0.2 Output variables
  
      REAL, INTENT(out) :: bending_moment_sub
  
      !! 0.3 Modified variables
   
      !! 0.4 Local variables
      REAL              :: f_crown_mass 
  !_ =============================================================================================
  
  !! 1. Subroutine
  
     bending_moment_sub = a_force_on_tree * a_height_of_force * f_crown_mass
  
    END SUBROUTINE calculate_bending_moment
  
  
  
  !! =============================================================================================
  !! SUBROUTINE      : calculate_cws
  !!
  !>\BRIEF           : Subroutine to calculate critical wind speed
  !!                   (to compare with wind speeds used for calculating streamlining in the
  !!                   iterative calculation of the final critical wind speed)
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : 
  !!
  !! REFERENCES      :
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
  !_ =============================================================================================
     
    SUBROUTINE calculate_cws(critical_moment_sub, zp_displ_sub, gamma_solved_sub, &
               current_spacing_sub, crit_wind_speed_sub, distance_from_force_sub)
  
    IMPLICIT NONE
  
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
  
      REAL, INTENT(in)  :: critical_moment_sub
      REAL, INTENT(in)  :: zp_displ_sub
      REAL, INTENT(in)  :: gamma_solved_sub
      REAL, INTENT(in)  :: current_spacing_sub
      REAL, INTENT(in)  :: distance_from_force_sub
      ! f_crown_mass and rho are parameters.
  
      !! 0.2 Output variables
  
      REAL, INTENT(out) :: crit_wind_speed_sub
  
      !! 0.3 Modified variables
  
      !! 0.4 Local variables
      REAL, PARAMETER :: air_density = 1.2250    
      REAL, PARAMETER :: f_crown_mass = 1.136
      REAL, PARAMETER :: f_knots = 1.000
 !_ =============================================================================================
  
  !! 1. Subroutine
  
     crit_wind_speed_sub = (gamma_solved_sub / current_spacing_sub) * (critical_moment_sub / &
     (f_crown_mass * air_density * (zp_displ_sub - distance_from_force_sub) ))**0.5
  
     END SUBROUTINE calculate_cws
  
  
  
 
  !! =============================================================================================
  !! SUBROUTINE      : elevate
  !!
  !>\BRIEF           : This subroutine converts the critical wind speeds calculated above to the
  !!                   wind speed 10 meters above the zero-plane displacement
  !! 
  !! DESCRIPTION	   : None
  !!
  !! RECENT CHANGE(S): None
  !!
  !! RETURN VALUE    : 
  !!
  !! REFERENCES      :
  !!
  !! FLOWCHART       : None
  !! \streamlining_n 
    SUBROUTINE elevate(uh_speed, the_surf_rough, the_zp_displ, the_mean_height,elevate_result_sub)
  
    IMPLICIT NONE
  
  !! 0. Variable and parameter declaration
  
      !! 0.1 Input variables
  
      REAL,  INTENT(in)  :: uh_speed
      REAL,  INTENT(in)  :: the_surf_rough
      REAL,  INTENT(in)  :: the_zp_displ
      REAL,  INTENT(in)  :: the_mean_height
      
      !! 0.2 Output variables
  
      REAL, INTENT(out)  :: elevate_result_sub
  
      !! 0.3 Modified variables
  
      !! 0.4 Local variables
  
  !_ ==============================================================================================
  
  !! 1. Subroutine
  
     elevate_result_sub = uh_speed * log(10.0 / the_surf_rough) / &
               log((the_mean_height - the_zp_displ) / the_surf_rough)
  
    END SUBROUTINE elevate



END MODULE stomate_windthrow
